import {
  ChangeAdminPassword,
  ChangeAdminPasswordVariables,
} from "./__generated__/ChangeAdminPassword";
import {
  ChangeUserPassword,
  ChangeUserPasswordVariables,
} from "./__generated__/ChangeUserPassword";
import { ExecutionResult, useMutation } from "react-apollo";
import { FORM_ERROR } from "final-form";
import { Field, Form } from "react-final-form";
import { Modal } from "../../ui/molecules/Modal";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import { Severity, actions } from "../../notifications.slice";
import { TextInput } from "../../ui/atoms/Input";
import { UserFormRow } from "../../ui/atoms/UserForm";
import { required } from "../../util/validations";
import { useDispatch } from "react-redux";
import ErrorMessage from "../../ui/molecules/ErrorMessage";
import React, { FC, useCallback } from "react";
import gql from "graphql-tag";
import styled from "@emotion/styled";

type FormValues = { password: string };

type Props = {
  admin?: boolean;
  userId: string;
  onCancel: () => void;
  onSubmit: () => void;
};

const ChangePasswordModal: FC<Props> = ({
  admin,
  userId,
  onCancel,
  onSubmit,
}) => {
  const [changeUserPassword] = useMutation<
    ChangeUserPassword,
    ChangeUserPasswordVariables
  >(CHANGE_USER_PASSWORD_MUTATION);

  const [changeAdminPassword] = useMutation<
    ChangeAdminPassword,
    ChangeAdminPasswordVariables
  >(CHANGE_ADMIN_PASSWORD_MUTATION);
  const dispatch = useDispatch();

  const handleSubmit = useCallback(
    (values: FormValues) => {
      const validateResponse = (res: ExecutionResult<unknown>) => {
        const errors = res.errors;
        if ((errors?.length ?? 0) > 0) {
          return { [FORM_ERROR]: errors?.map((e) => e.message).join(", ") };
        } else {
          dispatch(
            actions.showNotification({
              message: "Password has been changed successfully",
              severity: Severity.Success,
            })
          );
          return onSubmit();
        }
      };
      if (admin) {
        return changeAdminPassword({
          variables: { id: userId, new_password: values.password },
        }).then(validateResponse);
      } else {
        return changeUserPassword({
          variables: { id: userId, new_password: values.password },
        }).then(validateResponse);
      }
    },
    [admin, changeAdminPassword, changeUserPassword, userId, dispatch, onSubmit]
  );
  return (
    <Modal onCancel={onCancel} title="Change password">
      <Form
        onSubmit={handleSubmit}
        render={({
          handleSubmit,
          pristine,
          hasValidationErrors,
          submitting,
        }) => (
          <form onSubmit={handleSubmit}>
            <UserFormRow>
              <label>New password</label>
              <Field<string>
                component={TextInput}
                name="password"
                validate={required("Value is required")}
              />
              <Field component={ErrorMessage} name="password" />
            </UserFormRow>

            <UserFormAction>
              <SecondaryButton onClick={onCancel} type="button">
                Cancel
              </SecondaryButton>
              <PrimaryButton
                disabled={hasValidationErrors || pristine || submitting}
              >
                Save
              </PrimaryButton>
            </UserFormAction>
          </form>
        )}
      />
    </Modal>
  );
};

export default ChangePasswordModal;

const UserFormAction = styled.div`
  display: flex;
  flex-direction: row;
  justify-content: flex-end;
  width: 100%;

  > button {
    margin-left: 20px;
  }
`;

const CHANGE_USER_PASSWORD_MUTATION = gql`
  mutation ChangeUserPassword($id: String!, $new_password: String!) {
    admin {
      updateUser(account: { id: $id }, new_password: $new_password) {
        id
      }
    }
  }
`;

const CHANGE_ADMIN_PASSWORD_MUTATION = gql`
  mutation ChangeAdminPassword($id: String!, $new_password: String!) {
    admin {
      updateAdmin(account: { id: $id }, new_password: $new_password) {
        id
      }
    }
  }
`;
