import {
  SecondaryButton as Button,
  PrimaryButton,
} from "../../ui/atoms/Button";
import { Field, Form } from "react-final-form";
import { FormApi } from "final-form";
import { Modal } from "../../ui/molecules/Modal";
import {
  NotesAddNote,
  NotesAddNoteVariables,
} from "./graphql/mutations/__generated__/NotesAddNote";
import {
  NotesGetNotes,
  NotesGetNotesVariables,
} from "./graphql/queries/__generated__/NotesGetNotes";
import { TextareaFieldTouched } from "../../ui/molecules/TextareaField";
import { required } from "../../util/validations";
import { sortByDate } from "./utils";
import { useMutation, useQuery } from "react-apollo";
import ADD_NOTE from "./graphql/mutations/AddNote.graphql";
import GET_NOTES from "./graphql/queries/NotesGetNotes.graphql";
import Note from "./Note";
import React, { ComponentProps, FC, useState } from "react";
import Spinner from "../../ui/atoms/Spinner";
import cn from "classnames";
import css from "./index.module.css";

interface FormProps {
  note: string;
}

type Props = {
  propertyId: string;
  title: string;
  buttonClassName?: string;
};

const Notes: FC<Props & ComponentProps<"div">> = ({
  title,
  propertyId,
  buttonClassName,
}) => {
  const [modalVisible, setModalVisibility] = useState(false);

  const { data, loading, error, refetch } = useQuery<
    NotesGetNotes,
    NotesGetNotesVariables
  >(GET_NOTES, {
    variables: {
      propertyId,
    },
    skip: !propertyId,
  });

  const [addNote, { loading: addLoading, error: addError }] = useMutation<
    NotesAddNote,
    NotesAddNoteVariables
  >(ADD_NOTE);

  const handleSubmit = (
    values: FormProps,
    form: FormApi<FormProps, Partial<FormProps>>
  ) => {
    const myId = window.localStorage.getItem("adminId");

    if (!myId) {
      return;
    }

    addNote({
      variables: {
        user: myId,
        note: values.note,
        propertyId,
      },
    }).then((res) => {
      refetch();
      form.reset();
      form.resetFieldState("note");
    });
  };

  const notes = data?.admin?.allNotes?.notes ?? [];

  return (
    <>
      <Button
        className={cn(css.secondaryButton, buttonClassName)}
        onClick={() => setModalVisibility(true)}
        type="button"
      >
        Notes
        {notes.length > 0 ? (
          <div className={css.hasNotes}>{notes.length}</div>
        ) : null}
      </Button>

      {modalVisible && (
        <Modal onCancel={() => setModalVisibility(false)} title={title}>
          <>
            <div className={css.list}>
              {loading && <Spinner />}
              {[...notes].sort(sortByDate).map((item) => (
                <Note key={item.updated} note={item} />
              ))}
            </div>
            {error && <div>{error.message}</div>}
            {addError && <div>{addError.message}</div>}
            <Form<FormProps>
              onSubmit={handleSubmit}
              render={({ hasValidationErrors, form }) => (
                <>
                  <Field<string>
                    component={TextareaFieldTouched}
                    name="note"
                    rows={3}
                    validate={required()}
                  />
                  <div className={css.action}>
                    <PrimaryButton
                      disabled={hasValidationErrors || addLoading}
                      onClick={() => form.submit()}
                      type="button"
                    >
                      Add note
                    </PrimaryButton>
                  </div>
                </>
              )}
            />
          </>
        </Modal>
      )}
    </>
  );
};

export default Notes;
