import { FETCH_USER_NOTIFICATIONS } from "../../entities/platform-users/UserDetails/Notifications";
import {
  FetchUserNotifications,
  FetchUserNotificationsVariables,
} from "../../entities/platform-users/UserDetails/__generated__/FetchUserNotifications";
import { NavLink, withRouter } from "react-router-dom";
import {
  common_api_generic_query_QueryFilterOperatorEnum,
  notes_api_StatusEnum,
} from "../../globalTypes";
import { useQuery } from "react-apollo";
import Badge from "../../ui/atoms/Badge";
import Collapse from "../../ui/molecules/Collapse";
import React, { FC, useCallback, useMemo, useState } from "react";
import styled from "@emotion/styled";
import theme from "../../ui/theme";

const TitleIcon = ({ state }: { state: boolean }) => (
  <>
    {state ? (
      <IconUp>
        <i className="material-icons">expand_less</i>
      </IconUp>
    ) : (
      <IconDown>
        <i className="material-icons">expand_more</i>
      </IconDown>
    )}
  </>
);

const Sidebar = (props) => {
  const { pathname } = props.location;
  const isSectionOpened = (paths: string[]): boolean =>
    !!paths.find((pth) => ~pathname.indexOf(pth));

  const { data, error } = useQuery<
    FetchUserNotifications,
    FetchUserNotificationsVariables
  >(FETCH_USER_NOTIFICATIONS, {
    variables: {
      pagination: { page: 1, perPage: 1 },
      filter: [
        {
          fields: ["status"],
          value: [notes_api_StatusEnum.STATUS_ACTIVE],
          operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
        },
      ],
    },
  });

  const [popupOpen, setPopupOpen] = useState<boolean>(false);

  const numberOfActiveReminders = useMemo(() => data?.admin?.allNotes?.total, [
    data,
  ]);

  const handleOpenTooltip = useCallback(() => {
    if (error) {
      setPopupOpen(true);
    }
  }, [error]);
  const handleCloseTooltip = useCallback(() => {
    if (error || popupOpen) {
      setPopupOpen(false);
    }
  }, [error, popupOpen]);

  return (
    <SideBar>
      <SidebarTitle>My Dashboard</SidebarTitle>
      <Collapse opened={isSectionOpened(["/admin", "/users"])}>
        {({ isOpened, toggle }) => (
          <Group>
            <GroupTitle onClick={toggle}>
              <Title data-testid="sidemenu_users_administration_tab">
                Users administration
              </Title>
              <TitleIcon state={isOpened} />
            </GroupTitle>
            {isOpened && (
              <List>
                <ListItem>
                  <NavLink data-testid="platform_admins" to="/admin">
                    Admin Users
                  </NavLink>
                </ListItem>
                <ListItem>
                  <NavLink data-testid="platform_users" to="/users">
                    Platform Users
                  </NavLink>
                </ListItem>
                <ListItem>
                  <NavLink data-testid="platform_users" to="/re-engagement">
                    Re-engagement
                    <Badge
                      onMouseEnter={handleOpenTooltip}
                      onMouseLeave={handleCloseTooltip}
                      status="danger"
                      style={{ marginLeft: 10, position: "relative" }}
                    >
                      {error ? "?" : numberOfActiveReminders}
                      {error && popupOpen && (
                        <ErrorPopup>{error.message}</ErrorPopup>
                      )}
                    </Badge>
                  </NavLink>
                </ListItem>
                <ListItem>
                  <NavLink data-testid="system-notifications" to="/notifications">
                    System Notifications
                  </NavLink>
                </ListItem>
              </List>
            )}
          </Group>
        )}
      </Collapse>

      <Collapse
        opened={isSectionOpened([
          "/listing",
          "/offers",
          "/tenancies",
          "/payments",
          "/maintenances",
          "/invitations",
          "/overdue-payments",
          "/payouts-made",
          "/unmatched-transactions",
        ])}
      >
        {({ isOpened, toggle }) => (
          <Group>
            <GroupTitle onClick={toggle}>
              <Title data-testid="sidemenu_property_administration_tab">
                Property administration
              </Title>
              <TitleIcon state={isOpened} />
            </GroupTitle>
            {isOpened && (
              <List>
                <ListItem>
                  <NavLink data-testid="sidemenu_view_listings" to="/listing">
                    View Listing
                  </NavLink>
                </ListItem>
                <ListItem>
                  <NavLink data-testid="sidemenu_view_offers" to="/offers">
                    View Offers
                  </NavLink>
                </ListItem>
                <ListItem>
                  <NavLink
                    data-testid="sidemenu_view_tenancies"
                    to="/tenancies"
                  >
                    View Tenancies
                  </NavLink>
                </ListItem>
                <ListItem>
                  <NavLink data-testid="sidemenu_view_invoices" to="/invoices">
                    View Invoices
                  </NavLink>
                </ListItem>
                <ListItem>
                  <NavLink data-testid="sidemenu_view_payments" to="/payments">
                    View Payments
                  </NavLink>
                </ListItem>
                <ListItem>
                  <NavLink
                    data-testid="sidemenu_view_account_statement"
                    to="/statement"
                  >
                    Account statement
                  </NavLink>
                </ListItem>
                <ListItem>
                  <NavLink
                    data-testid="sidemenu_view_maintenance"
                    to="/maintenances"
                  >
                    Maintenance Requests
                  </NavLink>
                </ListItem>
                <ListItem>
                  <NavLink
                    data-testid="sidemenu_view_maintenance"
                    to="/invitations"
                  >
                    View Invitations
                  </NavLink>
                </ListItem>
                <ListItem>
                  <NavLink
                    data-testid="sidemenu_view_maintenance"
                    to="/unmatched-transactions"
                  >
                    Unmatched transactions
                  </NavLink>
                </ListItem>
                <ListItem>
                  <NavLink
                    data-testid="sidemenu_view_maintenance"
                    to="/overdue-payments"
                  >
                    Overdue payments
                  </NavLink>
                  <NavLink
                    data-testid="sidemenu_view_maintenance"
                    to="/payouts-made"
                  >
                    Payouts to be made
                  </NavLink>
                </ListItem>
              </List>
            )}
          </Group>
        )}
      </Collapse>
      <Collapse opened={isSectionOpened(["/products", "/advanced"])}>
        {({ isOpened, toggle }) => (
          <Group>
            <GroupTitle onClick={toggle}>
              <Title data-testid="sidemenu_products_tab">Products</Title>
              <TitleIcon state={isOpened} />
            </GroupTitle>
            {isOpened && (
              <List>
                <ListItem>
                  <NavLink
                    data-testid="sidemenu_view_all_products"
                    to="/products"
                  >
                    View all products
                  </NavLink>
                </ListItem>
                <ListItem>
                  <NavLink
                    data-testid="sidemenu_view_advanced_rent"
                    to="/advanced"
                  >
                    Advanced rent requests
                  </NavLink>
                </ListItem>
              </List>
            )}
          </Group>
        )}
      </Collapse>
      <Collapse opened={isSectionOpened(["/events-property", "/events-user"])}>
        {({ isOpened, toggle }) => (
          <Group>
            <GroupTitle onClick={toggle}>
              <Title data-testid="sidemenu_reports_tab">Reports</Title>
              <TitleIcon state={isOpened} />
            </GroupTitle>
            {isOpened && (
              <List>
                <ListItem>
                  <NavLink
                    data-testid="sidemenu_view_property_events_log"
                    to="/events-property"
                  >
                    View Property Event logs
                  </NavLink>
                </ListItem>
                <ListItem>
                  <NavLink
                    data-testid="sidemenu_view_user_events_log"
                    to="/events-user"
                  >
                    View User Event logs
                  </NavLink>
                </ListItem>
              </List>
            )}
          </Group>
        )}
      </Collapse>
      <Collapse opened={isSectionOpened(["/mortgages-lead-list"])}>
        {({ isOpened, toggle }) => (
          <Group>
            <GroupTitle onClick={toggle}>
              <Title data-testid="sidemenu_mortgages_tab">Mortgages</Title>
              <TitleIcon state={isOpened} />
            </GroupTitle>
            {isOpened && (
              <List>
                <ListItem>
                  <NavLink
                    data-testid="sidemenu_leadlist"
                    to="/mortgages-lead-list"
                  >
                    Lead list
                  </NavLink>
                </ListItem>
              </List>
            )}
          </Group>
        )}
      </Collapse>
      <Collapse opened={isSectionOpened(["/comments"])}>
        {({ isOpened, toggle }) => (
          <Group>
            <GroupTitle onClick={toggle}>
              <Title data-testid="sidemenu_users_administration_tab">
                Blog administration
              </Title>
              <TitleIcon state={isOpened} />
            </GroupTitle>
            {isOpened && (
              <List>
                <ListItem>
                  <NavLink data-testid="blog_comments" to="/comments">
                    Comments overview
                  </NavLink>
                </ListItem>
              </List>
            )}
          </Group>
        )}
      </Collapse>
    </SideBar>
  );
};

const ErrorPopup: FC = ({ children }) => (
  <ErrorPopupWrapper>
    <div style={{ fontWeight: "bold", marginBottom: 10 }}>
      An error occurred:
    </div>
    <div>{children}</div>
  </ErrorPopupWrapper>
);

const ErrorPopupWrapper = styled.div`
  padding: 12px;
  background: #fff;
  color: black;
  position: absolute;
  left: -150px;
  top: 30px;
  width: 250px;
  word-break: break-all;
  white-space: normal;
`;

const SideBar = styled.aside`
  position: fixed;
  left: 0;
  top: 60px;
  width: 280px;
  background: ${theme.colors.main.darkGrey};
  bottom: 0;
  overflow-y: auto;
  overflow-x: hidden;
  padding: 25px 0;
`;
const SidebarTitle = styled.h3`
  color: white;
  margin: 0 20px 25px;
  text-align: center;
`;

const Title = styled.span`
  flex: 1;
  user-select: none;
  letter-spacing: 0.025em;
`;
const IconUp = styled.span`
  display: inline-block;
  transform: translateX(8px);
  width: 24px;
  height: 24px;
`;
const IconDown = styled.span`
  display: inline-block;
  transform: translateX(8px);
  width: 24px;
  height: 24px;
`;
const Group = styled.div`
  margin-bottom: 1px;
`;
const GroupTitle = styled.h4`
  display: flex;
  align-items: center;
  justify-content: space-between;
  cursor: pointer;
  padding: 12px 20px 12px 30px;
  color: white;
  background: ${theme.colors.main.mainGrey};
  margin: 0;
  width: 100%;
  font-size: 14px;
`;
const List = styled.ul`
  list-style: none;
  margin: 0;
  padding: 10px 20px 10px 40px;

  a {
    color: ${theme.colors.main.mainBlue};
    text-decoration: none;
    padding: 8px 0;
    display: flex;
    align-items: center;
    font-weight: 500;
    transition: all 0.25s;

    &:before {
      content: "";
      display: inline-block;
      width: 0;
      align-self: center;
      border-top: solid 1px;
      margin-right: 0;
      transition: all 0.25s;
    }

    &.active,
    &.active:hover {
      color: white;
      cursor: default;

      &:before {
        width: 12px;
        margin-right: 8px;
      }
    }

    &:hover {
      color: ${theme.colors.main.lightBlue};
    }
  }
`;
const ListItem = styled.li`
  margin: 4px 0;
`;

export default withRouter(Sidebar);
