import { DEFAULT_FILTER } from "./constants";
import { Field, Form } from "react-final-form";
import {
  FilterSort,
  FilterSortItem,
  FilterSortItemInner,
} from "../../ui/atoms/FilterSort";
import { FormApi } from "final-form";
import {
  GetAdminRecords,
  GetAdminRecordsVariables,
  GetAdminRecords_admin_records_records,
} from "./graphql/__generated__/GetAdminRecords";
import { ListHead } from "./ListHead";
import { ListItem } from "./ListItem";
import {
  common_api_generic_query_QueryFilterOperatorEnum as OperatorEnum,
  common_api_generic_query_QueryParamsFilterInput as ParamsFilterInput,
} from "../../globalTypes";
import { Option } from "../../util/dropdown";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import { SearchField } from "../../ui/atoms/SearchField";
import { Table as TableOrigin, Tbody, Td } from "../../ui/atoms/Table";
import { TextInput } from "../../ui/atoms/Input";
import {
  addMonths,
  endOfDay,
  endOfMonth,
  format,
  startOfDay,
  startOfMonth,
} from "date-fns";
import { usePagination } from "../../ui/molecules/Pagination";
import { useQuery } from "react-apollo";
import { useSortable } from "../../ui/molecules/Sortable";
import Dropdown from "../../ui/molecules/Dropdown";
import GET_RECORDS from "./graphql/records.graphql";
import Pagebar from "../../ui/molecules/Pagebar";
import React, { FC, useCallback, useMemo, useState } from "react";
import SystemError from "../../ui/atoms/SystemError";
import TableOverlay from "../../ui/atoms/TableOverlay";
import TableWrapper from "../../ui/atoms/TableWrapper";
import getSortOrder from "../../util/ger_sort_order";
import styled from "@emotion/styled";

type FilterFormValues = {
  search: string;
  dateFrom: string;
  dateTo: string;
  status: Option;
};

const PER_PAGE = 30;
const SEARCH_FIELD_NAMES = ["property_id", "user_id"];
export const FilterStatusOptions = [
  {
    label: "All",
    value: "0",
  },
  {
    label: "Requested",
    value: "1",
  },
  {
    label: "Granted",
    value: "2",
  },
  {
    label: "Cancelled",
    value: "3",
  },
  {
    label: "Denied",
    value: "4",
  },
];

const AdvancedRentPage: FC = () => {
  const { field, order, setSort } = useSortable({
    field: ["date"],
    order: -1,
  });
  const [filter, setFilter] = useState<ParamsFilterInput[]>(DEFAULT_FILTER);
  const { page, setPage, perPage } = usePagination({
    perPage: PER_PAGE,
    filter,
    curPage: 1,
  });

  const { data, loading, error } = useQuery<
    GetAdminRecords,
    GetAdminRecordsVariables
  >(GET_RECORDS, {
    variables: {
      params: {
        filter,
        pagination: {
          page,
          per_page: perPage,
        },
      },
    },
  });

  const handlePageChange = useCallback(
    (page: number) => {
      setPage(page);
    },
    [setPage]
  );

  const handleSubmit = useCallback(
    (values: FilterFormValues) => {
      const isChanged =
        format(startOfDay(new Date()), "yyyy-MM-dd") !==
          format(startOfDay(new Date(values?.dateFrom ?? 0)), "yyyy-MM-dd") ||
        format(endOfDay(new Date()), "yyyy-MM-dd") !==
          format(
            endOfDay(new Date(values?.dateTo ?? "2100-01-01")),
            "yyyy-MM-dd"
          );

      if (Number(values.status.value) || isChanged) {
        setFilter([
          ...filter.filter(
            (f) =>
              f.fields?.includes("type") || f.fields?.includes("income_type")
          ),
          ...(Boolean(values.search)
            ? [
                {
                  fields: SEARCH_FIELD_NAMES,
                  value: [values.search],
                  operator: OperatorEnum.CONTAINS_ANY,
                },
              ]
            : []),
          ...(Boolean(+values.status.value)
            ? [
                {
                  operator: OperatorEnum.EQUALS,
                  fields: ["value.advance_status"],
                  value: [values.status.value],
                },
              ]
            : []),
          {
            operator: OperatorEnum.MORE,
            fields: ["time"],
            value: [
              format(startOfDay(new Date(values?.dateFrom ?? 0)), "yyyy-MM-dd"),
            ],
          },
          {
            operator: OperatorEnum.LESS,
            fields: ["time"],
            value: [
              format(
                endOfDay(new Date(values?.dateTo ?? "2100-01-01")),
                "yyyy-MM-dd"
              ),
            ],
          },
        ]);
      } else {
        setFilter(DEFAULT_FILTER);
      }
    },
    [filter]
  );

  const handleFilterReset = (
    form: FormApi<FilterFormValues, Partial<FilterFormValues>>
  ) => {
    setFilter(DEFAULT_FILTER);
    form.reset();
  };

  const initialValues = useMemo(
    () => ({
      search: "",
      dateFrom: format(startOfMonth(new Date()), "yyyy-MM-dd"),
      dateTo: format(endOfMonth(addMonths(new Date(), 3)), "yyyy-MM-dd"),
      status: {
        label: "All",
        value: "0",
      },
    }),
    []
  );

  const getOrder = useCallback(
    (fieldName) => getSortOrder({ field, fieldName, order }),
    [field, order]
  );

  const records = useMemo(() => data?.admin?.records?.records ?? [], [data]);

  const groupRecordsByPropertyId = useMemo(() => {
    const hashPropertyIds = new Set();
    let output: Array<{
      feeValue: number;
      monthCount: number;
      propertyId: string;
      record: GetAdminRecords_admin_records_records;
      totalAmount: number;
    }> = [];

    for (const record of records) {
      if (hashPropertyIds.has(record.property_id)) {
        output = output.map((i) => {
          if (
            i.propertyId === record.property_id &&
            Date.parse(record.time) < Date.parse(i.record.time)
          ) {
            return {
              feeValue:
                i.feeValue + (record.type.income?.advance?.fee_value ?? 0),
              monthCount: i.monthCount + 1,
              propertyId: i.propertyId,
              record,
              totalAmount: i.totalAmount + record.amount,
            };
          }
          return i;
        });
      } else {
        hashPropertyIds.add(record.property_id);
        output.push({
          feeValue: record.type.income?.advance?.fee_value ?? 0,
          monthCount: 1,
          propertyId: record.property_id,
          record,
          totalAmount: record.amount,
        });
      }
    }

    return output;
  }, [records]);

  return (
    <>
      <h2>Advanced Rent Requests</h2>

      <Form<FilterFormValues>
        initialValues={initialValues}
        onSubmit={handleSubmit}
      >
        {({ form, handleSubmit, values, submitting }) => {
          return (
            <form onSubmit={handleSubmit}>
              <SearchField>
                <Field
                  component={TextInput}
                  name="search"
                  placeholder="Enter property ID or landlord user ID"
                />
                <PrimaryButton disabled={submitting} type="submit">
                  Search
                </PrimaryButton>
                {Boolean(values.search) || Boolean(values.status?.value) ? (
                  <SecondaryButton
                    onClick={() => handleFilterReset(form)}
                    type="reset"
                  >
                    Clear
                  </SecondaryButton>
                ) : null}
              </SearchField>
              <FilterSort>
                <FilterSortItem>
                  <label htmlFor="dateFrom">Date from:</label>
                  <Field
                    max={values.dateTo}
                    name="dateFrom"
                    render={(props) => (
                      <TextInput className="input-sort-date" {...props} />
                    )}
                    type="date"
                  />
                </FilterSortItem>
                <FilterSortItem>
                  <label htmlFor="dateTo">Date to:</label>
                  <Field
                    min={values.dateFrom}
                    name="dateTo"
                    render={(props) => (
                      <TextInput className="input-sort-date" {...props} />
                    )}
                    type="date"
                  />
                </FilterSortItem>
                <FilterSortItem>
                  <label htmlFor="status">Status:</label>
                  <Field
                    name="status"
                    render={({ input }) => {
                      return (
                        <Dropdown input={input} options={FilterStatusOptions} />
                      );
                    }}
                  />
                </FilterSortItem>
                <FilterSortItem>
                  <PrimaryButton
                    disabled={submitting}
                    onClick={() => form.submit()}
                  >
                    Filter
                  </PrimaryButton>
                  {Boolean(values.search) || Boolean(values.status?.value) ? (
                    <FilterSortItemInner>
                      <SecondaryButton
                        onClick={() => handleFilterReset(form)}
                        type="reset"
                      >
                        Clear
                      </SecondaryButton>
                    </FilterSortItemInner>
                  ) : null}
                </FilterSortItem>
              </FilterSort>
            </form>
          );
        }}
      </Form>

      {error ? <SystemError error={error} /> : null}

      <TableWrapper>
        {loading && <TableOverlay />}

        <Pagebar
          changePage={handlePageChange}
          curPage={page}
          showNPages={3}
          total={Math.ceil((data?.admin?.records?.total ?? 0) / perPage)}
        />
        <Table>
          <ListHead
            field={field}
            getOrder={getOrder}
            order={order}
            setSort={setSort}
          />
          <Tbody>
            {groupRecordsByPropertyId.length > 0 ? (
              groupRecordsByPropertyId.map((r) => (
                <ListItem
                  key={r.record.id}
                  feeValue={r.feeValue}
                  monthCount={r.monthCount}
                  record={r.record}
                  totalAmount={r.totalAmount}
                />
              ))
            ) : (
              <tr>
                <Td colSpan={18}>
                  <NoFound>No records was found</NoFound>
                </Td>
              </tr>
            )}
          </Tbody>
        </Table>

        <Pagebar
          changePage={handlePageChange}
          curPage={page}
          showNPages={3}
          total={Math.ceil((data?.admin?.records?.total ?? 0) / perPage)}
        />
      </TableWrapper>
    </>
  );
};

const Table = styled(TableOrigin)`
  table-layout: fixed;
`;

const NoFound = styled.div`
  text-align: center;
  font-size: 18px;
  font-weight: 600;
  padding: 20px;
`;

export default AdvancedRentPage;
