import { ApolloClient } from "apollo-client";
import { ApolloConsumer } from "react-apollo";
import {
  AuthFetchAdmin,
  AuthFetchAdminVariables,
} from "./graphql/queries/__generated__/AuthFetchAdmin";
import { NormalizedCacheObject } from "apollo-cache-inmemory";
import AUTH_FETCH_ADMIN from "./graphql/queries/admin.graphql";
import React from "react";
import Spinner from "../../ui/atoms/Spinner";

interface ContextProps {
  loggedIn: boolean;
  logIn: () => void;
  logOut: () => void;
}

export const AuthContext = React.createContext<Partial<ContextProps>>({
  loggedIn: false,
  logIn: () => void 0,
  logOut: () => void 0,
});

interface Props {
  client: ApolloClient<NormalizedCacheObject>;
}

interface State {
  loggedIn: boolean;
  loading: boolean;
  logIn: () => void;
  logOut: () => void;
}

class AuthProviderImpl extends React.Component<Props, State> {
  constructor(props: Props) {
    super(props);

    this.state = {
      loggedIn: false,
      loading: true,
      logIn: () => {
        this.setState(() => ({
          loggedIn: true,
        }));
      },
      logOut: () => {
        this.setState(() => ({
          loggedIn: false,
        }));
      },
    };
  }

  componentDidMount() {
    this.fetchAuth();
    window.addEventListener("focus", this.fetchAuth);
  }

  componentWillUnmount() {
    window.removeEventListener("focus", this.fetchAuth);
  }

  fetchAuth = async () => {
    try {
      this.setState({ loading: true });
      const response = await this.props.client.query<
        AuthFetchAdmin,
        AuthFetchAdminVariables
      >({
        query: AUTH_FETCH_ADMIN,
        fetchPolicy: "network-only",
        variables: {
          id: window.localStorage.getItem("adminId"),
        },
      });

      if (response.errors) {
        this.state.logOut();
      } else {
        const adminId = response.data?.admin?.Admin?.id;
        if (adminId && !this.state.loggedIn) {
          this.state.logIn();
        }
      }
    } catch (error) {
      this.state.logOut();
    } finally {
      this.setState({ loading: false });
    }
  };

  render() {
    const { children } = this.props;

    return (
      <AuthContext.Provider value={this.state}>
        {this.state.loading && !this.state.loggedIn ? <Spinner /> : children}
      </AuthContext.Provider>
    );
  }
}

export const AuthProvider = (props) => (
  <ApolloConsumer>
    {(client) => <AuthProviderImpl client={client} {...props} />}
  </ApolloConsumer>
);

export default AuthProvider;
