import {
  AllComments,
  AllCommentsVariables,
} from "./graphql/__generated__/AllComments";
import {
  DeleteComments,
  DeleteCommentsVariables,
} from "./graphql/__generated__/DeleteComments";
import { Form } from "react-final-form";
import {
  common_api_generic_query_QueryParamsFilterInput as ParamsFilterInput,
  comments_api_StatusEnum,
  common_api_generic_query_QueryFilterOperatorEnum,
} from "../../globalTypes";
import { Table, Tbody, Thead } from "../../ui/atoms/Table";
import { useMutation, useQuery } from "react-apollo";
import { usePagination } from "../../ui/molecules/Pagination";
import { useSortable } from "../../ui/molecules/Sortable";
import ALL_COMMENTS from "./graphql/allComments.graphql";
import FilterSearch from "../../ui/molecules/FilterSearch";
import ListHead from "./ListHead";
import ListItem from "./ListItem";
import Pagebar from "../../ui/molecules/Pagebar";
import React, { FC, useCallback, useState } from "react";
import SystemError from "../../ui/atoms/SystemError";
import TableOverlay from "../../ui/atoms/TableOverlay";
import TableWrapper from "../../ui/atoms/TableWrapper";
import UPDATE_COMMENT from "./graphql/updateComment.graphql";

type CommentsFilterState = ParamsFilterInput[];

const defaultFilter: ParamsFilterInput = {
  fields: ["status"],
  operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
  value: [comments_api_StatusEnum.STATUS_ACTIVE],
};

const List: FC = () => {
  const [filter, setFilter] = useState<ParamsFilterInput[]>([defaultFilter]);

  const [deleteComment] = useMutation<DeleteComments, DeleteCommentsVariables>(
    UPDATE_COMMENT,
    {
      refetchQueries: ["AllComments"],
    }
  );

  const handleFilterSubmit = useCallback((values) => {
    const newFilter: CommentsFilterState = [];

    if (values.search) {
      newFilter.push({
        fields: ["text", "correlation_id"],
        value: [values.search],
        operator: common_api_generic_query_QueryFilterOperatorEnum.CONTAINS,
      });

      setFilter(newFilter.concat(defaultFilter));
    }
  }, []);

  const handleDelete = useCallback(
    (id: string) => {
      void deleteComment({
        variables: {
          id,
          status: comments_api_StatusEnum.STATUS_DISABLED,
        },
      });
    },
    [deleteComment]
  );

  const { field, order, setSort } = useSortable({
    field: ["created"],
    order: -1,
  });

  const { page, setPage, perPage } = usePagination({
    filter,
  });

  const getOrder = useCallback(
    (fieldName): 1 | -1 => {
      return field.includes(fieldName) ? (order === 1 ? -1 : 1) : 1;
    },
    [field, order]
  );

  const { data, loading, error } = useQuery<AllComments, AllCommentsVariables>(
    ALL_COMMENTS,
    {
      variables: {
        filter,
        pagination: { page, perPage },
        sort: field.map((v) => ({ field: v, order })),
      },
    }
  );

  if (error) {
    return <SystemError error={error} />;
  }

  return (
    <>
      <h2>Blog Comments</h2>

      <Form
        onSubmit={handleFilterSubmit}
        render={({ handleSubmit, values, form }) => (
          <form onSubmit={handleSubmit}>
            <FilterSearch
              handleReset={() => {
                form.reset();
                handleFilterSubmit({});
              }}
              name="search"
              placeholder="Part of comment message or Post ID"
              values={values}
            />
          </form>
        )}
      />

      <div>
        <TableWrapper>
          {loading && <TableOverlay />}
          <Table>
            <Thead>
              <ListHead
                field={field}
                getOrder={getOrder}
                order={order}
                setSort={setSort}
              />
            </Thead>
            <Tbody>
              {(data?.admin?.allComments?.comments ?? []).map((comment) => {
                if (!comment) {
                  return null;
                }
                return (
                  <ListItem
                    key={comment.id}
                    commentId={comment.id}
                    date={comment.created}
                    onDelete={handleDelete}
                    postId={comment.correlation_id}
                    text={comment.text}
                    user={comment.user}
                  />
                );
              })}
            </Tbody>
          </Table>

          <Pagebar
            changePage={setPage}
            curPage={page}
            showNPages={5}
            total={Math.ceil((data?.admin?.allComments?.total ?? 1) / perPage)}
          />
        </TableWrapper>
      </div>
    </>
  );
};

export default List;
