import {
  DeleteDocument,
  DeleteDocumentVariables,
} from "./graphql/mutations/__generated__/DeleteDocument";
import { Field } from "react-final-form";
import { GetProperty_admin_Property_documents } from "./graphql/queries/__generated__/GetProperty";
import { Modal, ModalConfirm } from "../../ui/molecules/Modal";
import { TdInner, TdInnerWidth30, TrInner } from "../../ui/atoms/Table";
import { documentTypeFilter } from "./filters";
import { document_api_DocumentStateEnum } from "../../globalTypes";
import { enumToList, formatGenericEnumValue } from "../../util/enums";
import { formatDate } from "../../util/date";
import { isImageMimeType } from "../../util/helpers";
import { useMutation } from "react-apollo";
import DELETE_DOCUMENT from "./graphql/mutations/DeleteDocument.graphql";
import DocumentEditModal from "./DocumentEditModal";
import Dropdown from "../../ui/molecules/Dropdown";
import LinkWithPreviewFile from "../../ui/atoms/LinkWithPreviewFile";
import React, { FC, useCallback, useState } from "react";
import Spinner from "../../ui/atoms/Spinner";
import VerifyContext from "./PropertyEditContext";
import styled from "@emotion/styled";
import theme from "../../ui/theme";

type Props = {
  document: GetProperty_admin_Property_documents;
  index: number;
  propertyId: string;
  landlordId: string;
};

const DocumentsRow: FC<Props> = ({
  document,
  index,
  propertyId,
  landlordId,
}) => {
  const [showEditModal, setShowEditModal] = useState(false);
  const [showConfirm, setConfirm] = useState<boolean>(false);
  const [showAlert, setAlert] = useState<string | null>(null);
  const [deleteDocument, { loading }] = useMutation<
    DeleteDocument,
    DeleteDocumentVariables
  >(DELETE_DOCUMENT);

  const handleDelete = useCallback(() => {
    deleteDocument({
      variables: {
        id: document.id,
      },
      awaitRefetchQueries: true,
      refetchQueries: ["GetAllDocuments"],
    }).then((res) => {
      setConfirm(false);

      if (res.data?.admin?.deleteDocument?.success) {
        setAlert("Document was deleted successfully!");
      } else {
        const errorMessage =
          (res.errors ?? []).length > 0
            ? res.errors?.map((err) => err.message).join(", ")
            : null;

        setAlert(
          errorMessage != null
            ? errorMessage
            : "Something was wrong, try again later"
        );
      }
    });
  }, [deleteDocument, document.id]);

  const fileName =
    document.file.name.length > 28 ? (
      <>
        {document.file.name.substring(0, 20)}
        &nbsp;&hellip;&nbsp;
        {document.file.name.substr(-8)}
      </>
    ) : (
      document.file.name
    );

  return (
    <>
      {showConfirm && (
        <ModalConfirm
          onCancel={() => {
            setConfirm(false);
          }}
          onConfirm={() => {
            handleDelete();
          }}
        >
          <span>
            Are you sure you really <br />
            want to delete document <br />
            <b>{fileName}</b>?
          </span>
        </ModalConfirm>
      )}
      {showAlert && (
        <Modal
          onCancel={() => {
            setAlert(null);
          }}
        >
          <b>{showAlert}</b>
        </Modal>
      )}
      <VerifyContext.Consumer>
        {({ documentsVerified, submitting }) => (
          <TrInner>
            <TdInnerWidth30>
              <LinkWithPreviewFile
                href={`/media/${document.file.id}`}
                rel="noopener noreferrer"
                target="_blank"
                title={document.file.name}
              >
                {isImageMimeType(document.file.mime) && (
                  <img alt="" src={`/media/thumb-${document.file.id}`} />
                )}
                <span>{fileName}</span>
              </LinkWithPreviewFile>
            </TdInnerWidth30>
            <TdInner>
              <b>{documentTypeFilter(document.type, document.code)}</b>
            </TdInner>
            <TdInner>{document.description}</TdInner>
            <TdInner>{formatDate(document.expiration)}</TdInner>
            <TdInner>
              {documentsVerified && document ? (
                formatGenericEnumValue(document.state)
              ) : (
                <Field
                  name={`documents[${index}].state`}
                  render={({ input }) => {
                    return (
                      <Dropdown
                        disabled={documentsVerified || submitting}
                        input={input}
                        itemToString={formatGenericEnumValue}
                        options={enumToList(document_api_DocumentStateEnum)}
                      />
                    );
                  }}
                />
              )}
            </TdInner>
            <TdInner>
              {!document.removed && (
                <ButtonEdit
                  onClick={() => setShowEditModal(true)}
                  type="button"
                >
                  Edit
                </ButtonEdit>
              )}
            </TdInner>
            <TdInner>
              {document.removed ? (
                <b>Deleted</b>
              ) : (
                <ButtonEdit onClick={() => setConfirm(true)} type="button">
                  Delete
                  {loading ? (
                    <>
                      &nbsp;&nbsp;
                      <Spinner size="sm" />
                    </>
                  ) : null}
                </ButtonEdit>
              )}
            </TdInner>
          </TrInner>
        )}
      </VerifyContext.Consumer>

      {showEditModal && (
        <DocumentEditModal
          document={document}
          landlordId={landlordId}
          onClose={() => setShowEditModal(false)}
          propertyId={propertyId}
        />
      )}
    </>
  );
};

export default DocumentsRow;

const ButtonEdit = styled.button`
  font-weight: 700;
  font-size: 1em;
  color: ${theme.colors.icons.darkBlue};
  background-color: transparent;
  border: none;
  cursor: pointer;
`;
