import {
  ConnectDragSource,
  ConnectDropTarget,
  DragSource,
  DragSourceConnector,
  DragSourceMonitor,
  DropTarget,
  DropTargetConnector,
  DropTargetMonitor,
} from "react-dnd";
import { XYCoord } from "dnd-core";
import { css } from "@emotion/core";
import React, { ReactNode, useImperativeHandle, useRef } from "react";
import styled from "@emotion/styled";

export interface DraggableFileProps {
  id: string;
  index: number;
  onMove: (dragIndex: number, hoverIndex: number) => void;
  isDragging: boolean;
  connectDragSource: ConnectDragSource;
  connectDropTarget: ConnectDropTarget;
  children: ReactNode;
}

interface DraggableFileInstance {
  getNode(): HTMLDivElement | null;
}

const DraggableFile = React.forwardRef<HTMLDivElement, DraggableFileProps>(
  ({ children, isDragging, connectDragSource, connectDropTarget }, ref) => {
    const elementRef = useRef(null);
    connectDragSource(elementRef);
    connectDropTarget(elementRef);

    useImperativeHandle<{}, DraggableFileInstance>(ref, () => ({
      getNode: () => elementRef.current,
    }));

    return (
      <Container ref={elementRef} isDragging={isDragging}>
        {children}
      </Container>
    );
  }
);

export default DropTarget(
  "file",
  {
    hover(
      props: DraggableFileProps,
      monitor: DropTargetMonitor,
      component: DraggableFileInstance
    ) {
      if (!component) {
        return null;
      }
      // node = HTML Div element from imperative API
      const node = component.getNode();
      if (!node) {
        return null;
      }

      const dragIndex = monitor.getItem().index;
      const hoverIndex = props.index;

      // Don't replace items with themselves
      if (dragIndex === hoverIndex) {
        return null;
      }

      // Determine rectangle on screen
      const hoverBoundingRect = node.getBoundingClientRect();

      // Get vertical middle
      const hoverMiddleY =
        (hoverBoundingRect.bottom - hoverBoundingRect.top) / 2;

      // Determine mouse position
      const clientOffset = monitor.getClientOffset();

      // Get pixels to the left
      const hoverClientY = (clientOffset as XYCoord).y - hoverBoundingRect.top;

      // Only perform the move when the mouse has crossed half of the items height
      // When dragging downwards, only move when the cursor is below 50%
      // When dragging upwards, only move when the cursor is above 50%

      // Dragging downwards
      if (dragIndex < hoverIndex && hoverClientY < hoverMiddleY) {
        return null;
      }

      // Dragging upwards
      if (dragIndex > hoverIndex && hoverClientY > hoverMiddleY) {
        return null;
      }

      // Time to actually perform the action
      props.onMove(dragIndex, hoverIndex);

      // Note: we're mutating the monitor item here!
      // Generally it's better to avoid mutations,
      // but it's good here for the sake of performance
      // to avoid expensive index searches.
      monitor.getItem().index = hoverIndex;

      return null;
    },
  },
  (connect: DropTargetConnector) => ({
    connectDropTarget: connect.dropTarget(),
  })
)(
  DragSource(
    "file",
    {
      beginDrag: (props: DraggableFileProps) => ({
        id: props.id,
        index: props.index,
      }),
    },
    (connect: DragSourceConnector, monitor: DragSourceMonitor) => ({
      connectDragSource: connect.dragSource(),
      isDragging: monitor.isDragging(),
    })
  )(DraggableFile)
);

type ContainerProps = {
  isDragging: boolean;
};
const Container = styled.tr<ContainerProps>`
  cursor: grab;

  &:nth-of-type(even) td,
  &:nth-of-type(even) th {
    background: #f5fcfe;
  }

  &:hover {
    td {
      background: #e5f8ff;
    }
  }

  ${({ isDragging }) =>
    isDragging &&
    css`
      opacity: 0.3;
    `}
`;
