import {
  ButtonsGroup,
  UserFormItem,
  UserFormRow,
  UserFormRowFlex,
} from "../../../ui/atoms/UserForm";
import { DocumentTypeOptions } from "../constants";
import { Field, useForm, useFormState } from "react-final-form";
import { FileField } from "../../../ui/molecules/FileField/FileField";
import { FormValues } from "./FormDocumentAdd";
import { InputFieldTouched } from "../../../ui/molecules/InputField";
import { PrimaryButton, SecondaryButton } from "../../../ui/atoms/Button";
import { Products_admin_products_product } from "../graphql/queries/__generated__/Products";
import { document_api_DocumentStateEnum } from "../../../globalTypes";
import { enumToList, formatGenericEnumValue } from "../../../util/enums";
import { required } from "../../../util/validations";
import Dropdown from "../../../ui/molecules/Dropdown";
import ErrorMessage from "../../../ui/molecules/ErrorMessage";
import React, { FC, useMemo } from "react";

interface Props {
  products: readonly Products_admin_products_product[] | null | undefined;
  onCancel: () => void;
}

export const DocumentForm: FC<Props> = ({ products, onCancel }) => {
  const { submit: handleSubmit } = useForm<FormValues>();
  const { submitting, valid } = useFormState<FormValues>();

  const options = useMemo(
    () =>
      (products ?? []).map((el) => ({
        value: el,
        label: `Product ID: ${el.id}; ${el.service?.service.title}`,
      })),
    [products]
  );

  return (
    <form onSubmit={handleSubmit}>
      <Field
        name="file"
        render={({ input, meta }) => {
          return (
            <FileField
              acceptedTypes={acceptedFileTypes}
              error={meta.touched && meta.error}
              fileId={input.value}
              handleChange={input.onChange}
            />
          );
        }}
        validate={required()}
      />
      <UserFormRowFlex>
        <UserFormItem>
          <label htmlFor="type">Document type</label>
          <Field
            name="type"
            render={({ input, meta }) => (
              <Dropdown
                error={meta.touched && meta.error}
                input={input}
                options={DocumentTypeOptions}
                placeholder="-- select --"
              />
            )}
            validate={required("Required field")}
          />
          <Field component={ErrorMessage} name="type" />
        </UserFormItem>
        <UserFormItem>
          <label htmlFor="state">Document state</label>
          <Field
            name="state"
            render={({ input, meta }) => (
              <Dropdown
                error={meta.touched && meta.error}
                input={input}
                itemToString={formatGenericEnumValue}
                options={enumToList(document_api_DocumentStateEnum)}
                placeholder="-- select --"
              />
            )}
            validate={required("Required field")}
          />
          <Field component={ErrorMessage} name="state" />
        </UserFormItem>
        <UserFormItem>
          <label htmlFor="expiration">Expire date</label>
          <Field
            name="expiration"
            render={({ input, meta }) => (
              <InputFieldTouched input={input} meta={meta} />
            )}
            type="date"
            validate={required("Required field")}
          />
        </UserFormItem>
      </UserFormRowFlex>
      <UserFormRow>
        <label htmlFor="linkToPurchase">Link to purchase</label>
        <Field
          name="linkToPurchase"
          render={({ input, meta }) => (
            <Dropdown
              error={meta.touched && meta.error}
              input={input}
              inputStyle={{ width: "100%" }}
              options={options}
              style={{ width: "65%" }}
            />
          )}
        />
      </UserFormRow>
      <UserFormRow>
        <label htmlFor="description">Description</label>
        <Field component={InputFieldTouched} name="description" />
      </UserFormRow>
      <ButtonsGroup>
        <SecondaryButton onClick={onCancel} type="reset">
          Cancel
        </SecondaryButton>
        <PrimaryButton disabled={!valid || submitting}>Add</PrimaryButton>
      </ButtonsGroup>
    </form>
  );
};

const acceptedFileTypes = [
  "application/pdf",
  "image/gif",
  "image/jpeg",
  "image/png",
];
