import {
  AccountInfo,
  AccountInfoVariables,
} from "./graphql/queries/__generated__/AccountInfo";
import {
  AllAdmins,
  AllAdminsVariables,
} from "../admin/graphql/queries/__generated__/AllAdmins";
import { ApiError } from "../../ui/atoms/ApiError/ApiError";
import { Field, Form } from "react-final-form";
import {
  FilterLastUpdateOptions,
  FilterStatusOptions,
  ListingInitialState,
  listingActions,
} from "./constants";
import {
  FilterSort,
  FilterSortItem,
  FilterSortItemInner,
} from "../../ui/atoms/FilterSort";
import {
  common_api_generic_query_QueryFilterOperatorEnum as OperatorEnum,
  common_api_generic_query_QueryParamsFilterInput as ParamsFilterInput,
  common_api_generic_query_QueryParamsFilterInput,
  property_api_PropertyStatusEnum,
} from "../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import {
  PropertiesList,
  PropertiesListVariables,
  PropertiesList_admin_allProperties_data as Property,
} from "./graphql/queries/__generated__/PropertiesList";
import { Query, useQuery } from "react-apollo";
import { RouteComponentProps } from "react-router";
import { SearchField } from "../../ui/atoms/SearchField";
import { Table as TableOrigin, Tbody } from "../../ui/atoms/Table";
import { TextInput } from "../../ui/atoms/Input";
import { makeFilters } from "../../util/search_filters";
import { oc } from "ts-optchain";
import ACCOUNT_INFO from "./graphql/queries/AccountInfo.graphql";
import ALL_ADMINS from "../admin/graphql/queries/AllAdmins.graphql";
import ALL_PROPERTIES from "./graphql/queries/GetListOfProperties.graphql";
import Checkbox from "../../ui/molecules/Checkbox";
import Dropdown from "../../ui/molecules/Dropdown";
import ListHead from "./ListHead";
import ListItem from "./ListItem";
import Pagebar from "../../ui/molecules/Pagebar";
import React, { FC, useEffect, useMemo, useState } from "react";
import SystemError from "../../ui/atoms/SystemError";
import TableOverlay from "../../ui/atoms/TableOverlay";
import TableWrapper from "../../ui/atoms/TableWrapper";
import getSortOrder from "../../util/ger_sort_order";
import styled from "@emotion/styled";

export type ListingAction = {
  type: string;
  value: string | number | boolean;
};
export type ListProps = {
  state: ListingInitialState;
  dispatch(arg1: ListingAction[]): void;
  props: RouteComponentProps;
};

const getFilters = (values) => {
  const statusFilter: Array<common_api_generic_query_QueryParamsFilterInput> = [];
  if (values.status?.value === property_api_PropertyStatusEnum.AMENDMENT) {
    statusFilter.push(
      {
        fields: ["status"],
        value: ["VERSION_VERIFYING"],
        operator: OperatorEnum.EQUALS,
      },
      {
        fields: ["last_verified"],
        value: ["0"],
        operator: OperatorEnum.EQUALS,
        not: true,
      },
      {
        fields: ["latest"],
        value: ["true"],
        operator: OperatorEnum.EQUALS,
      }
    );
  } else if (values.status?.value === property_api_PropertyStatusEnum.MISSING) {
    statusFilter.push(
      {
        fields: ["status"],
        value: ["VERSION_VERIFYING"],
        operator: OperatorEnum.EQUALS,
      },
      {
        fields: ["validation.verifiable"],
        value: ["true"],
        operator: OperatorEnum.EQUALS,
      },
      {
        fields: ["validation.valid"],
        value: ["false"],
        operator: OperatorEnum.EQUALS,
      },
      {
        fields: ["latest"],
        value: ["true"],
        operator: OperatorEnum.EQUALS,
      },
      {
        fields: ["last_verified"],
        value: ["0"],
        operator: OperatorEnum.EQUALS,
      }
    );
  } else if (values.status && values.status.value !== "") {
    statusFilter.push(
      {
        fields: ["status"],
        value: [values.status?.value],
        operator: OperatorEnum.EQUALS,
      },
      {
        fields: ["latest"],
        value: ["true"],
        operator: OperatorEnum.EQUALS,
      }
    );
  }
  return [
    ...makeFilters({
      values,
      filters: [
        {
          fields: ["id", "ivr"],
          key: "search",
          operator: OperatorEnum.CONTAINS,
        },
        {
          fields: ["location.postcode"],
          key: "postcode",
          operator: OperatorEnum.CONTAINS,
        },
        {
          fields: ["updated_at"],
          key: "updated_at.value",
          operator: OperatorEnum.EQUALS,
        },
        {
          fields: ["people.administrator"],
          key: "admin_name.value",
          operator: OperatorEnum.EQUALS,
        },
        {
          fields: ["is_archived"],
          key: "is_archived",
          operator: OperatorEnum.EQUALS,
        },
      ],
    }),
    ...statusFilter,
  ];
};

const withDefaultFilters = (filters: ParamsFilterInput[]) => {
  const result: any = [...filters];

  const is_archived = filters.filter((filter) =>
    filter.fields?.includes("is_archived")
  )[0];

  if (!is_archived) {
    result.push({
      operator: OperatorEnum.EQUALS,
      fields: ["is_archived"],
      value: "false",
    });
  }

  return result;
};

interface ADMIN_NAME {
  label?: string | null;
  value?: string | null;
}

const List: FC<ListProps> = ({ state, dispatch }) => {
  const [filter, setFilter] = useState<ParamsFilterInput[]>(getFilters(state));
  const { data, error } = useQuery<AllAdmins, AllAdminsVariables>(ALL_ADMINS, {
    variables: {
      pagination: {
        page: 1,
        perPage: 100,
      },
    },
  });

  const adminId = useMemo(() => window.localStorage.getItem("adminId"), []);

  const { data: accountInfo, error: accountInfoError } = useQuery<
    AccountInfo,
    AccountInfoVariables
  >(ACCOUNT_INFO, { variables: { id: adminId } });

  useEffect(() => {
    if (accountInfo?.admin?.Admin?.test) {
      setFilter(getFilters({ ...state, is_archived: true }));
    }
  }, [accountInfo, state]);

  const adminNames: ADMIN_NAME[] | undefined = useMemo(
    () =>
      (data?.admin?.allAdmins?.data ?? [])
        .filter((item) => !item.block)
        .map((item) => ({
          label: `${item.details?.first_name} ${item.details?.last_name}`,
          value: item.id,
        })),
    [data]
  );

  const handleFilterSubmit = (values) => {
    const filters = getFilters(values);
    const resultState: Array<{
      type: string;
      value: string | number | boolean;
    }> = [
      { type: listingActions.SET_CURRENT_PAGE, value: 1 },
      { type: listingActions.SET_IS_ARCHIVED, value: false },
    ];

    Object.keys(values).forEach((valueKey) => {
      let type: string = "";
      const value: string = values[valueKey];

      if (valueKey === "status") {
        type = listingActions.SET_STATUS;
      } else if (valueKey === "updated_at") {
        type = listingActions.SET_UPDATED_AT;
      } else if (valueKey === "admin_name") {
        type = listingActions.SET_ADMIN_NAME;
      } else if (valueKey === "search") {
        type = listingActions.SET_SEARCH;
      } else if (valueKey === "postcode") {
        type = listingActions.SET_POSTCODE;
      } else if (valueKey === "is_archived") {
        type = listingActions.SET_IS_ARCHIVED;
      }

      if (type) {
        resultState.push({ type, value });
      }
    });
    setFilter(filters);
    dispatch(resultState);
  };

  const handleFilterReset = (cb: () => void) => {
    cb();
    dispatch([
      { type: listingActions.SET_STATUS, value: "" },
      { type: listingActions.SET_UPDATED_AT, value: "" },
      { type: listingActions.SET_ADMIN_NAME, value: "" },
      { type: listingActions.SET_SEARCH, value: "" },
      { type: listingActions.SET_POSTCODE, value: "" },
      { type: listingActions.SET_CURRENT_PAGE, value: 1 },
      { type: listingActions.SET_IS_ARCHIVED, value: false },
    ]);
    setFilter([]);
  };

  const getOrder = (fieldName) =>
    getSortOrder({ field: state.sortField, fieldName, order: state.sortOrder });

  const isFiltersExist = Object.keys(filter).length > 0;

  return (
    <>
      <h2>View Listing</h2>
      <ApiError errors={[error, accountInfoError]} />
      <Form<Partial<ListingInitialState>>
        initialValues={{
          search: state.search,
          postcode: state.postcode,
          status: state.status,
          updated_at: state.updated_at,
          is_archived: state.is_archived,
          admin_name: state.admin_name,
        }}
        onSubmit={handleFilterSubmit}
        render={({
          handleSubmit,
          values,
          valid,
          submitting,
          form,
          pristine,
        }) => (
          <form onSubmit={handleSubmit}>
            <SearchField>
              <Field
                component={TextInput}
                name="search"
                placeholder="Search by 6-digit ref. number or Property ID"
              />
              {isFiltersExist ? (
                <SecondaryButton
                  onClick={() => handleFilterReset(form.reset)}
                  type="reset"
                >
                  Clear
                </SecondaryButton>
              ) : (
                <PrimaryButton
                  disabled={
                    (values.postcode && !valid) || !values.search || submitting
                  }
                  onClick={() => handleFilterSubmit(values)}
                  type="submit"
                >
                  Search
                </PrimaryButton>
              )}
            </SearchField>
            <FilterSort>
              <FilterSortItem>
                <label htmlFor="postcode">Postcode:</label>
                <Field
                  name="postcode"
                  render={(props) => <TextInput errorOnly {...props} />}
                />
              </FilterSortItem>
              <FilterSortItem>
                <label htmlFor="status">Status:</label>
                <Field
                  name="status"
                  render={({ input }) => {
                    return (
                      <Dropdown input={input} options={FilterStatusOptions} />
                    );
                  }}
                />
              </FilterSortItem>
              <FilterSortItem>
                <label htmlFor="updated_at">Last update:</label>
                <Field
                  name="updated_at"
                  render={({ input }) => {
                    return (
                      <Dropdown
                        input={input}
                        options={FilterLastUpdateOptions}
                      />
                    );
                  }}
                />
              </FilterSortItem>
              <FilterSortItem>
                <label htmlFor="admin_name">Assign to:</label>
                <Field
                  name="admin_name"
                  render={({ input }) => {
                    return <Dropdown input={input} options={adminNames} />;
                  }}
                />
              </FilterSortItem>
              <FilterSortItem hidden={Boolean(accountInfo?.admin?.Admin?.test)}>
                <label htmlFor="is_archived">Show archived:</label>
                <Field
                  id="is_archived"
                  name="is_archived"
                  render={({ input }) => <Checkbox input={input} />}
                  type="checkbox"
                />
              </FilterSortItem>
              <FilterSortItem>
                <PrimaryButton
                  disabled={(values.postcode && !valid) || submitting}
                  onClick={() => handleFilterSubmit(values)}
                >
                  Filter
                </PrimaryButton>
                {isFiltersExist && (
                  <FilterSortItemInner>
                    <SecondaryButton
                      onClick={() => handleFilterReset(form.reset)}
                      type="reset"
                    >
                      Clear
                    </SecondaryButton>
                  </FilterSortItemInner>
                )}
              </FilterSortItem>
            </FilterSort>
          </form>
        )}
      />

      <Query<PropertiesList, PropertiesListVariables>
        query={ALL_PROPERTIES}
        variables={{
          pagination: { page: state.currentPage, perPage: state.perPage },
          sort:
            state.sortField.length > 0
              ? state.sortField.reduce<Array<{ field: string; order: number }>>(
                  (acc, item) => [
                    ...acc,
                    { field: item, order: state.sortOrder },
                  ],
                  []
                )
              : undefined,
          filter: withDefaultFilters(filter),
        }}
      >
        {({ data, loading, error }) => {
          if (error) {
            return <SystemError error={error} />;
          }

          return (
            <TableWrapper>
              {loading && <TableOverlay />}

              <Pagebar
                changePage={(value) =>
                  dispatch([{ type: listingActions.SET_CURRENT_PAGE, value }])
                }
                curPage={state.currentPage}
                showNPages={3}
                total={Math.ceil(
                  oc(data).admin.allProperties.total(1) / state.perPage
                )}
              />
              <Table>
                <ListHead
                  field={state.sortField}
                  getOrder={getOrder}
                  order={state.sortOrder}
                  setSort={({ field, order }) => {
                    dispatch([
                      { type: listingActions.SET_SORT_FIELD, value: field },
                      { type: listingActions.SET_SORT_ORDER, value: order },
                    ]);
                  }}
                />
                <Tbody>
                  {oc(data)
                    .admin.allProperties.data([])
                    .map((property: Property) => {
                      const {
                        created_at,
                        listing_expiry_date,
                        location,
                        people: { landlord },
                        status,
                      } = property;

                      if (property.id != null && landlord != null) {
                        return (
                          <ListItem
                            key={property.id as string}
                            created={created_at}
                            expires={listing_expiry_date}
                            landlord={landlord}
                            location={location}
                            propertyId={property.id}
                            status={status}
                          />
                        );
                      }

                      return null;
                    })}
                </Tbody>
              </Table>

              <Pagebar
                changePage={(value) =>
                  dispatch([{ type: listingActions.SET_CURRENT_PAGE, value }])
                }
                curPage={state.currentPage}
                showNPages={3}
                total={Math.ceil(
                  oc(data).admin.allProperties.total(1) / state.perPage
                )}
              />
            </TableWrapper>
          );
        }}
      </Query>
    </>
  );
};

const Table = styled(TableOrigin)`
  table-layout: fixed;
`;

export default List;
