import { CheckboxToggle } from "../../../ui/molecules/CheckboxToggle";
import { Field, Form } from "react-final-form";
import { ModalAlert, ModalConfirm } from "../../../ui/molecules/Modal";
import {
  PropertyAsideRightmove,
  PropertyCardItemTitle,
  PropertyCardItemValue,
  PropertyContent,
  PropertyItemPublish,
  PropertyItemRightMove,
  PublicationBlocked,
} from "../../../ui/atoms/Property";
import { PropertyRightmove } from "../graphql/queries/__generated__/PropertyRightmove";
import { TitleError } from "../../../ui/atoms/TitleError";
import { TitleSuccess } from "../../../ui/atoms/TitleSuccess";
import { UserFormRow } from "../../../ui/atoms/UserForm";
import { usePublish } from "./use-publish";
import { withApollo } from "react-apollo";
import CopyText from "../../../ui/molecules/CopyToClipboard";
import FormAsideAdminUser from "../../../components/FormAsideAdminUser";
import React from "react";
import VerifyContext from "../PropertyEditContext";

type Props = {
  propertyId: string;
  client?: any;
  rightmove: PropertyRightmove;
  version_id: string;
};

const PublishRightmove = ({
  client,
  propertyId,
  rightmove,
  version_id,
}: Props) => {
  const {
    errorText,
    modalMode,
    onClearMessage,
    onSubmit,
    publishing,
    setModalMode,
    successText,
    unpublishing,
  } = usePublish(version_id, "rightmove");

  return (
    <>
      {errorText || successText ? (
        <ModalAlert
          onCancel={onClearMessage}
          title={
            errorText != null ? (
              <TitleError>Error</TitleError>
            ) : (
              <TitleSuccess>Message</TitleSuccess>
            )
          }
        >
          <b>{errorText || successText}</b>
        </ModalAlert>
      ) : null}
      <VerifyContext.Consumer>
        {({ publishedInRightmove, publishedInMashroom }) => (
          <Form
            initialValues={{
              publish_property_rightmove: publishedInRightmove,
            }}
            onSubmit={(fn) => fn}
            render={({ form }) => {
              return (
                <>
                  {modalMode === "confirm" ||
                    (publishing && (
                      <ModalConfirm
                        disabled={publishing}
                        loading={publishing}
                        onCancel={() => {
                          form.reset();
                          setModalMode(undefined);
                        }}
                        onConfirm={() => {
                          onSubmit(form, "publish");
                          setModalMode(undefined);
                        }}
                      >
                        <span>
                          Are you sure you really want to
                          <br />
                          <b>publish</b> property on rightmove?
                        </span>
                      </ModalConfirm>
                    ))}
                  {modalMode === "delete" ||
                    (unpublishing && (
                      <ModalConfirm
                        disabled={unpublishing}
                        loading={unpublishing}
                        onCancel={() => {
                          form.reset();
                          setModalMode(undefined);
                        }}
                        onConfirm={() => {
                          onSubmit(form, "unpublish");
                          setModalMode(undefined);
                        }}
                      >
                        <span>
                          Are you sure you really want to
                          <br />
                          <b>unpublish</b> property from rightmove?
                        </span>
                      </ModalConfirm>
                    ))}
                  <PropertyItemRightMove>
                    <PropertyContent>
                      <PropertyItemPublish>
                        <h3>Rightmove</h3>
                        {publishedInRightmove && (
                          <>
                            <PropertyCardItemTitle>
                              Property ID
                            </PropertyCardItemTitle>
                            <PropertyCardItemValue>
                              {rightmove.id}
                              <CopyText
                                text="Copy Rightmove ID"
                                textForCopy={rightmove.id}
                              />
                            </PropertyCardItemValue>
                            <br />
                            <PropertyCardItemTitle>
                              Property Link
                            </PropertyCardItemTitle>
                            <PropertyCardItemValue>
                              <a
                                className="link"
                                href={rightmove.url}
                                rel="noopener noreferrer"
                                target="_blank"
                                title={`Open in new tab ${rightmove.url}`}
                              >
                                {`${rightmove.url.substring(
                                  0,
                                  30
                                )} . . . ${rightmove.url.substr(-10)}`}
                              </a>
                              <CopyText
                                text="Copy link"
                                textForCopy={rightmove.url}
                              />
                            </PropertyCardItemValue>
                          </>
                        )}
                        {!publishedInRightmove && publishedInMashroom && (
                          <h4>
                            Something went wrong. Please, try again later.
                          </h4>
                        )}
                        {!publishedInRightmove && !publishedInMashroom && (
                          <h4>
                            Property will not be published on Rightmove until it
                            is published on Mashroom
                          </h4>
                        )}
                      </PropertyItemPublish>
                    </PropertyContent>
                    <PropertyAsideRightmove>
                      <UserFormRow>
                        {!publishedInRightmove && !publishedInMashroom ? (
                          <PublicationBlocked>
                            <Field
                              component={CheckboxToggle}
                              disabled={
                                publishedInRightmove ||
                                (!publishedInRightmove && !publishedInMashroom)
                              }
                              labelText="Published"
                              name="publish_property_rightmove"
                              onChange={(event) => {
                                form.change(
                                  "publish_property_rightmove",
                                  event.target.checked
                                );
                                setModalMode("confirm");
                              }}
                              type="checkbox"
                            />
                          </PublicationBlocked>
                        ) : (
                          <Field
                            component={CheckboxToggle}
                            disabled={
                              !publishedInRightmove && !publishedInMashroom
                            }
                            labelText="Published"
                            name="publish_property_rightmove"
                            onChange={(event) => {
                              form.change(
                                "publish_property_rightmove",
                                event.target.checked
                              );
                              setModalMode(
                                event.target.checked ? "confirm" : "delete"
                              );
                            }}
                            type="checkbox"
                          />
                        )}
                      </UserFormRow>
                      {publishedInRightmove && (
                        <FormAsideAdminUser
                          date={rightmove.created_at as string}
                          id={rightmove.published_by as string}
                        />
                      )}
                    </PropertyAsideRightmove>
                  </PropertyItemRightMove>
                </>
              );
            }}
          />
        )}
      </VerifyContext.Consumer>
    </>
  );
};

export default withApollo<Props>(PublishRightmove);
