import * as B from "../../../ui/atoms/Button";
import { CheckboxConfirm } from "../../../ui/atoms/CheckboxInput";
import { Field, FormSpy, useForm } from "react-final-form";
import { GetProperty_admin_Property } from "../graphql/queries/__generated__/GetProperty";
import { OutsideSpaceOptions, ParkingOptions } from "../constants";
import { PropertyBody, PropertyContent } from "../../../ui/atoms/Property";
import { TextareaField } from "../../../ui/molecules/TextareaField";
import { UserFormRow } from "../../../ui/atoms/UserForm";
import { enumToList, formatGenericEnumValue } from "../../../util/enums";
import {
  property_api_PropertyFurnitureEnum,
  property_api_PropertyStatusEnum,
} from "../../../globalTypes";
import { required } from "../../../util/validations";
import Dropdown from "../../../ui/molecules/Dropdown";
import PropertyVerification from "../PropertyVerification";
import React, { useState } from "react";
import VerifyContext from "../PropertyEditContext";
import styled from "@emotion/styled";

type Props = {
  property: GetProperty_admin_Property;
};

enum CutPropertyTypesEnum {
  PROPERTY_TYPE_STUDIO = "PROPERTY_TYPE_STUDIO",
  PROPERTY_TYPE_FLAT = "PROPERTY_TYPE_FLAT",
  PROPERTY_TYPE_DETACHED = "PROPERTY_TYPE_DETACHED",
  PROPERTY_TYPE_SEMI_DETACHED = "PROPERTY_TYPE_SEMI_DETACHED",
  PROPERTY_TYPE_TERRACED = "PROPERTY_TYPE_TERRACED",
  PROPERTY_TYPE_OTHER = "PROPERTY_TYPE_OTHER",
  PROPERTY_TYPE_MAISONETTE = "PROPERTY_TYPE_MAISONETTE",
  PROPERTY_TYPE_BUNGALOW = "PROPERTY_TYPE_BUNGALOW",
  PROPERTY_TYPE_TOWN_HOUSE = "PROPERTY_TYPE_TOWN_HOUSE",
  PROPERTY_TYPE_COTTAGE = "PROPERTY_TYPE_COTTAGE",
  PROPERTY_TYPE_APARTMENT = "PROPERTY_TYPE_APARTMENT",
  PROPERTY_TYPE_PENTHOUSE = "PROPERTY_TYPE_PENTHOUSE",
  PROPERTY_TYPE_PARK_HOME = "PROPERTY_TYPE_PARK_HOME",
}

export function parseIntlNumber(
  value: string | undefined | null,
  locale = "en-EN"
) {
  return Number(value != null ? value.replace(/[^\d.]/g, "") : 0);
}

const Description = (props: Props) => {
  const [isEditMode, setEditMode] = useState<boolean>(false);
  const form = useForm();

  return (
    <VerifyContext.Consumer>
      {({
        descriptionVerified,
        submitting,
        somethingWasChanged,
        publishedInMashroom,
      }) => {
        const readOnly = isEditMode ? false : submitting || descriptionVerified;

        return (
          <PropertyBody>
            <FormSpy
              onChange={(props) => {
                if (!props.pristine && descriptionVerified) {
                  form.change("admin_verify.description.verified", false);
                }
              }}
              subscription={{ pristine: true }}
            />

            <Content>
              <ColMain>
                <UserFormRow>
                  <label htmlFor="description.post_property_pitch">
                    Description
                  </label>
                  <Field
                    component={TextareaField}
                    name="description.post_property_pitch"
                    readOnly={readOnly}
                    rows={17}
                  />
                </UserFormRow>
                <UserFormRow>
                  <label htmlFor="description.post_property_admin_pitch">
                    Admin pitch
                  </label>
                  <Field
                    component={TextareaField}
                    name="description.post_property_admin_pitch"
                    readOnly={readOnly}
                    rows={17}
                  />
                </UserFormRow>
              </ColMain>
              <ColAside>
                <UserFormRow>
                  <label htmlFor="rental.rental_price">Rental Price</label>
                  <Field
                    format={(value) =>
                      value != null
                        ? `£${new Intl.NumberFormat("en-EN", {
                            maximumFractionDigits: 2,
                          }).format(value)}`
                        : ""
                    }
                    name="rental.rental_price"
                    parse={parseIntlNumber}
                    render={({ input, meta }) => (
                      <TextField>
                        <TextFieldHelper hasValue={!!input.value}>
                          {!!input.value ? (
                            <>
                              <span>{input.value}</span> pcm
                            </>
                          ) : null}
                        </TextFieldHelper>
                        <Input
                          {...input}
                          placeholder="£0,0 pcm"
                          readOnly={readOnly}
                        />
                      </TextField>
                    )}
                    validate={required()}
                  />
                </UserFormRow>

                <UserFormRow>
                  <label htmlFor="details.amenity_furniture">Furnished</label>

                  <Field
                    name="details.amenity_furniture"
                    render={({ input }) => {
                      return (
                        <Dropdown
                          disabled={readOnly}
                          input={input}
                          inputStyle={{
                            width: "100%",
                          }}
                          itemToString={(v) =>
                            formatGenericEnumValue(
                              v.replace("PROPERTY_FURNITURE_", "")
                            )
                          }
                          options={enumToList(
                            property_api_PropertyFurnitureEnum
                          )}
                          style={{
                            width: "100%",
                          }}
                        />
                      );
                    }}
                  />
                </UserFormRow>

                <UserFormRow>
                  <label htmlFor="details.type">Type</label>

                  <Field
                    name="details.type"
                    render={({ input }) => {
                      return (
                        <Dropdown
                          disabled={readOnly}
                          input={input}
                          inputStyle={{
                            width: "100%",
                          }}
                          itemToString={(v) =>
                            formatGenericEnumValue(
                              v.replace("PROPERTY_TYPE_", "")
                            )
                          }
                          options={enumToList(CutPropertyTypesEnum)}
                          style={{
                            width: "100%",
                          }}
                        />
                      );
                    }}
                  />
                </UserFormRow>

                <UserFormRow>
                  <label htmlFor="rental.tenancy_deposit_insurance">
                    Deposit replacement
                  </label>

                  <Field
                    name="rental.tenancy_deposit_insurance"
                    render={({ input }) => (
                      <CheckboxConfirm>
                        <input type="checkbox" {...input} disabled={readOnly} />
                        <span>{input.value ? "On" : "Off"}</span>
                      </CheckboxConfirm>
                    )}
                    type="checkbox"
                  />
                </UserFormRow>

                <UserFormRow>
                  <label htmlFor="details.amenity_outside_space">
                    Outside Space
                  </label>
                  {OutsideSpaceOptions.map(({ label, value }) => (
                    <div key={value}>
                      <Field
                        name="details.amenity_outside_space"
                        render={({ input }) => (
                          <LabelCheckbox>
                            <input {...input} disabled={readOnly} />
                            {input.checked ? (
                              <LabelCheckedText>{label}</LabelCheckedText>
                            ) : (
                              <LabelCheckboxText>{label}</LabelCheckboxText>
                            )}
                          </LabelCheckbox>
                        )}
                        type="checkbox"
                        value={value}
                      />
                    </div>
                  ))}
                </UserFormRow>

                <UserFormRow>
                  <label htmlFor="details.amenity_parking">Parking</label>
                  {ParkingOptions.map(({ label, value }) => (
                    <div key={value}>
                      <Field
                        name="details.amenity_parking"
                        render={({ input }) => (
                          <LabelCheckbox>
                            <input {...input} disabled={readOnly} />
                            {input.checked ? (
                              <LabelCheckedText>{label}</LabelCheckedText>
                            ) : (
                              <LabelCheckboxText>{label}</LabelCheckboxText>
                            )}
                          </LabelCheckbox>
                        )}
                        type="checkbox"
                        value={value}
                      />
                    </div>
                  ))}
                </UserFormRow>
              </ColAside>
            </Content>

            <PropertyVerification
              adminVerify={props.property.admin_verify}
              disabled={isEditMode && somethingWasChanged}
              isVerified={!!descriptionVerified}
              property={props.property}
              showVerifiedButtons={isEditMode && somethingWasChanged}
              verifiedLabel="Verified"
              verifiedName="admin_verify.description.verified"
              verifyKey="description"
            >
              <>
                {props.property.status ===
                  property_api_PropertyStatusEnum.LISTED &&
                (!(isEditMode && somethingWasChanged) || readOnly) ? (
                  <DeblockFieldsButton>
                    <B.PrimaryButton
                      disabled={isEditMode}
                      onClick={() => setEditMode(true)}
                    >
                      Edit Fields
                    </B.PrimaryButton>
                  </DeblockFieldsButton>
                ) : null}
              </>
            </PropertyVerification>
          </PropertyBody>
        );
      }}
    </VerifyContext.Consumer>
  );
};

const DeblockFieldsButton = styled.div`
  padding: 30px 0;
  text-align: center;
`;

const Content = styled(PropertyContent)`
  display: flex;
`;

const ColMain = styled.div`
  flex: 1;
`;

const ColAside = styled.div`
  width: 280px;
  margin-left: 30px;
`;

const TextField = styled.div`
  position: relative;
  white-space: nowrap;
  border: solid 1px #d8d8db;
  border-radius: 5px;
  background: white;

  input {
    position: relative;
    z-index: 1;
  }
`;

const TextFieldHelper = styled.div<{ hasValue: boolean }>`
  position: absolute;
  left: 0;
  top: 0;
  bottom: 0;
  font-size: 16px;
  font-weight: 500;
  line-height: 20px;
  padding: 9px 12px;
  cursor: text;
  opacity: 0.75;

  span {
    opacity: 0;
  }
`;

const Input = styled.input`
  width: 100%;
  display: block;
  padding: 9px 12px;
  border-radius: 5px;
  font-size: 16px;
  font-weight: 500;
  line-height: 20px;
  transition: all 0.25s;
  background: transparent;
  outline: 0;
  border: 0;
  font-weight: 600;
`;

const LabelCheckboxText = styled.div`
  padding: 4px 0 4px 16px;
  border-left: solid 2px #008bc1;
  color: #2c2c39;
  background: #ffffff66;
  font-size: 14px;
`;

const LabelCheckbox = styled.label`
  display: block;
  margin: 0 !important;
  position: relative;

  input {
    position: absolute;
    left: -9999px;
  }
`;

const LabelCheckedText = styled(LabelCheckboxText)`
  color: #06c500;
  font-weight: 700;
  background: #33333311;
  font-size: 14px;
`;

export default Description;
