import {
  CategoryOptions,
  ChannelOptions,
  TargetRoleOptions,
  defaultNotification,
  findElement,
} from "./constants";
import { EditNotificationDefinitionDocument } from "./graphql/mutations/EditNotificationDefinition.generated";
import { Field, Form } from "react-final-form";
import { FilterSort, FilterSortItem } from "../../ui/atoms/FilterSort";
import { FormData, Notification } from "./types";
import { ModalWide } from "../../ui/molecules/Modal";
import { NotificationDefinitionListDocument } from "./graphql/queries/NotificationDefinitionList.generated";
// usePagination below uses old types
import { common_api_generic_query_QueryParamsFilterInput as ParamsFilterInput } from "../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import { SearchFieldWide } from "../../ui/atoms/SearchField";
import { Table, Tbody, Thead } from "../../ui/atoms/Table";
import { TextInput } from "../../ui/atoms/Input";
import { makeFilters } from "../../util/search_filters";
import { useMutation, useQuery } from "react-apollo";
import { usePagination } from "../../ui/molecules/Pagination";
import { wrapInputEmptyOption } from "../../util/dropdown";
import Dropdown from "../../ui/molecules/Dropdown";
import EditForm from "./EditForm";
import ListHead from "./ListHead";
import ListItem from "./ListItem";
import Pagebar from "../../ui/molecules/Pagebar";
import React, { memo, useCallback, useMemo, useState } from "react";
import SystemError from "../../ui/atoms/SystemError";
import TableOverlay from "../../ui/atoms/TableOverlay";
import TableWrapper from "../../ui/atoms/TableWrapper";

const NotificationsPage = memo(() => {
  const [filter, setFilter] = useState<ParamsFilterInput[]>([]);

  const { page, setPage, perPage } = usePagination({ filter });

  const handleFilterSubmit = (values) => {
    setFilter(
      makeFilters({
        values,
        filters: [
          {
            fields: ["template"],
            key: "search",
            operator: "CONTAINS_ANY",
            transformation: {
              preprocessor: "REGEXP_REPLACE",
              param: ["[- /]", ""],
            },
          },
          {
            fields: ["channel"],
            key: "channel.value",
            operator: "EQUALS",
          },
          {
            fields: ["category"],
            key: "category.value",
            operator: "EQUALS",
          },
          {
            fields: ["role"],
            key: "target.value",
            operator: "EQUALS",
          },
        ],
      })
    );
  };

  const handleFilterReset = (cb) => {
    cb();
    setFilter([]);
  };

  const { data, loading, error, refetch } = useQuery(
    NotificationDefinitionListDocument,
    {
      variables: { filter, pagination: { page, per_page: perPage } },
    }
  );

  const [saveNotification] = useMutation(EditNotificationDefinitionDocument);

  // convert data element from [{key: string, value: string},...] to {key: string, ...}
  const allNotifications = (
    data?.admin?.notificationDefinitionList?.notifications ?? []
  ).map(({ data, ...n }) => ({
    ...n,
    data: Object.fromEntries(data.map(({ key, value }) => [key, value])),
  }));
  const totalRows = data?.admin?.notificationDefinitionList?.total ?? 0;

  const [isModalOpened, setIsModalOpened] = useState<boolean>(false);

  const [currentNotificationData, setCurrentNotificationData] = useState<
    Notification
  >(defaultNotification);

  const initialValues = useMemo<FormData>(() => {
    return {
      notification: {
        ...currentNotificationData,
        category: findElement(
          CategoryOptions,
          currentNotificationData.category
        ),
        channel: findElement(ChannelOptions, currentNotificationData.channel),
        target: findElement(TargetRoleOptions, currentNotificationData.target),
      },
    };
  }, [currentNotificationData]);

  const handleOpenDetailsClick = useCallback(
    (notification) => {
      setCurrentNotificationData(notification);
      setIsModalOpened(true);
    },
    [setCurrentNotificationData, setIsModalOpened]
  );

  const handleCloseModal = useCallback(() => {
    setIsModalOpened(false);
    setCurrentNotificationData(defaultNotification);
  }, []);

  const handleSave = async ({ notification }) => {
    await saveNotification({
      variables: {
        ...notification,
        category: notification.category.value,
        channel: notification.channel.value,
        target: notification.target.value,
        seconds: notification.duration.seconds,
        nanos: notification.duration.nanos,
        data: Object.entries(notification.data).map(([k, v]) => ({
          key: k,
          value: v,
        })),
      },
    });
    await refetch();
    handleCloseModal();
  };

  if (error) {
    return <SystemError error={error} />;
  }

  return (
    <>
      <Form
        onSubmit={handleFilterSubmit}
        render={({
          handleSubmit,
          submitting,
          valid,
          values,
          form,
          pristine,
        }) => (
          <form onSubmit={handleSubmit}>
            <SearchFieldWide>
              <Field
                component={TextInput}
                name="search"
                placeholder="Enter Search Message Text"
              />
              <PrimaryButton
                disabled={!valid || submitting || pristine}
                onClick={() => handleFilterSubmit(values)}
                type="submit"
              >
                Search
              </PrimaryButton>
              {!(!valid || submitting || pristine) && (
                <>
                  &nbsp;&nbsp;
                  <SecondaryButton
                    onClick={() => handleFilterReset(form.reset)}
                    type="reset"
                  >
                    Clear
                  </SecondaryButton>
                </>
              )}
            </SearchFieldWide>
            <FilterSort>
              <FilterSortItem>
                <Field
                  name="channel"
                  render={({ input }) => {
                    return (
                      <Dropdown
                        input={wrapInputEmptyOption(input)}
                        options={ChannelOptions}
                        placeholder="Channel"
                      />
                    );
                  }}
                />
              </FilterSortItem>
              <FilterSortItem>
                <Field
                  name="category"
                  render={({ input }) => {
                    return (
                      <Dropdown
                        input={wrapInputEmptyOption(input)}
                        options={CategoryOptions}
                        placeholder="Category"
                      />
                    );
                  }}
                />
              </FilterSortItem>
              <FilterSortItem>
                <Field
                  name="target"
                  render={({ input }) => {
                    return (
                      <Dropdown
                        input={wrapInputEmptyOption(input)}
                        options={TargetRoleOptions}
                        placeholder="User"
                      />
                    );
                  }}
                />
              </FilterSortItem>
            </FilterSort>
          </form>
        )}
      />

      <TableWrapper>
        {loading && <TableOverlay />}
        <Table>
          <Thead>
            <ListHead />
          </Thead>
          <Tbody>
            {allNotifications.map((n) => (
              <ListItem
                key={n.ID}
                notification={n}
                onOpenDetailsClick={handleOpenDetailsClick}
              />
            ))}
          </Tbody>
        </Table>

        <Pagebar
          changePage={setPage}
          curPage={page}
          showNPages={3}
          total={Math.ceil(totalRows / perPage)}
        />
      </TableWrapper>
      {isModalOpened && (
        <ModalWide onCancel={handleCloseModal} title="Notification" width={800}>
          <EditForm
            currentNotificationData={initialValues}
            onClose={handleCloseModal}
            onSubmit={handleSave}
          />
        </ModalWide>
      )}
    </>
  );
});

export default NotificationsPage;
