import { CheckboxConfirm } from "../../ui/atoms/CheckboxInput";
import { Field, Form } from "react-final-form";
import { GetOffer_admin_Offer } from "./graphql/queries/__generated__/GetOffer";
import { InputFieldTouched } from "../../ui/molecules/InputField";
import { ModalWide } from "../../ui/molecules/Modal";
import { PAYMENT_FREQUENCY_OPTIONS } from "./constants";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import { UserFormRow } from "../../ui/atoms/UserForm";
import { WithApolloClient, withApollo } from "react-apollo";
import { format, parse, subDays } from "date-fns";
import { oc } from "ts-optchain";
import { required } from "../../util/validations";
import Collapse from "../../ui/molecules/Collapse";
import Dropdown from "../../ui/molecules/Dropdown";
import EmailLink from "../../ui/atoms/EmailLink";
import GET_OFFER from "./graphql/queries/GetOffer.graphql";
import React from "react";
import UPDATE_OFFER from "./graphql/mutations/UpdateOffer.graphql";
import get from "lodash.get";
import strip_typenames from "../../util/strip_typenames";
import styled from "@emotion/styled";
import theme from "../../ui/theme";

type Props = {
  offer: GetOffer_admin_Offer;
  handleCancel: () => void;
};

const PrepareContractModal = ({
  client,
  offer,
  handleCancel,
}: WithApolloClient<Props>) => {
  // Collect userpaths to iterate after
  const userPaths = ["landlord"];
  (offer.tenants ?? []).forEach((tenant, idx) => {
    userPaths.push(`tenants[${idx}]`);
  });
  (offer.guarantors ?? []).forEach((g, idx) => {
    userPaths.push(`guarantors[${idx}]`);
  });

  let errorText: string = "";

  const handleSubmit = (values) => {
    const updatedOffer: any = strip_typenames(values);
    return client
      .mutate({
        mutation: UPDATE_OFFER,
        variables: {
          user: "",
          offer: {
            id: updatedOffer.id,
            landlord: updatedOffer.landlord,
            tenants: updatedOffer.tenants,
            conditions: updatedOffer.conditions,
          },
          render_contract: true,
        },
        refetchQueries: [
          {
            query: GET_OFFER,
            variables: { id: offer.id },
          },
        ],
      })
      .then((data) => {
        // eslint-disable-next-line no-console
        console.log(data);
        if (oc(data).data.admin.updateOffer.status() === "SUCCESS") {
          handleCancel();
          alert("Offer updated successfully. Contract rendered");
        } else if (data.errors && data.errors[0]) {
          errorText = data.errors[0].message;
        }
      });
  };

  const { conditions, ...restOffer } = offer;

  return (
    <ModalWide
      onCancel={handleCancel}
      title={`Contract preparation. Offer ID: ${offer.id}`}
    >
      <Form
        initialValues={{
          ...restOffer,
          conditions: {
            movein_date: conditions.movein_date,
            payment_frequency: PAYMENT_FREQUENCY_OPTIONS.find(
              (v) => v.value === offer.conditions.payment_frequency
            ),
          },
        }}
        onSubmit={handleSubmit}
        render={({ handleSubmit, submitting, valid, values }) => (
          <form onSubmit={handleSubmit}>
            <SectionTitle>Addresses</SectionTitle>
            {userPaths.map((userPath, idx) => {
              const {
                first_name,
                last_name,
                email_primary,
                phone_primary,
              }: { [key: string]: string | null } = get(offer, userPath);
              const firstName = (first_name ?? "").trim();
              const lastName = (last_name ?? "").trim();

              return (
                <Collapse
                  key={`${firstName}${email_primary}`}
                  opened={idx === 0}
                >
                  {({ isOpened, toggle }) => (
                    <Group>
                      <GroupTitle>
                        <GroupTitleIcon onClick={toggle}>
                          <TitleIcon state={isOpened} />
                        </GroupTitleIcon>
                        <GroupTitleText>
                          <b>
                            {firstName} {lastName}
                          </b>
                          {email_primary && (
                            <span>
                              <EmailLink href={`mailto:${email_primary}`}>
                                {email_primary}
                              </EmailLink>
                            </span>
                          )}
                          {phone_primary && <span>{phone_primary}</span>}
                        </GroupTitleText>
                      </GroupTitle>

                      {isOpened && (
                        <GroupBody>
                          <UserFormRow>
                            <label htmlFor={`${userPath}.billing_address`}>
                              Address
                            </label>
                            <Field
                              component={InputFieldTouched}
                              name={`${userPath}.billing_address`}
                            />
                          </UserFormRow>
                        </GroupBody>
                      )}
                    </Group>
                  )}
                </Collapse>
              );
            })}

            <SectionTitle>Clauses</SectionTitle>
            <Group>
              <GroupBody style={{ border: "none", margin: 0 }}>
                <UserFormRow>
                  <label>Move in date</label>
                  <Field
                    component={InputFieldTouched}
                    format={(value: string) =>
                      format(new Date(value), "yyyy-MM-dd")
                    }
                    name="conditions.movein_date"
                    parse={(value: string) =>
                      parse(value, "yyyy-MM-dd", new Date()).toUTCString()
                    }
                    type="date"
                  />
                </UserFormRow>
                <UserFormRow>
                  <label>First Rent Payment Date</label>
                  <InputFieldTouched
                    disabled
                    input={{
                      onBlur: () => {},
                      onChange: () => {},
                      onFocus: () => {},
                      name: "firstPaymentDate",
                      value: format(
                        subDays(new Date(values.conditions.movein_date), 3),
                        "yyyy-MM-dd"
                      ),
                    }}
                    meta={{}}
                    type="date"
                  />
                </UserFormRow>
                <UserFormRow>
                  <label>Rent Payment Dates</label>
                  <InputFieldTouched
                    disabled
                    input={{
                      onBlur: () => {},
                      onChange: () => {},
                      onFocus: () => {},
                      name: "rentPaymentDates",
                      value: format(
                        new Date(values.conditions.movein_date),
                        "do"
                      ),
                    }}
                    meta={{}}
                  />
                </UserFormRow>
                <UserFormRow>
                  <label>Rent payment frequency</label>
                  <Field
                    name="conditions.payment_frequency"
                    render={({ input }) => (
                      <Dropdown
                        input={input}
                        itemToString={(v) => v?.label}
                        itemToValue={(v) => v?.value}
                        options={PAYMENT_FREQUENCY_OPTIONS}
                      />
                    )}
                  />
                </UserFormRow>
              </GroupBody>
            </Group>

            <div>
              <Field
                name="confirm"
                render={({ input }) => (
                  <CheckboxConfirm>
                    <input type="checkbox" {...input} />
                    <span>
                      I confirm that addresses were verified and entered
                      correctly
                    </span>
                  </CheckboxConfirm>
                )}
                type="checkbox"
                validate={required("Required")}
              />
            </div>

            {errorText !== "" && <ErrorText>{errorText}</ErrorText>}

            <Buttons>
              <SecondaryButton onClick={handleCancel} type="reset">
                Cancel
              </SecondaryButton>
              <PrimaryButton disabled={!valid || submitting}>
                Proceed
              </PrimaryButton>
            </Buttons>
          </form>
        )}
      />
    </ModalWide>
  );
};

export default withApollo<Props>(PrepareContractModal);

const TitleIcon = ({ state }: { state: boolean }) => (
  <>
    {state ? (
      <i className="material-icons">expand_less</i>
    ) : (
      <i className="material-icons">expand_more</i>
    )}
  </>
);

const SectionTitle = styled.b`
  font-size: 16px;
  margin: 12px 0 0;
`;
const Group = styled.div`
  background-color: white;
  border: 1px solid ${theme.colors.border.default};
  padding: 10px 15px;
  margin-bottom: 15px;
`;

const GroupTitle = styled.div`
  display: flex;
`;

const GroupTitleText = styled.div`
  font-size: 16px;
  padding-left: 15px;

  span,
  b {
    margin-right: 15px;
  }
`;

const GroupTitleIcon = styled.div`
  cursor: pointer;
  width: 24px;
  height: 24px;
  user-select: none;
`;

const GroupBody = styled.div`
  padding: 15px 0;
  margin-top: 10px;
  border-top: 1px solid ${theme.colors.border.default};
`;

const Buttons = styled.div`
  display: flex;
  justify-content: space-between;
  margin-top: 20px;
`;

const ErrorText = styled.div`
  color: red;
  margin: 6px 0;
`;
