import { ButtonsGroup } from "../../ui/atoms/UserForm";
import { Field, Form } from "react-final-form";
import { GetOffer_admin_Offer } from "./graphql/queries/__generated__/GetOffer";
import { ModalWide } from "../../ui/molecules/Modal";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import { RENT_PLAN_OPTIONS } from "./constants";
import {
  UpdateOffer,
  UpdateOfferVariables,
} from "./graphql/mutations/__generated__/UpdateOffer";
import {
  offer_api_OfferStatusEnum,
  offer_api_RentPlanEnum,
} from "../../globalTypes";
import { required } from "../../util/validations";
import { useMutation } from "@apollo/react-hooks";
import Dropdown from "../../ui/molecules/Dropdown";
import React, { FC, useCallback, useMemo } from "react";
import UPDATE_OFFER from "./graphql/mutations/UpdateOffer.graphql";

interface FormValues {
  rent_plan: { label: string; value: offer_api_RentPlanEnum };
}

type Props = {
  setModalVisibility: (open: boolean) => void;
  offer: GetOffer_admin_Offer;
};

const UpdateRentPlanModal: FC<Props> = ({ setModalVisibility, offer }) => {
  const [updateOffer, { error, loading }] = useMutation<
    UpdateOffer,
    UpdateOfferVariables
  >(UPDATE_OFFER);

  const handleSubmit = useCallback(
    (values: FormValues) => {
      void updateOffer({
        variables: {
          offer: {
            id: offer.id,
            rent_plan: values.rent_plan.value,
          },
        },
        refetchQueries: ["GetOffer"],
      }).then((res) => {
        if (
          res.data?.admin?.updateOffer?.status ===
          offer_api_OfferStatusEnum.SUCCESS
        ) {
          setModalVisibility(false);
        }
      });
    },
    [updateOffer, setModalVisibility, offer.id]
  );

  const initialValues: Partial<FormValues> = useMemo(
    () => ({
      rent_plan: RENT_PLAN_OPTIONS.find((r) => r.value === offer.rent_plan),
    }),
    [offer.rent_plan]
  );

  return (
    <ModalWide
      onCancel={() => setModalVisibility(false)}
      title="Change rent collection plan"
      width={450}
      withoutOverflow
    >
      <Form<FormValues> initialValues={initialValues} onSubmit={handleSubmit}>
        {({ handleSubmit, hasValidationErrors }) => (
          <form onSubmit={handleSubmit}>
            <div style={{ marginBottom: "24px" }}>
              <Field
                name="rent_plan"
                render={({ input, meta }) => (
                  <Dropdown
                    error={meta.touched && meta.error}
                    input={input}
                    options={RENT_PLAN_OPTIONS}
                    placeholder="-- select --"
                  />
                )}
                validate={required("Required field")}
              />
            </div>

            {error && <div>{error.message}</div>}

            <ButtonsGroup>
              <SecondaryButton
                onClick={() => setModalVisibility(false)}
                type="button"
              >
                Cancel
              </SecondaryButton>
              <PrimaryButton disabled={hasValidationErrors || loading}>
                Save
              </PrimaryButton>
            </ButtonsGroup>
          </form>
        )}
      </Form>
    </ModalWide>
  );
};

export default UpdateRentPlanModal;
