import {
  CancelSinging,
  CancelSingingVariables,
} from "./__generated__/CancelSinging";
import { DocumentItem, DocumentItemDate } from "../../../../ui/atoms/Offer";
import { GetOffer_admin_Offer } from "../../graphql/queries/__generated__/GetOffer";
import {
  MarkAsSigned,
  MarkAsSignedVariables,
} from "./__generated__/MarkAsSigned";
import { ModalConfirm } from "../../../../ui/molecules/Modal";
import { offer_api_OfferStateEnum as OfferStateEnum } from "../../../../globalTypes";
import { SecondaryButton, SuccessButton } from "../../../../ui/atoms/Button";
import { Table, Tbody, Thead } from "../../../../ui/atoms/Table";
import { formatDateWordTime } from "../../../../util/date";
import { useMutation } from "react-apollo";
import ClauseHead from "./ClauseHead";
import ClauseItem from "./ClauseItem";
import React, { useEffect, useState } from "react";
import RecreateContract from "./RecreateContract";
import SystemError from "../../../../ui/atoms/SystemError";
import TabContractHead from "./TabContractHead";
import TabContractItem from "./TabContractItem";
import TableWrapper from "../../../../ui/atoms/TableWrapper";
import gql from "graphql-tag";
import styled from "@emotion/styled";

type Props = { offer: GetOffer_admin_Offer; refetchOffer: () => void };

const TabContract: React.FC<Props> = ({ offer, refetchOffer }) => {
  const people = [...(offer.tenants ?? []), ...(offer.guarantors ?? [])];

  const clauses = offer.clause ?? [];

  const document =
    offer.state === OfferStateEnum.CONTRACT_REVIEW_AND_CLAUSES
      ? offer.draft_contract
      : offer.contract;
  const documentDate =
    offer.state === OfferStateEnum.CONTRACT_REVIEW_AND_CLAUSES
      ? offer.draft_contract_created_at
      : offer.contract_created_at;

  let timer: number | null = null;
  useEffect(() => {
    return () => {
      timer && clearTimeout(timer);
    };
  }, [timer]);

  const [showConfirmMark, setShowConfirmMark] = useState(false);
  const [markAsSigned, { error, loading: markAsSignedLoading }] = useMutation<
    MarkAsSigned,
    MarkAsSignedVariables
  >(MARK_AS_SIGNED);

  const handleConfirmMarkClick = () => {
    markAsSigned({
      variables: { offerId: offer.id, propertyId: offer.property.id },
    }).then((res) => {
      if (res.data?.admin?.updateSignature?.success) {
        setShowConfirmMark(false);
        timer = window.setTimeout(() => {
          refetchOffer();
        }, 1500);
      }
    });
  };

  const [showConfirmCancel, setShowConfirmCancel] = useState(false);
  const [
    cancelSigning,
    { error: cancelError, loading: cancelLoading },
  ] = useMutation<CancelSinging, CancelSingingVariables>(CANCEL_SIGNING);

  const handleConfirmCancelClick = () => {
    cancelSigning({
      variables: { offer: offer.id },
    }).then((res) => {
      if (res.data?.admin?.cancelSigningOffer?.success) {
        setShowConfirmCancel(false);
        refetchOffer();
      }
    });
  };

  return (
    <>
      {offer.state === OfferStateEnum.CONTRACT_REVIEW_AND_CLAUSES ? (
        <RecreateContract offerId={offer.id} />
      ) : null}
      <ContractWrapper>
        {document !== "" && (
          <DocumentItem>
            <div>
              <a
                download={`Contract_Offer_${offer.id.slice(-5)}`}
                href={`/media/${document}`}
              >
                {offer.state === OfferStateEnum.CONTRACT_REVIEW_AND_CLAUSES ? (
                  <span>Download draft contract</span>
                ) : (
                  <span>Download contract</span>
                )}
                <i className="material-icons">file_download</i>
              </a>
            </div>

            {documentDate !== "" && (
              <DocumentItemDate>
                {formatDateWordTime(documentDate)}
              </DocumentItemDate>
            )}
          </DocumentItem>
        )}
        {offer.state === OfferStateEnum.SIGNING && (
          <>
            <SuccessButton
              disabled={markAsSignedLoading}
              onClick={() => setShowConfirmMark(true)}
            >
              Mark As Signed
            </SuccessButton>
            <SecondaryButton
              disabled={cancelLoading}
              onClick={() => setShowConfirmCancel(true)}
            >
              Cancel signing
            </SecondaryButton>
          </>
        )}
      </ContractWrapper>

      <h3>Tenants</h3>
      <ContentWrapper>
        <TableWrapper>
          <Table>
            <Thead>
              <TabContractHead />
            </Thead>
            <Tbody>
              {people
                .filter((t) => t?.first_name != null)
                .map((t) => (
                  <TabContractItem key={t!.email_primary ?? ""} tenant={t!} />
                ))}
            </Tbody>
          </Table>
        </TableWrapper>
      </ContentWrapper>

      <h3>Clauses</h3>
      <ContentWrapper>
        <TableWrapper>
          <Table>
            <Thead>
              <ClauseHead />
            </Thead>
            <Tbody>
              {clauses.map((c, idx) => (
                <ClauseItem key={c.id} clause={c} idx={idx} />
              ))}
            </Tbody>
          </Table>
        </TableWrapper>
      </ContentWrapper>

      {showConfirmMark && (
        <ModalConfirm
          disabled={markAsSignedLoading}
          onCancel={() => setShowConfirmMark(false)}
          onConfirm={handleConfirmMarkClick}
        >
          Are you sure you want to mark the contract as signed?
          {error != null ? <SystemError error={error.message} /> : null}
        </ModalConfirm>
      )}

      {showConfirmCancel && (
        <ModalConfirm
          disabled={cancelLoading}
          onCancel={() => setShowConfirmCancel(false)}
          onConfirm={handleConfirmCancelClick}
        >
          Are you sure you want to cancel the contract signing?
          {cancelError != null ? (
            <SystemError error={cancelError.message} />
          ) : null}
        </ModalConfirm>
      )}
    </>
  );
};

export default TabContract;

const ContentWrapper = styled.div`
  display: flex;
`;

const ContractWrapper = styled.div`
  display: flex;
  align-items: center;
  margin-bottom: 30px;

  & button {
    margin-left: 20px;
  }
`;

const MARK_AS_SIGNED = gql`
  mutation MarkAsSigned(
    $offerId: String!
    $propertyId: String!
    $event_type: String = "signature_request_all_signed"
    $is_complete: Boolean = true
  ) {
    admin {
      updateSignature(
        metadata: { offer_id: $offerId, property_id: $propertyId }
        event_type: $event_type
        is_complete: $is_complete
      ) {
        success
      }
    }
  }
`;

const CANCEL_SIGNING = gql`
  mutation CancelSinging($offer: String) {
    admin {
      cancelSigningOffer(offer: $offer) {
        errors
        success
      }
    }
  }
`;
