import { CheckboxToggle } from "../../../../ui/molecules/CheckboxToggle";
import {
  DepositSelect,
  DepositSelectVariables,
} from "./graphql/query/__generated__/DepositSelect";
import { Field, Form } from "react-final-form";
import { GetOffer_admin_Offer } from "../../graphql/queries/__generated__/GetOffer";
import { Link } from "react-router-dom";
import {
  invoice_api_PaymentStatusEnum as PaymentStatusEnum,
  extras_api_ServiceRoleEnum,
  extras_api_ServiceTypeEnum,
  offer_api_DepositTypeEnum,
} from "../../../../globalTypes";
import {
  PaymentsGetInvoice,
  PaymentsGetInvoiceVariables,
} from "../../../payments/graphql/queries/__generated__/PaymentsGetInvoice";
import {
  PropertyAside,
  PropertyCardItemTitle,
  PropertyCardItemValue,
  PropertyGrid,
  PropertyVerifiedBlock,
} from "../../../../ui/atoms/Property";
import {
  UpdatePayment,
  UpdatePaymentVariables,
} from "../../graphql/mutations/__generated__/UpdatePayment";
import { UserFormRow } from "../../../../ui/atoms/UserForm";
import { css } from "@emotion/core";
import { formatPrice } from "../../../../util/price";
import { sentenceCase } from "change-case";
import { useMutation, useQuery } from "react-apollo";
import DEPOSIT_SELECT from "./graphql/query/DepositSelect.graphql";
import GET_INVOICE from "../../../payments/graphql/queries/PaymentsGetInvoice.graphql";
import Invoice from "./molecules/Invoice";
import Payment from "./molecules/Payment";
import React, { FC, Fragment, useMemo, useState } from "react";
import SystemError from "../../../../ui/atoms/SystemError";
import UPDATE_PAYMENT_STATUS from "../../graphql/mutations/UpdatePayment.graphql";
import getInvoicePaymentStatus from "../../../../util/payments";
import styled from "@emotion/styled";
import theme from "../../../../ui/theme";

type Props = { offer: GetOffer_admin_Offer; refetchOffer: () => void };

enum TABS {
  FIRST_MONTH_RENT = "First Month Rent",
  RENT_PAYMENTS = "Rent Payments",
  DIRECT_DEBIT = "Direct Debit",
}

const TabRentPayment: FC<Props> = ({ offer }) => {
  const [tab, setTab] = useState<TABS>(TABS.FIRST_MONTH_RENT);

  const depositReplacement =
    offer?.conditions.deposit_type ===
    offer_api_DepositTypeEnum.DEPOSIT_REPLACEMENT;

  const [confirmPayment, { loading, error }] = useMutation<
    UpdatePayment,
    UpdatePaymentVariables
  >(UPDATE_PAYMENT_STATUS);

  const { data: depositSelectData, error: depositSelectError } = useQuery<
    DepositSelect,
    DepositSelectVariables
  >(DEPOSIT_SELECT, {
    variables: {
      offer: offer.id,
    },
    skip: !offer.id || !depositReplacement,
  });

  const depositSelect = depositSelectData?.offer?.depositselect;
  const monthlyPayment = depositSelect?.schedule_deposit.monthly_payment;

  const { data: directDebitInvoiceData } = useQuery<
    PaymentsGetInvoice,
    PaymentsGetInvoiceVariables
  >(GET_INVOICE, {
    variables: {
      invoice_id: depositSelect?.invoice,
    },
    skip: !depositSelect || !depositReplacement,
  });

  const directDebitInvoice = directDebitInvoiceData?.admin?.Invoice;

  const invoice = useMemo(
    () =>
      offer.invoices.find((p) =>
        (p.services ?? []).find(
          (s) =>
            s.service.type === extras_api_ServiceTypeEnum.RENT &&
            s.service.role === extras_api_ServiceRoleEnum.TENANT_SERVICES
        )
      ),
    [offer]
  );

  const firstMonthRentInvoice = useMemo(
    () =>
      offer.invoices.find((p) =>
        (p.services ?? []).find(
          (s) =>
            s.service.type === extras_api_ServiceTypeEnum.RENT &&
            s.service.code === "first_month_rent"
        )
      ),
    [offer.invoices]
  );

  const paymentsSortedByScheduleDate = useMemo(
    () =>
      [...(invoice?.payments ?? [])].sort((a, b) => {
        if (a.scheduled_at && b.scheduled_at) {
          return (
            new Date(a.scheduled_at).getTime() -
            new Date(b.scheduled_at).getTime()
          );
        }

        return 0;
      }),
    [invoice]
  );

  if (!invoice || !invoice.payments) {
    return null;
  }

  const handleApproveSubmit = (
    payment_id: string,
    status: PaymentStatusEnum
  ) => {
    confirmPayment({
      variables: {
        payment_id,
        status,
      },
      awaitRefetchQueries: true,
      refetchQueries: ["GetOffer"],
    });
  };

  return (
    <>
      <Tabs>
        {Object.keys(TABS).map((t) => {
          return (
            <Tab
              key={t}
              active={tab === TABS[t]}
              onClick={() => setTab(TABS[t])}
            >
              {TABS[t]}
            </Tab>
          );
        })}
      </Tabs>

      {tab === TABS.FIRST_MONTH_RENT ? (
        <>
          {(firstMonthRentInvoice?.payments ?? []).length === 0 ? (
            <h3>No payments have been made yet</h3>
          ) : (
            <>
              {firstMonthRentInvoice != null ? (
                <>
                  <h3>Invoice</h3>
                  <ContentWrapper>
                    <Invoice
                      amount={firstMonthRentInvoice.amount}
                      created_at={firstMonthRentInvoice.created_at}
                      invoice_id={firstMonthRentInvoice.invoice_id}
                      reference={firstMonthRentInvoice.reference}
                      status={firstMonthRentInvoice.status}
                      updated_at={firstMonthRentInvoice.updated_at}
                    />
                  </ContentWrapper>
                </>
              ) : null}

              <h3>Payment</h3>
              {firstMonthRentInvoice?.payments?.map((payment) => {
                const canChangeStatus =
                  payment.status === PaymentStatusEnum.PAYMENT_PENDING ||
                  payment.status === PaymentStatusEnum.PAYMENT_CONFIRM;

                return (
                  <Fragment key={payment.payment_id}>
                    <Wrapper key={payment.payment_id}>
                      <ContentWrapper>
                        <Payment
                          amount={payment.amount}
                          created_at={payment.created_at}
                          matches={payment.matches}
                          payment_id={payment.payment_id}
                          status={payment.status}
                          updated_at={payment.updated_at}
                        />
                      </ContentWrapper>

                      <PropertyAside>
                        <PropertyVerifiedBlock>
                          <Form
                            initialValues={{
                              verified:
                                payment.status ===
                                PaymentStatusEnum.PAYMENT_SUCCESS,
                            }}
                            onSubmit={() => {}}
                            render={({ handleSubmit }) => (
                              <form onSubmit={handleSubmit}>
                                <UserFormRow
                                  disabled={loading || !canChangeStatus}
                                >
                                  <Field
                                    component={CheckboxToggle}
                                    disabled={loading || !canChangeStatus}
                                    labelText={"Confirmed"}
                                    name={"verified"}
                                    onChange={(event) => {
                                      if (
                                        canChangeStatus &&
                                        event.target.checked
                                      ) {
                                        handleApproveSubmit(
                                          payment.payment_id,
                                          PaymentStatusEnum.PAYMENT_SUCCESS
                                        );
                                      }
                                    }}
                                    type="checkbox"
                                  />
                                </UserFormRow>

                                {error && <div>{error.message}</div>}
                              </form>
                            )}
                          />
                        </PropertyVerifiedBlock>
                      </PropertyAside>
                    </Wrapper>
                  </Fragment>
                );
              })}
            </>
          )}
        </>
      ) : null}

      {tab === TABS.RENT_PAYMENTS ? (
        <>
          {invoice.payments.length === 0 ? (
            <h3>No payments have been made yet</h3>
          ) : (
            <>
              <h3>Invoice</h3>
              <ContentWrapper>
                <Invoice
                  amount={invoice.amount}
                  created_at={invoice.created_at}
                  invoice_id={invoice.invoice_id}
                  reference={invoice.reference}
                  status={invoice.status}
                  updated_at={invoice.updated_at}
                />
              </ContentWrapper>

              <h3>Payments</h3>
              {paymentsSortedByScheduleDate.map((payment) => {
                const canChangeStatus =
                  payment.status === PaymentStatusEnum.PAYMENT_PENDING ||
                  payment.status === PaymentStatusEnum.PAYMENT_CONFIRM;

                return (
                  <Fragment key={payment.payment_id}>
                    <Wrapper key={payment.payment_id}>
                      <ContentWrapper>
                        <Payment
                          amount={payment.amount}
                          created_at={payment.created_at}
                          matches={payment.matches}
                          payment_id={payment.payment_id}
                          scheduled_at={payment.scheduled_at}
                          status={payment.status}
                          updated_at={payment.updated_at}
                        />
                      </ContentWrapper>

                      <PropertyAside>
                        <PropertyVerifiedBlock>
                          <Form
                            initialValues={{
                              verified:
                                payment.status ===
                                PaymentStatusEnum.PAYMENT_SUCCESS,
                            }}
                            onSubmit={() => {}}
                            render={({ handleSubmit }) => (
                              <form onSubmit={handleSubmit}>
                                <UserFormRow
                                  disabled={loading || !canChangeStatus}
                                >
                                  <Field
                                    component={CheckboxToggle}
                                    disabled={loading || !canChangeStatus}
                                    labelText={"Confirmed"}
                                    name={"verified"}
                                    onChange={(event) => {
                                      if (
                                        canChangeStatus &&
                                        event.target.checked
                                      ) {
                                        handleApproveSubmit(
                                          payment.payment_id,
                                          PaymentStatusEnum.PAYMENT_SUCCESS
                                        );
                                      }
                                    }}
                                    type="checkbox"
                                  />
                                </UserFormRow>

                                {error && <div>{error.message}</div>}
                              </form>
                            )}
                          />
                        </PropertyVerifiedBlock>
                      </PropertyAside>
                    </Wrapper>
                  </Fragment>
                );
              })}
            </>
          )}
        </>
      ) : null}

      {tab === TABS.DIRECT_DEBIT ? (
        <>
          {depositReplacement && (
            <>
              <h3>Direct Debit</h3>
              {depositSelectError ? (
                <SystemError error={depositSelectError} />
              ) : (
                <Wrapper>
                  <ContentWrapper>
                    <Content>
                      <BankDetails>
                        <Col>
                          <PropertyCardItemTitle>
                            Direct debit amount
                          </PropertyCardItemTitle>
                          <PropertyCardItemValue data-testid="tab-final-payment-direct-debit-amount">
                            {monthlyPayment
                              ? `${formatPrice(monthlyPayment)} per month`
                              : "—"}
                          </PropertyCardItemValue>
                        </Col>

                        <Col>
                          <PropertyCardItemTitle>
                            Status of the direct debit
                          </PropertyCardItemTitle>
                          <PropertyCardItemValue data-testid="tab-final-payment-status-of-the-direct-debit">
                            {directDebitInvoice?.status
                              ? sentenceCase(
                                  getInvoicePaymentStatus(directDebitInvoice)
                                )
                              : "—"}
                            &nbsp;&nbsp;&nbsp;
                            {depositSelect?.invoice && (
                              <Link
                                className="link"
                                to={{
                                  pathname: "/invoices/details",
                                  search: `invoiceId=${depositSelect.invoice}`,
                                }}
                              >
                                View invoice
                              </Link>
                            )}
                          </PropertyCardItemValue>
                        </Col>
                      </BankDetails>
                    </Content>
                  </ContentWrapper>
                </Wrapper>
              )}
            </>
          )}
        </>
      ) : null}
    </>
  );
};

const Tabs = styled.div`
  display: flex;
  margin: 0 0 20px;
`;

const Tab = styled.div<{ active: boolean }>`
  padding: 5px 0;
  margin: 0 40px 0 0;
  font-size: 16px;
  font-weight: 500;
  border-bottom: dashed 2px #ddd;
  position: relative;
  top: 1px;
  cursor: pointer;

  ${({ active }) =>
    active
      ? css`
          color: #00b8ff;
          border-bottom: solid 2px #00b8ff;
          cursor: default;
        `
      : ""}
`;

const Col = styled.div`
  padding-right: 30px;
`;
const Content = styled.div`
  margin-top: 34px;

  &:first-child {
    margin-top: 0;
  }
`;
const BankDetails = styled(PropertyGrid)`
  padding: 30px 30px 44px;
  background: #f9f6f1;
  border: dashed 2px ${theme.colors.main.lightBlue};
  margin-bottom: 40px;

  &:last-child {
    margin-bottom: 0;
  }
`;
const Wrapper = styled.div`
  display: flex;
`;
const ContentWrapper = styled.div`
  width: calc(100% - 270px);
  margin-bottom: 30px;
  line-height: 24px;
  font-weight: 400;
  font-size: 15px;
  border: dashed 1px #ddd;
  padding: 2px;
  background: white;
`;

export default TabRentPayment;
