import {
  GetOfferScheduledPayments,
  GetOfferScheduledPaymentsVariables,
  GetOfferScheduledPayments_admin_allScheduledPayments_data as ScheduledPayment,
} from "../../graphql/queries/__generated__/GetOfferScheduledPayments";
import { Link } from "react-router-dom";
import { GetOffer_admin_Offer as Offer } from "../../graphql/queries/__generated__/GetOffer";
import { Table, Tbody, Td, Th, Thead, Tr } from "../../../../ui/atoms/Table";
import {
  common_api_generic_query_QueryFilterOperatorEnum,
  offer_api_AdvanceStateEnum,
} from "../../../../globalTypes";
import { format } from "date-fns";
import { formatDate } from "../../../../util/formatters/date";
import { formatPrice } from "../../../../util/price";
import { useQuery } from "@apollo/react-hooks";
import Pagebar from "../../../../ui/molecules/Pagebar";
import React, { FC, useMemo, useState } from "react";
import SCHEDULED_PAYMENTS from "../../graphql/queries/GetOfferScheduledPayments.graphql";
import Spinner from "../../../../ui/atoms/Spinner";
import SystemError from "../../../../ui/atoms/SystemError";
import TableWrapper from "../../../../ui/atoms/TableWrapper";
import UserInfo from "../../../payments/UserInfo";

type Props = { offer: Offer };

const TabScheduled: FC<Props> = ({ offer }) => {
  const [schedulePage, setSchedulePage] = useState(1);
  const { data, loading, error } = useQuery<
    GetOfferScheduledPayments,
    GetOfferScheduledPaymentsVariables
  >(SCHEDULED_PAYMENTS, {
    variables: {
      filter: [
        {
          fields: ["offer"],
          operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
          value: [offer.id],
        },
      ],
      pagination: {
        page: schedulePage,
        perPage: 15,
      },
    },
  });

  const scheduled = data?.admin?.allScheduledPayments?.data ?? [];

  if (loading) {
    return <Spinner />;
  }

  if (error) {
    return <SystemError error={error} />;
  }

  return (
    <TableWrapper>
      <Table>
        <Thead>
          <TabScheduledHead />
        </Thead>
        <Tbody>
          {scheduled.map((p) => (
            <TabScheduledItem key={p.id} payment={p} />
          ))}
        </Tbody>
      </Table>
      <Pagebar
        changePage={(n) => setSchedulePage(n)}
        curPage={schedulePage}
        showNPages={3}
        total={Math.ceil((data?.admin?.allScheduledPayments?.total ?? 0) / 15)}
      />
    </TableWrapper>
  );
};

export default TabScheduled;

const TabScheduledHead: FC = () => (
  <Tr>
    <Th>Payment ID</Th>
    <Th>Invoice ID</Th>
    <Th>Created at</Th>
    <Th>Due date</Th>
    <Th>Payer</Th>
    <Th>Amount</Th>
    <Th>Advanced</Th>
  </Tr>
);

const TabScheduledItem: FC<{ payment: ScheduledPayment }> = ({ payment }) => {
  const firstPayment = useMemo(() => {
    return [...(payment?.payout?.invoice?.payments ?? [])].sort((a, b) =>
      a.scheduled_at && b.scheduled_at
        ? new Date(b.scheduled_at).getTime() -
          new Date(a.scheduled_at).getTime()
        : 0
    )[0];
  }, [payment]);

  return (
    <Tr>
      <Td>{payment.id}</Td>
      <Td>
        {payment.payout.invoice_id !== "" ? (
          <Link
            className="link"
            to={{
              pathname: "/invoices/details",
              search: `invoiceId=${payment.payout.invoice_id}`,
            }}
          >
            {payment.payout.invoice_id}
          </Link>
        ) : (
          "—"
        )}
      </Td>
      <Td>
        {format(
          new Date(payment.date.year, payment.date.month - 1),
          "MMM yyyy"
        )}
      </Td>
      <Td>
        {firstPayment != null ? formatDate(firstPayment.scheduled_at) : "—"}
      </Td>
      <Td>
        {Boolean(payment.payout.invoice?.user) ? (
          <UserInfo userId={payment.payout.invoice?.user!} />
        ) : (
          "—"
        )}
      </Td>
      <Td>{formatPrice(payment.payout.invoice?.amount)}</Td>
      <Td>
        {payment.advance.state === offer_api_AdvanceStateEnum.ADVANCE_GRANTED
          ? "true"
          : "false"}
        {payment.advance.state ===
          offer_api_AdvanceStateEnum.ADVANCE_GRANTED && (
          <>
            <br />
            {`fee: ${payment.advance.fee_percentage}%`}
          </>
        )}
      </Td>
    </Tr>
  );
};
