import { Field, Form } from "react-final-form";
import {
  FilterSort,
  FilterSortGroup,
  FilterSortItem,
} from "../../ui/atoms/FilterSort";
import {
  common_api_generic_query_QueryFilterOperatorEnum as OperatorEnum,
  common_api_generic_query_QueryParamsFilterInput as ParamsFilterInput,
  invoice_api_PaymentStatusEnum,
} from "../../globalTypes";
import {
  PayoutsGetAllPayments,
  PayoutsGetAllPaymentsVariables,
} from "../payouts-made/graphql/queries/__generated__/PayoutsGetAllPayments";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import { SearchFieldWide } from "../../ui/atoms/SearchField";
import { Table, Tbody } from "../../ui/atoms/Table";
import { TextInput } from "../../ui/atoms/Input";
import { makeFilters } from "../../util/search_filters";
import { usePagination } from "../../ui/molecules/Pagination";
import { useQuery } from "react-apollo";
import { useSortable } from "../../ui/molecules/Sortable";
import Dropdown from "../../ui/molecules/Dropdown";
import GET_ALL_PAYMENTS from "../payouts-made/graphql/queries/PayoutsGetAllPayments.graphql";
import ListHead from "./ListHead";
import ListItem from "./ListItem";
import Pagebar from "../../ui/molecules/Pagebar";
import React, { FC, useState } from "react";
import Spinner from "../../ui/atoms/Spinner";
import TableWrapper from "../../ui/atoms/TableWrapper";
import getSortOrder from "../../util/ger_sort_order";

interface FormValues {
  search: string;
  date: string;
  correlationTarget: {
    label: string;
    value: string;
  };
  paymentDateMin: string;
  paymentDateMax: string;
  payoutDateMin: string;
  payoutDateMax: string;
}

const CORRELATION_TARGET_OPTIONS = [
  {
    label: "standing order",
    value: "tenant-services"
  },
  {
    label: "other",
    value: "tenant-services"
  },
];

const OverduePayments: FC = () => {
  const { field, order, setSort } = useSortable({
    field: ["scheduled"],
    order: 1,
  });
  const today = new Date().toUTCString();
  const basicFilter: ParamsFilterInput[] = [
    {
      fields: ["status"],
      operator: OperatorEnum.EQUALS,
      value: [invoice_api_PaymentStatusEnum.PAYMENT_PENDING],
    },
    {
      fields: ["scheduled"],
      operator: OperatorEnum.LESS,
      value: [today],
    },
  ];
  const [filter, setFilter] = useState<ParamsFilterInput[]>(basicFilter);
  const { page, setPage, perPage } = usePagination({ perPage: 25 });

  const { data, loading } = useQuery<
    PayoutsGetAllPayments,
    PayoutsGetAllPaymentsVariables
  >(GET_ALL_PAYMENTS, {
    variables: {
      pagination: { page, perPage },
      sort: field.map((v) => ({ field: v, order })),
      filter,
    },
  });

  const getOrder = (fieldName: string) =>
    getSortOrder({ field, fieldName, order });

  const paymentsData = data?.admin?.allPayments;
  const payments = paymentsData?.payments ?? [];
  const total = paymentsData?.total ?? 1;

  const handleFilterSubmit = (values: FormValues) => {
    let correlationTargetNot = false;
    if(values.correlationTarget.label === "other") {
      correlationTargetNot = true;
    }
    setFilter([
      ...basicFilter,
      ...makeFilters({
        values,
        filters: [
          {
            fields: ["amount", "ivr"],
            key: "search",
            operator: "CONTAINS_ANY",
          },
          {
            fields: ["scheduled"],
            key: "min,max",
            operator: "RANGE",
          },
          {
            fields: ["correlation_target"],
            key: "correlationTarget.value",
            operator: OperatorEnum.EQUALS,
            not: correlationTargetNot
          }
        ],
      }),
    ]);
  };

  return (
    <>
      <h2>Overdue payments</h2>
      <Form<FormValues> onSubmit={handleFilterSubmit}>
        {({ form, handleSubmit, hasValidationErrors, dirty }) => (
          <form onSubmit={handleSubmit}>
            <SearchFieldWide>
              <Field
                component={TextInput}
                name="search"
                placeholder="Enter ivr, amount"
              />
            </SearchFieldWide>
            <FilterSort>
              <FilterSortItem>
                <label htmlFor="min">Payment date filter:</label>
                <FilterSortGroup>
                  <Field
                    name="min"
                    render={(props) => (
                      <TextInput className="input-sort-date" {...props} />
                    )}
                    type="date"
                  />
                  &nbsp;&ndash;&nbsp;
                  <Field
                    name="max"
                    render={(props) => (
                      <TextInput className="input-sort-date" {...props} />
                    )}
                    type="date"
                  />
                </FilterSortGroup>
              </FilterSortItem>
              <FilterSortItem>
                  <label htmlFor="status">Tenancy status:</label>
                  <div>
                    <Field
                      name="correlationTarget"
                      render={({ input }) => (
                        <Dropdown
                          input={input}
                          options={CORRELATION_TARGET_OPTIONS}
                          placeholder="Choose tenancy status"
                        />
                      )}
                    />
                  </div>
                </FilterSortItem>

              <PrimaryButton disabled={hasValidationErrors} type="submit">
                Search
              </PrimaryButton>
              {dirty && (
                <>
                  &nbsp;&nbsp;
                  <SecondaryButton
                    onClick={() => {
                      form.reset();
                      setFilter(basicFilter);
                    }}
                    type="button"
                  >
                    Clear
                  </SecondaryButton>
                </>
              )}
              <FilterSortItem />
            </FilterSort>
          </form>
        )}
      </Form>
      {loading ? (
        <Spinner />
      ) : (
        <TableWrapper>
          <Pagebar
            changePage={setPage}
            curPage={page}
            showNPages={3}
            total={Math.ceil(total / perPage)}
          />
          <Table>
            <ListHead
              field={field}
              getOrder={getOrder}
              order={order}
              setSort={setSort}
            />
            <Tbody>
              {payments.map(
                (
                  {
                    amount,
                    overdue_payment_notice_count,
                    payment_id,
                    property,
                    scheduled_at,
                  },
                  index
                ) => {
                  return (
                    <ListItem
                      key={index}
                      address={property?.location.address}
                      amount={amount}
                      landlord_details={property?.people.landlord?.details}
                      overdue_payment_notice_count={
                        overdue_payment_notice_count
                      }
                      payment_id={payment_id}
                      scheduled_at={scheduled_at}
                      submitter_details={property?.people.submitter?.details}
                    />
                  );
                }
              )}
            </Tbody>
          </Table>
          <Pagebar
            changePage={setPage}
            curPage={page}
            showNPages={3}
            total={Math.ceil(total / perPage)}
          />
        </TableWrapper>
      )}
    </>
  );
};

export default OverduePayments;
