import { CreateTokenRequest } from "../graphql/mutations/__generated__/CreateTokenRequest";
import { Field, Form } from "react-final-form";
import {
  FilterSort,
  FilterSortGroup,
  FilterSortItem,
} from "../../../ui/atoms/FilterSort";
import { common_api_generic_query_QueryParamsFilterInput as ParamsFilterInput } from "../../../globalTypes";
import {
  PaymentsGetAllTransactions,
  PaymentsGetAllTransactionsVariables,
} from "../graphql/queries/__generated__/PaymentsGetAllTransactions";
import { PrimaryButton, SecondaryButton } from "../../../ui/atoms/Button";
import {
  PullTransactions,
  PullTransactionsVariables,
} from "../graphql/mutations/__generated__/PullTransactions";
import { SearchFieldWide } from "../../../ui/atoms/SearchField";
import { Table, Tbody } from "../../../ui/atoms/Table";
import { TextInput } from "../../../ui/atoms/Input";
import { makeFilters } from "../../../util/search_filters";
import { useMutation, useQuery } from "react-apollo";
import { usePagination } from "../../../ui/molecules/Pagination";
import { useSortable } from "../../../ui/molecules/Sortable";
import { validRanges } from "../../../util/validations";
import GET_ALL_TRANSACTIONS from "../graphql/queries/PaymentsGetAllTransactions.graphql";
import ListHead from "./ListHead";
import ListItem from "./ListItem";
import Pagebar from "../../../ui/molecules/Pagebar";
import REFRESH_AUTH from "../graphql/mutations/CreateTokenRequest.graphql";
import REFRESH_PAYMENTS from "../graphql/mutations/PullTransactions.graphql";
import React, { FC, useState } from "react";
import Spinner from "../../../ui/atoms/Spinner";
import TableWrapper from "../../../ui/atoms/TableWrapper";
import getSortOrder from "../../../util/ger_sort_order";
import styled from "@emotion/styled";

interface FormValues {
  search: string;
  date: string;
  min: string;
  max: string;
}

const AccountStatement: FC = () => {
  const { field, order, setSort } = useSortable({
    field: ["created"],
    order: -1,
  });
  const [filter, setFilter] = useState<ParamsFilterInput[]>([]);
  const { page, setPage, perPage } = usePagination({ perPage: 25 });

  const { data, loading, error, refetch } = useQuery<
    PaymentsGetAllTransactions,
    PaymentsGetAllTransactionsVariables
  >(GET_ALL_TRANSACTIONS, {
    variables: {
      pagination: { page, perPage },
      sort: field.map((v) => ({ field: v, order })),
      filter,
    },
  });

  const [refreshPayments, { loading: refreshPaymentsLoading }] = useMutation<
    PullTransactions,
    PullTransactionsVariables
  >(REFRESH_PAYMENTS, {
    onCompleted: () => refetch(),
  });

  const [
    refreshAuth,
    { loading: refreshAuthLoading, error: refreshAutherror },
  ] = useMutation<CreateTokenRequest>(REFRESH_AUTH);

  const handleRefreshPayments = () => refreshPayments();

  const handleRefreshAuth = () =>
    refreshAuth().then(
      (v) =>
        v.data?.admin?.createTokenRequest?.redirect_url &&
        window.open(v.data?.admin?.createTokenRequest?.redirect_url)
    );

  const getOrder = (fieldName: string) =>
    getSortOrder({ field, fieldName, order });
  const onSetPage = (pageNumber: number) => setPage(pageNumber);

  const transactionsData = data?.admin?.allTransactions;
  const transactions = transactionsData?.transactions ?? [];
  const total = transactionsData?.total ?? 1;

  const handleFilterSubmit = (values: FormValues) => {
    setFilter(
      makeFilters({
        values,
        filters: [
          {
            fields: ["id", "ivr"],
            key: "search",
            operator: "CONTAINS_ANY",
            transformation: {
              preprocessor: "REGEXP_REPLACE",
              param: ["[- /]", ""],
            },
          },
          {
            fields: ["created"],
            key: "date",
            operator: "EQUALS",
          },
          {
            fields: ["amount"],
            key: "min,max",
            operator: "RANGE",
          },
        ],
      })
    );
  };

  return (
    <>
      <h2>Account statement</h2>
      <Form<FormValues> onSubmit={handleFilterSubmit}>
        {({ form, handleSubmit, hasValidationErrors, dirty }) => (
          <form onSubmit={handleSubmit}>
            <SearchFieldWide>
              <Field
                component={TextInput}
                name="search"
                placeholder="Enter transaction id or ivr"
              />
            </SearchFieldWide>
            <FilterSort>
              <FilterSortItem>
                <label htmlFor="date">Date:</label>
                <Field
                  name="date"
                  render={(props) => (
                    <TextInput className="input-sort-date" {...props} />
                  )}
                  type="date"
                />
              </FilterSortItem>
              <FilterSortItem>
                <label htmlFor="min">Amount (&pound;):</label>
                <FilterSortGroup>
                  <Field
                    className="input-amount"
                    component={TextInput}
                    name="min"
                    placeholder="from"
                    validate={validRanges("max")}
                  />
                  &nbsp;&ndash;&nbsp;
                  <Field
                    className="input-amount"
                    component={TextInput}
                    name="max"
                    placeholder="to"
                    validate={validRanges("min")}
                  />
                </FilterSortGroup>
              </FilterSortItem>
              <PrimaryButton disabled={hasValidationErrors} type="submit">
                Search
              </PrimaryButton>
              {dirty && (
                <>
                  &nbsp;&nbsp;
                  <SecondaryButton
                    onClick={() => {
                      form.reset();
                      setFilter([]);
                    }}
                    type="button"
                  >
                    Clear
                  </SecondaryButton>
                </>
              )}
              <Container>
                <PrimaryButton onClick={handleRefreshPayments} type="button">
                  Refresh payments
                </PrimaryButton>
                <Spacer />
                <PrimaryButton onClick={handleRefreshAuth} type="button">
                  Refresh auth
                </PrimaryButton>
              </Container>
              <FilterSortItem />
            </FilterSort>
          </form>
        )}
      </Form>
      {loading || refreshAuthLoading || refreshPaymentsLoading ? (
        <Spinner />
      ) : error ?? refreshAutherror ? (
        <>{error?.message ?? refreshAutherror?.message}</>
      ) : (
        <TableWrapper>
          <Pagebar
            changePage={onSetPage}
            curPage={page}
            showNPages={3}
            total={Math.ceil(total / perPage)}
          />
          <Table>
            <ListHead
              field={field}
              getOrder={getOrder}
              order={order}
              setSort={setSort}
            />
            <Tbody>
              {transactions.map((data) => (
                <ListItem key={data.id} transaction={data} />
              ))}
            </Tbody>
          </Table>
          <Pagebar
            changePage={onSetPage}
            curPage={page}
            showNPages={3}
            total={Math.ceil(total / perPage)}
          />
        </TableWrapper>
      )}
    </>
  );
};

export default AccountStatement;

const Container = styled.div`
  display: flex;
  justify-content: flex-end;
  width: 100%;
  padding: 0 0 0 10px;
`;

const Spacer = styled.div`
  width: 15px;
`;
