import {
  ColumnNarrow,
  ColumnTitle,
  ColumnValue,
  ColumnWide,
  PaymentConfirmed,
  PaymentDescription,
  PaymentHead,
  PaymentRow,
  PaymentsGroup,
  PaymentsGroupHead,
} from "../../../ui/atoms/Payment";
import {
  CreatePayment,
  CreatePaymentVariables,
} from "../graphql/mutations/__generated__/CreatePayment";
import {
  DeleteButton,
  PrimaryButton,
  SecondaryButton,
  SuccessButton,
} from "../../../ui/atoms/Button";
import { Link, useParams } from "react-router-dom";
import { ModalAlert } from "../../../ui/molecules/Modal";
import {
  PaymentsGetSelectedScheduledPayments,
  PaymentsGetSelectedScheduledPaymentsVariables,
} from "../graphql/queries/__generated__/PaymentsGetSelectedScheduledPayments";
import {
  UpdateScheduledPayment,
  UpdateScheduledPaymentVariables,
} from "../graphql/mutations/__generated__/UpdateScheduledPayment";
import {
  common_api_generic_query_QueryFilterOperatorEnum,
  offer_api_AdvanceStateEnum,
} from "../../../globalTypes";
import { format } from "date-fns";
import { formatPrice } from "../../../util/price";
import { getAdvancedRentStatus } from "./util";
import { useMutation, useQuery } from "@apollo/react-hooks";
import CREATE_PAYMENT from "../graphql/mutations/CreatePayment.graphql";
import GET_PAYMENT from "../graphql/queries/PaymentsGetSelectedScheduledPayments.graphql";
import React, { useState } from "react";
import Spinner from "../../../ui/atoms/Spinner";
import UPDATE_PAYMENT from "../graphql/mutations/UpdateScheduledPayment.graphql";
import UserInfo from "../UserInfo";
import styled from "@emotion/styled";
import theme from "../../../ui/theme";

const ScheduledPayment = ({ location }) => {
  const params = new URLSearchParams(location.search);
  const offerId = params.get("offer");
  const { paymentId } = useParams();
  const [alertMessage, setAlertMessage] = useState<string | null>(null);

  const { data, loading, error, refetch } = useQuery<
    PaymentsGetSelectedScheduledPayments,
    PaymentsGetSelectedScheduledPaymentsVariables
  >(GET_PAYMENT, {
    variables: {
      filter: [
        {
          fields: ["id"],
          operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
          value: [paymentId],
        },
        {
          fields: ["offer"],
          operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
          value: [offerId],
        },
      ],
    },
    skip: paymentId == null || offerId == null,
  });

  const scheduledPayments = data?.admin?.allScheduledPayments?.data ?? [];
  const payment = scheduledPayments[0];

  const [createPayment, { loading: createLoading }] = useMutation<
    CreatePayment,
    CreatePaymentVariables
  >(CREATE_PAYMENT);

  const handleClickPayout = () => {
    createPayment({
      variables: {
        invoice_id: payment.payout.invoice_id,
      },
    }).then((res) => {
      if (res.data?.admin?.createPayment?.payment) {
        refetch();
        setAlertMessage("Payment created");
      } else if (res.errors && res.errors.length > 0) {
        setAlertMessage(res.errors.map((item) => item.message).join(", "));
      }
    });
  };

  const [updateScheduledPayment, { loading: updateLoading }] = useMutation<
    UpdateScheduledPayment,
    UpdateScheduledPaymentVariables
  >(UPDATE_PAYMENT);

  const handleUpdate = (state: offer_api_AdvanceStateEnum) => {
    updateScheduledPayment({
      variables: {
        id: payment.id,
        offer: offerId,
        advance: {
          state,
        },
      },
    }).then((res) => {
      if (res.data?.admin?.updateScheduledPayment?.id) {
        refetch();
      } else if (res.errors && res.errors.length > 0) {
        setAlertMessage(res.errors.map((item) => item.message).join(", "));
      }
    });
  };

  if (loading) {
    return <Spinner />;
  }

  if (error) {
    return <>{error.message}</>;
  }

  if (!payment || !offerId) {
    return null;
  }

  return (
    <>
      <PaymentsGroup>
        <PaymentsGroupHead>
          <Link className="link-button" to="/advanced">
            <SecondaryButton>Back</SecondaryButton>
          </Link>
          <h2>Request</h2>
        </PaymentsGroupHead>

        <PaymentConfirmed>
          <PaymentHead>
            <PaymentDescription>
              Payment ID <b>{payment.id}</b>
            </PaymentDescription>
            <PaymentDescription>
              Invoice ID <b>{payment.payout.invoice?.invoice_id}</b>
            </PaymentDescription>
          </PaymentHead>
        </PaymentConfirmed>

        <PaymentRow>
          <ColumnWide>
            <ColumnTitle>Requested at</ColumnTitle>
            <ColumnValue>
              {format(
                new Date(payment.date.year, payment.date.month - 1),
                "MMM yyyy"
              )}
            </ColumnValue>
          </ColumnWide>
          <ColumnNarrow>
            <ColumnTitle>Months requested</ColumnTitle>
            <ColumnValue>
              1 (
              {format(
                new Date(payment.date.year, payment.date.month - 1),
                "MMMM"
              )}
              )
            </ColumnValue>
          </ColumnNarrow>
        </PaymentRow>

        <PaymentRow>
          <ColumnNarrow>
            <ColumnTitle>Amount requested</ColumnTitle>
            <ColumnValue>
              {formatPrice(payment.offer?.conditions.price)}
            </ColumnValue>
          </ColumnNarrow>
          <ColumnNarrow>
            <ColumnTitle>Mashroom fee</ColumnTitle>
            <ColumnValue>
              {payment.offer?.conditions.price
                ? formatPrice(
                    (payment.offer.conditions.price / 100) *
                      payment.advance.fee_percentage
                  )
                : "—"}
            </ColumnValue>
          </ColumnNarrow>
          <ColumnNarrow>
            <ColumnTitle>Status</ColumnTitle>
            <ColumnValue>
              {getAdvancedRentStatus(payment.advance.state)}
            </ColumnValue>
          </ColumnNarrow>
        </PaymentRow>

        {(payment.advance.state ===
          offer_api_AdvanceStateEnum.ADVANCE_REQUESTED ||
          payment.advance.state ===
            offer_api_AdvanceStateEnum.UNKNOWN_ADVANCE_STATE) && (
          <Buttons>
            <SuccessButton
              disabled={updateLoading}
              onClick={() =>
                handleUpdate(offer_api_AdvanceStateEnum.ADVANCE_GRANTED)
              }
              type="button"
            >
              Accept
            </SuccessButton>
            <DeleteButton
              disabled={updateLoading}
              onClick={() =>
                handleUpdate(offer_api_AdvanceStateEnum.ADVANCE_CANCELLED)
              }
              type="button"
            >
              Decline
            </DeleteButton>
          </Buttons>
        )}

        <h3>Payout</h3>
        <ContentWrapper>
          <PayoutInfo>
            <PayoutInfoCol>
              <PayoutInfoTitle>Amount owed</PayoutInfoTitle>
              <div>{formatPrice(payment.offer?.conditions.price)}</div>
            </PayoutInfoCol>
            <PayoutInfoCol>
              <PayoutInfoTitle>Name</PayoutInfoTitle>
              <div>
                {payment.offer ? (
                  <UserInfo userId={payment.offer.landlord.user} />
                ) : (
                  "—"
                )}
              </div>
            </PayoutInfoCol>
            <PayoutInfoCol>
              <PayoutInfoTitle>Sort code</PayoutInfoTitle>
              <div>{payment.offer?.property?.bank_details.sort_code}</div>
            </PayoutInfoCol>
            <PayoutInfoCol>
              <PayoutInfoTitle>Account number</PayoutInfoTitle>
              <div>{payment.offer?.property?.bank_details.account_number}</div>
            </PayoutInfoCol>
          </PayoutInfo>

          <PrimaryButton
            disabled={createLoading}
            onClick={handleClickPayout}
            type="button"
          >
            Payout {formatPrice(payment.offer?.conditions.price)}
          </PrimaryButton>
        </ContentWrapper>
      </PaymentsGroup>

      {alertMessage && (
        <ModalAlert
          onCancel={() => {
            setAlertMessage(null);
          }}
        >
          <b>{alertMessage}</b>
        </ModalAlert>
      )}
    </>
  );
};

export default ScheduledPayment;

const ContentWrapper = styled.div`
  padding: 20px 40px;
  background: white;
  border: solid 1px ${theme.colors.icons.darkBlue};
  border-radius: 5px;
  margin-bottom: 30px;
  font-size: 18px;
  line-height: 1.5;
  font-weight: 500;
`;

const PayoutInfo = styled.div`
  display: flex;
`;

const PayoutInfoCol = styled.div`
  width: 25%;
  padding: 7px 0 18px;
  font-size: 14px;

  > div {
    width: 100%;
    overflow: hidden;
    white-space: nowrap;
    text-overflow: ellipsis;
  }
`;

const PayoutInfoTitle = styled.div`
  text-transform: uppercase;
  font-weight: 700;
  margin-bottom: 6px;
`;

const Buttons = styled.div`
  display: flex;
  margin-bottom: 20px;

  button {
    margin-right: 20px;
  }
`;
