import { DownloadPayment } from "../../../ui/atoms/Payment";
import { Field, Form } from "react-final-form";
import {
  FilterSort,
  FilterSortGroup,
  FilterSortItem,
} from "../../../ui/atoms/FilterSort";
import { InvoiceStatusOptions } from "../constants";
import { Link } from "react-router-dom";
import {
  common_api_generic_query_QueryParamsFilterInput as ParamsFilterInput,
  invoice_api_ReportFormatEnum as ReportFormat,
  invoice_api_ReportTypeEnum as ReportType,
} from "../../../globalTypes";
import {
  PaymentsDownloadInvoice,
  PaymentsDownloadInvoiceVariables,
} from "../graphql/queries/__generated__/PaymentsDownloadInvoice";
import {
  PaymentsGetAllInvoices,
  PaymentsGetAllInvoicesVariables,
} from "../graphql/queries/__generated__/PaymentsGetAllInvoices";
import { PrimaryButton, SecondaryButton } from "../../../ui/atoms/Button";
import { SearchFieldWide } from "../../../ui/atoms/SearchField";
import { Table, Tbody } from "../../../ui/atoms/Table";
import { TextInput } from "../../../ui/atoms/Input";
import { WithApolloClient, useQuery, withApollo } from "react-apollo";
import { format } from "date-fns";
import { makeFilters } from "../../../util/search_filters";
import { usePagination } from "../../../ui/molecules/Pagination";
import { useSortable } from "../../../ui/molecules/Sortable";
import { validRanges } from "../../../util/validations";
import { wrapInputEmptyOption } from "../../../util/dropdown";
import Dropdown from "../../../ui/molecules/Dropdown";
import GET_ALL_INVOICES from "../graphql/queries/PaymentsGetAllInvoices.graphql";
import ListHead from "./ListHead";
import ListItem from "./ListItem";
import PAYMENT_DOWNLOAD_CSV from "../graphql/queries/PaymentsDownloadInvoice.graphql";
import Pagebar from "../../../ui/molecules/Pagebar";
import React, { FC, useState } from "react";
import Spinner from "../../../ui/atoms/Spinner";
import SystemError from "../../../ui/atoms/SystemError";
import TableWrapper from "../../../ui/atoms/TableWrapper";
import getSortOrder from "../../../util/ger_sort_order";

export type ListProps = WithApolloClient<{}>;

const Invoices: FC<ListProps> = ({ client }) => {
  const [selectedRows, setSelected] = useState<string[]>([]);
  const [filter, setFilter] = useState<ParamsFilterInput[]>([]);
  const { field, order, setSort } = useSortable({
    field: ["created"],
    order: -1,
  });
  const { page, setPage, perPage } = usePagination({ perPage: 25, filter });

  const { data, loading, error } = useQuery<
    PaymentsGetAllInvoices,
    PaymentsGetAllInvoicesVariables
  >(GET_ALL_INVOICES, {
    variables: {
      pagination: { page, perPage },
      sort: field.map((v) => ({ field: v, order })),
      filter,
    },
  });

  const invoicesData = data?.admin?.allInvoices;
  const allInvoices = invoicesData?.invoices ?? [];
  const total = invoicesData?.total ?? 0;

  const resetSelectedRows = () => {
    setSelected([]);
  };

  const getOrder = (fieldName: string) =>
    getSortOrder({ field, fieldName, order });
  const onSetPage = (pageNumber: number) => {
    setPage(pageNumber);
    resetSelectedRows();
  };

  const handleFilterSubmit = (values) => {
    setFilter(
      makeFilters({
        values,
        filters: [
          {
            fields: ["id", "reference", "property", "user"],
            key: "search",
            operator: "CONTAINS_ANY",
            transformation: {
              preprocessor: "REGEXP_REPLACE",
              param: ["[- /]", ""],
            },
          },
          {
            fields: ["status"],
            key: "status.value",
            operator: "EQUALS",
          },
          {
            fields: ["created"],
            key: "date",
            operator: "EQUALS",
          },
          {
            fields: ["amount"],
            key: "min,max",
            operator: "RANGE",
          },
        ],
      })
    );
  };

  const selectRow = (id: string) => () => {
    if (selectedRows.indexOf(id) > -1) {
      setSelected(selectedRows.filter((i) => i !== id));
      return;
    }

    setSelected([...selectedRows, id]);
  };

  const handleFilterReset = (cb) => {
    cb();
    setFilter([]);
  };

  const handleDownloadInvoice = (
    report_type: ReportType,
    report_format: ReportFormat = ReportFormat.CSV
  ) => {
    client
      .query<PaymentsDownloadInvoice, PaymentsDownloadInvoiceVariables>({
        query: PAYMENT_DOWNLOAD_CSV,
        variables: {
          report_type,
          report_format,
        },
      })
      .then((data) => {
        if (data?.data?.admin?.downloadInvoicesCsv?.file ?? false) {
          const csv = data!.data!.admin!.downloadInvoicesCsv!.file;
          const link = document.createElement("a");
          link.href = `/media/${csv}`;
          link.setAttribute(
            "download",
            `${report_type}_${format(new Date(), "dd-MM-yyyy-HH-mm-ss")}.csv`
          );
          document.body.appendChild(link);
          link.click();
          link.remove();
        } else if (data.errors && data.errors[0]) {
          // eslint-disable-next-line no-console
          console.log(data.errors[0]);
        }
      });
  };

  return (
    <>
      <h2>View Invoices</h2>

      <Form
        onSubmit={handleFilterSubmit}
        render={({
          handleSubmit,
          submitting,
          valid,
          values,
          form,
          pristine,
        }) => (
          <form onSubmit={handleSubmit}>
            <SearchFieldWide>
              <Field
                component={TextInput}
                name="search"
                placeholder="Enter Reference number or Invoice ID or Payer ID or Property ID"
              />
              <PrimaryButton
                disabled={!valid || submitting || pristine}
                onClick={() => handleFilterSubmit(values)}
                type="submit"
              >
                Search
              </PrimaryButton>
              {!(!valid || submitting || pristine) && (
                <>
                  &nbsp;&nbsp;
                  <SecondaryButton
                    onClick={() => handleFilterReset(form.reset)}
                    type="reset"
                  >
                    Clear
                  </SecondaryButton>
                </>
              )}
            </SearchFieldWide>
            <DownloadPayment>
              <strong>Invoice csv downloads: </strong>
              <PrimaryButton
                onClick={() =>
                  handleDownloadInvoice(ReportType.ADD_ON_PRODUCTS)
                }
              >
                Add-on products
              </PrimaryButton>
              <PrimaryButton
                onClick={() =>
                  handleDownloadInvoice(ReportType.INSURANCE_PRODUCTS)
                }
              >
                Insurance products
              </PrimaryButton>
            </DownloadPayment>
            <FilterSort>
              <FilterSortItem>
                <Field
                  name="date"
                  render={(props) => (
                    <TextInput className="input-sort-date" {...props} />
                  )}
                  type="date"
                />
              </FilterSortItem>
              <FilterSortItem>
                <Field
                  name="status"
                  render={({ input }) => {
                    return (
                      <Dropdown
                        input={wrapInputEmptyOption(input)}
                        options={InvoiceStatusOptions}
                        placeholder="Status"
                      />
                    );
                  }}
                />
              </FilterSortItem>
              <FilterSortItem>
                <label htmlFor="min">Amount (&pound;):</label>
                <FilterSortGroup>
                  <Field
                    className="input-amount"
                    component={TextInput}
                    name="min"
                    placeholder="from"
                    validate={validRanges("max")}
                  />
                  &nbsp;&ndash;&nbsp;
                  <Field
                    className="input-amount"
                    component={TextInput}
                    name="max"
                    placeholder="to"
                    validate={validRanges("min")}
                  />
                </FilterSortGroup>
              </FilterSortItem>

              <FilterSortItem />
            </FilterSort>
          </form>
        )}
      />

      {loading ? (
        <Spinner />
      ) : error ? (
        <SystemError error={error} />
      ) : (
        <TableWrapper>
          {selectedRows.length > 0 ? (
            <Link
              className="link-button"
              to={{
                pathname: "/invoices/details",
                search: `invoiceId=${selectedRows.join("&invoiceId=")}`,
              }}
            >
              <PrimaryButton>Show selected</PrimaryButton>
            </Link>
          ) : (
            <PrimaryButton disabled> Show selected </PrimaryButton>
          )}

          <Pagebar
            changePage={onSetPage}
            curPage={page}
            showNPages={3}
            total={Math.ceil(total / perPage)}
          />

          <Table>
            <ListHead
              field={field}
              getOrder={getOrder}
              order={order}
              setSort={setSort}
            />
            <Tbody>
              {allInvoices.map((v) => (
                <ListItem
                  key={v.invoice_id}
                  invoice={v}
                  selectedRows={selectedRows}
                  selectRow={selectRow}
                />
              ))}
            </Tbody>
          </Table>
          <Pagebar
            changePage={onSetPage}
            curPage={page}
            showNPages={3}
            total={Math.ceil(total / perPage)}
          />
        </TableWrapper>
      )}
    </>
  );
};

export default withApollo(Invoices);
