import { Link } from "react-router-dom";
import { ModalWide } from "../../../../ui/molecules/Modal";
import {
  PaymentsGetAllScheduledPayments,
  PaymentsGetAllScheduledPaymentsVariables,
} from "../../graphql/queries/__generated__/PaymentsGetAllScheduledPayments";
import { ScheduledPaymentOutputFields } from "../../graphql/fragments/__generated__/ScheduledPaymentOutputFields";
import { SecondaryButton } from "../../../../ui/atoms/Button";
import { Table, Tbody, Td, Th, Thead, Tr } from "../../../../ui/atoms/Table";
import { common_api_generic_query_QueryFilterOperatorEnum } from "../../../../globalTypes";
import { format } from "date-fns";
import { getAdvancedRentStatus } from "../../AdvancedRent/util";
import { usePagination } from "../../../../ui/molecules/Pagination";
import { useQuery } from "react-apollo";
import { useSortable } from "../../../../ui/molecules/Sortable";
import GET_SCHEDULE from "../../graphql/queries/PaymentsGetAllScheduledPayments.graphql";
import Pagebar from "../../../../ui/molecules/Pagebar";
import React, { FC, useState } from "react";
import SortableTh from "../../../../ui/molecules/SortableTh";
import Spinner from "../../../../ui/atoms/Spinner";
import SystemError from "../../../../ui/atoms/SystemError";
import css from "./index.module.css";
import getSortOrder from "../../../../util/ger_sort_order";

const PER_PAGE = 10;

type Props = {
  offerId: string;
};

const AdvancedRentModal: FC<Props> = ({ offerId }) => {
  const { field, order, setSort } = useSortable({
    field: ["date"],
    order: -1,
  });
  const { page, setPage, perPage } = usePagination({ perPage: PER_PAGE });
  const [showModal, setShowModal] = useState(false);

  const { data } = useQuery<
    PaymentsGetAllScheduledPayments,
    PaymentsGetAllScheduledPaymentsVariables
  >(GET_SCHEDULE, {
    variables: {
      filter: [
        {
          fields: ["offer"],
          operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
          value: [offerId],
        },
        {
          fields: ["advance.available"],
          operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
          value: ["true"],
        },
      ],
    },
  });

  const advancedCount = data?.admin?.allScheduledPayments?.total ?? 0;

  const { data: allData, loading, error } = useQuery<
    PaymentsGetAllScheduledPayments,
    PaymentsGetAllScheduledPaymentsVariables
  >(GET_SCHEDULE, {
    variables: {
      sort: field.map((v) => ({ field: v, order })),
      filter: [
        {
          fields: ["offer"],
          operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
          value: [offerId],
        },
      ],
      pagination: { page, perPage },
    },
    skip: !showModal,
  });

  const scheduledPayments = allData?.admin?.allScheduledPayments?.data ?? [];
  // console.log(scheduledPayments);

  const getOrder = (fieldName: string) =>
    getSortOrder({ field, fieldName, order });

  return (
    <>
      <SecondaryButton
        className={css.button}
        onClick={() => setShowModal(true)}
        type="button"
      >
        Advanced rent requests
        {advancedCount > 0 && <div className={css.badge}>{advancedCount}</div>}
      </SecondaryButton>

      {showModal && (
        <ModalWide
          onCancel={() => setShowModal(false)}
          title="Advanced rent requests"
        >
          <>
            <div className={css.offerId}>
              <b>Offer ID:</b> {offerId}
            </div>
            {loading ? (
              <Spinner />
            ) : error ? (
              <SystemError error={error} />
            ) : (
              <div>
                <Pagebar
                  changePage={setPage}
                  curPage={page}
                  showNPages={3}
                  total={Math.ceil(
                    (allData?.admin?.allScheduledPayments?.total ?? 0) /
                      PER_PAGE
                  )}
                />
                <Table>
                  <Thead>
                    <AdvancedRentHead
                      field={field}
                      getOrder={getOrder}
                      order={order}
                      setSort={setSort}
                    />
                  </Thead>
                  <Tbody>
                    {scheduledPayments.map((p, idx) => (
                      <AdvancedRentItem key={`${p.id}_${idx}`} payment={p} />
                    ))}
                  </Tbody>
                </Table>
                <Pagebar
                  changePage={setPage}
                  curPage={page}
                  showNPages={3}
                  total={Math.ceil(
                    (allData?.admin?.allScheduledPayments?.total ?? 0) /
                      PER_PAGE
                  )}
                />
              </div>
            )}
          </>
        </ModalWide>
      )}
    </>
  );
};

export default AdvancedRentModal;

type HeadProps = {
  order: number;
  setSort: ({ field, order }: { field: string[]; order: number }) => void;
  getOrder: (fieldName: string) => number;
  field: string | string[];
};

const AdvancedRentHead: FC<HeadProps> = ({
  order,
  setSort,
  getOrder,
  field,
}) => (
  <Tr>
    <Th>Payment ID</Th>
    <SortableTh
      handleClick={() =>
        setSort({
          field: ["date"],
          order: getOrder("date"),
        })
      }
      include={field.includes("date")}
      order={order}
    >
      Month
    </SortableTh>
    <Th>Status</Th>
    <Th>Details</Th>
  </Tr>
);

const AdvancedRentItem: FC<{ payment: ScheduledPaymentOutputFields }> = ({
  payment,
}) => (
  <Tr>
    <Td>{payment.id}</Td>
    <Td>
      {format(new Date(payment.date.year, payment.date.month - 1), "MMMM")}
    </Td>
    <Td>{getAdvancedRentStatus(payment.advance.state)}</Td>
    <Td>
      <Link
        className="link"
        target="_blank"
        to={{
          pathname: `/advanced/${payment.id}`,
          search: `offer=${payment.offer?.id}`,
        }}
      >
        View -&gt;
      </Link>
    </Td>
  </Tr>
);
