import {
  ColumnTitle,
  ColumnValue,
  PaymentRow,
} from "../../../../ui/atoms/Payment";
import {
  DepositSelect,
  DepositSelectVariables,
} from "../../../offers/tabs/TabFinalPayment/graphql/query/__generated__/DepositSelect";
import { DocumentItem, DocumentItemDate } from "../../../../ui/atoms/Offer";
import {
  GetMandate,
  GetMandateVariables,
} from "../../../offers/tabs/TabFinalPayment/graphql/query/__generated__/GetMandate";
import {
  PaymentsDeleteDocument,
  PaymentsDeleteDocumentVariables,
} from "../../graphql/mutations/__generated__/PaymentsDeleteDocument";
import {
  PaymentsGetAllDocuments,
  PaymentsGetAllDocumentsVariables,
} from "../../graphql/queries/__generated__/PaymentsGetAllDocuments";
import {
  PaymentsGetInvoice,
  PaymentsGetInvoiceVariables,
} from "../../graphql/queries/__generated__/PaymentsGetInvoice";
import { PaymentsGetOfferForTenancy_admin_Offer } from "../../graphql/queries/__generated__/PaymentsGetOfferForTenancy";
import { PrimaryButton } from "../../../../ui/atoms/Button";
import { document_api_DocumentTypeEnum } from "../../../../globalTypes";
import { formatDateTime, formatDateWordTime } from "../../../../util/date";
import { formatOrdinal } from "../../../../util/ordinal";
import { formatPrice } from "../../../../util/price";
import { sentenceCase } from "sentence-case";
import { useMutation, useQuery } from "react-apollo";
import DELETE_DOCUMENT from "../../graphql/mutations/PaymentsDeleteDocument.graphql";
import DEPOSIT_SELECT from "../../../offers/tabs/TabFinalPayment/graphql/query/DepositSelect.graphql";
import GET_ALL_DOCUMENTS from "../../graphql/queries/PaymentsGetAllDocuments.graphql";
import GET_INVOICE from "../../graphql/queries/PaymentsGetInvoice.graphql";
import GET_MANDATE from "../../../offers/tabs/TabFinalPayment/graphql/query/GetMandate.graphql";
import React, { FC, useState } from "react";
import Spinner from "../../../../ui/atoms/Spinner";
import SystemError from "../../../../ui/atoms/SystemError";
import UploadRegistrationModal from "../UploadRegistrationModal";
import css from "./index.module.css";

type Props = {
  isDRP: boolean;
  propertyId: string;
  offer: PaymentsGetOfferForTenancy_admin_Offer;
};

const DepositRegistration: FC<Props> = ({ isDRP, propertyId, offer }) => {
  const { data, loading, error, refetch } = useQuery<
    PaymentsGetAllDocuments,
    PaymentsGetAllDocumentsVariables
  >(GET_ALL_DOCUMENTS, {
    variables: {
      property: propertyId,
      removed: false,
      types: [document_api_DocumentTypeEnum.DEPOSIT_REGISTRATION_CERTIFICATE],
    },
    skip: isDRP,
  });

  const documents = data?.admin?.allDocuments?.documents ?? [];

  const {
    data: depositSelectData,
    loading: depositSelectLoading,
    error: depositSelectError,
  } = useQuery<DepositSelect, DepositSelectVariables>(DEPOSIT_SELECT, {
    variables: {
      offer: offer.id,
    },
    skip: !isDRP,
  });

  const depositSelect = depositSelectData?.offer?.depositselect;
  const monthlyPayment = depositSelect?.schedule_deposit.monthly_payment;

  const {
    data: mandateData,
    loading: mandateLoading,
    error: mandateError,
  } = useQuery<GetMandate, GetMandateVariables>(GET_MANDATE, {
    variables: {
      mandateId: depositSelect?.mandate,
    },
    skip: !depositSelect || !isDRP || depositSelect.mandate === "",
  });

  const mandate = mandateData?.offer?.getmandate?.mandate;

  const {
    data: directDebitInvoiceData,
    loading: directDebitInvoiceLoading,
    error: directDebitInvoiceError,
  } = useQuery<PaymentsGetInvoice, PaymentsGetInvoiceVariables>(GET_INVOICE, {
    variables: {
      invoice_id: depositSelect?.invoice,
    },
    skip: !depositSelect || !isDRP,
  });

  const directDebitInvoice = directDebitInvoiceData?.admin?.Invoice;

  const [showUploadModal, setShowUploadModal] = useState(false);

  const [deleteDocument, { error: deleteError }] = useMutation<
    PaymentsDeleteDocument,
    PaymentsDeleteDocumentVariables
  >(DELETE_DOCUMENT);

  function handleDelete(id: string) {
    deleteDocument({
      variables: {
        id,
      },
    }).then((res) => {
      if (res.data?.admin?.deleteDocument?.success) {
        refetch();
      }
    });
  }

  const depositLoading =
    depositSelectLoading || directDebitInvoiceLoading || mandateLoading;
  const depositError =
    depositSelectError || directDebitInvoiceError || mandateError;

  return isDRP ? (
    <>
      <h3 className={css.subTitle}>DRP Direct Debit</h3>
      {depositLoading ? (
        <Spinner />
      ) : depositError ? (
        <SystemError error={depositError} />
      ) : (
        <div className={css.blockContainer}>
          <PaymentRow>
            <div className={css.column25}>
              <ColumnTitle>Monthly fee</ColumnTitle>
              <ColumnValue>{formatPrice(monthlyPayment)}</ColumnValue>
            </div>
            <div className={css.column25}>
              <ColumnTitle>Setup date</ColumnTitle>
              <ColumnValue>
                {formatDateTime(directDebitInvoice?.created_at)}
              </ColumnValue>
            </div>
            <div className={css.column25}>
              <ColumnTitle>Monthly payment date</ColumnTitle>
              <ColumnValue>
                {formatOrdinal(offer.conditions.rent_payment_day ?? 1)} of the
                month
              </ColumnValue>
            </div>
            <div className={css.column25}>
              <ColumnTitle>Mandate status</ColumnTitle>
              <ColumnValue>
                {mandate ? sentenceCase(mandate.status) : "Unknown"}
              </ColumnValue>
            </div>
          </PaymentRow>
        </div>
      )}
    </>
  ) : (
    <>
      <h3 className={css.subTitle}>
        Deposit Registration&nbsp;&nbsp;
        <a
          href="https://secure.mydeposits.co.uk/Login.aspx?ReturnUrl=%2f"
          rel="noopener noreferrer"
          target="_blank"
        >
          MYDEPOSITS Login&nbsp;
          <i className="material-icons">open_in_new</i>
        </a>
      </h3>
      <div className={css.blockContainer}>
        <ColumnTitle>Registration Confirmation document</ColumnTitle>
        {loading ? (
          <Spinner />
        ) : error ? (
          <SystemError error={error} />
        ) : documents.length === 0 ? (
          <PrimaryButton onClick={() => setShowUploadModal(true)} type="button">
            Add document
          </PrimaryButton>
        ) : (
          documents.map((d) => (
            <DocumentItem key={d.id}>
              <div>
                <a download={d.file.name} href={`/media/${d.file.id}`}>
                  {d.file.name}
                </a>
                <i
                  className="material-icons"
                  onClick={() => handleDelete(d.id)}
                  role="button"
                  tabIndex={0}
                >
                  delete
                </i>
              </div>

              <DocumentItemDate>
                Uploaded: {formatDateWordTime(d.created_at)}
              </DocumentItemDate>
            </DocumentItem>
          ))
        )}
        {deleteError && <SystemError error={deleteError} />}
      </div>

      {showUploadModal && (
        <UploadRegistrationModal
          onClose={() => setShowUploadModal(false)}
          onComplete={() => {
            setShowUploadModal(false);
            refetch();
          }}
          propertyId={propertyId}
        />
      )}
    </>
  );
};

export default DepositRegistration;
