import {
  AddPaymentTransactionMatch,
  AddPaymentTransactionMatchVariables,
} from "../../graphql/mutations/__generated__/AddPaymentTransactionMatch";
import { PaymentsGetAllScheduledPayments_admin_allScheduledPayments_data_parts_payment as AllScheduledPayments_data_parts_payment } from "../../graphql/queries/__generated__/PaymentsGetAllScheduledPayments";
import {
  PaymentsGetAllTransactions,
  PaymentsGetAllTransactionsVariables,
  PaymentsGetAllTransactions_admin_allTransactions_transactions,
} from "../../graphql/queries/__generated__/PaymentsGetAllTransactions";
import { PrimaryButton, SecondaryButton } from "../../../../ui/atoms/Button";
import {
  ReconciliationStatusFull,
  ReconciliationStatusPartial,
  StatusBlock,
} from "../../../../ui/atoms/Status";
import { Table, Tbody, Td, Th, Thead, Tr } from "../../../../ui/atoms/Table";
import {
  common_api_generic_query_QueryFilterOperatorEnum,
  invoice_api_TransactionReconciliationEnum,
} from "../../../../globalTypes";
import { formatDateTime } from "../../../../util/date";
import { formatPrice } from "../../../../util/price";
import { sentenceCase } from "change-case";
import { useMutation, useQuery } from "react-apollo";
import { usePagination } from "../../../../ui/molecules/Pagination";
import { useSortable } from "../../../../ui/molecules/Sortable";
import ADD_TRANSACTION_MATCH from "../../graphql/mutations/AddPaymentTransactionMatch.graphql";
import GET_ALL_TRANSACTIONS from "../../graphql/queries/PaymentsGetAllTransactions.graphql";
import Pagebar from "../../../../ui/molecules/Pagebar";
import React, { FC, useState } from "react";
import Spinner from "../../../../ui/atoms/Spinner";
import SystemError from "../../../../ui/atoms/SystemError";
import TableOverlay from "../../../../ui/atoms/TableOverlay";
import TableWrapper from "../../../../ui/atoms/TableWrapper";
import css from "./index.module.css";

enum RECONCILIATION_RESULT {
  NOT_FOUND,
  PARTIAL,
  FULL,
}

const getReconciliationResultStatus = (
  payment: AllScheduledPayments_data_parts_payment
) => {
  const matches = payment.matches ?? [];
  const isFull = matches.some(
    (m) =>
      m.transaction?.reconciliation ===
      invoice_api_TransactionReconciliationEnum.TRANSACTION_RECONCILIATION_FULL
  );
  if (matches.length === 0) {
    return RECONCILIATION_RESULT.NOT_FOUND;
  } else if (isFull) {
    return RECONCILIATION_RESULT.FULL;
  } else {
    return RECONCILIATION_RESULT.PARTIAL;
  }
};

type ReconciliationProps = {
  payment: AllScheduledPayments_data_parts_payment;
  ivr: string | null;
};

const Reconciliation: FC<ReconciliationProps> = ({ payment, ivr }) => {
  const reconciliationResultStatus = getReconciliationResultStatus(payment);
  const isFULL = reconciliationResultStatus === RECONCILIATION_RESULT.FULL;
  const [showTransactions, setShowTransactions] = useState(
    reconciliationResultStatus === RECONCILIATION_RESULT.PARTIAL
  );
  const [showMatchedTransaction, setShowMatchedTransaction] = useState(isFULL);

  const { field, order } = useSortable({
    field: ["created"],
    order: -1,
  });
  const { page, setPage, perPage } = usePagination({ perPage: 10 });

  const { data, loading, error, refetch } = useQuery<
    PaymentsGetAllTransactions,
    PaymentsGetAllTransactionsVariables
  >(GET_ALL_TRANSACTIONS, {
    variables: {
      pagination: { page, perPage },
      sort: field.map((v) => ({ field: v, order })),
      filter: showMatchedTransaction
        ? [
            {
              fields: ["reconciliation"],
              operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
              value: ["TRANSACTION_RECONCILIATION_FULL"],
            },
            {
              fields: ["ivr"],
              operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
              value: [ivr ?? ""],
            },
          ]
        : [
            {
              fields: ["reconciliation"],
              operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
              value: ["TRANSACTION_RECONCILIATION_FULL"],
              not: true,
            },
            {
              fields: ["status"],
              operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
              value: ["TRANSACTION_STATUS_SUCCESS"],
            },
          ],
    },
    skip: !showTransactions,
  });

  const transactions = data?.admin?.allTransactions?.transactions ?? [];
  // console.log(transactions);

  const [
    addPaymentTransactionMatch,
    { loading: addMatchLoading, error: addMatchError },
  ] = useMutation<
    AddPaymentTransactionMatch,
    AddPaymentTransactionMatchVariables
  >(ADD_TRANSACTION_MATCH);

  const handleMatchPayment = (paymentId: string, transactionId: string) => {
    addPaymentTransactionMatch({
      variables: {
        payment_id: paymentId,
        transaction_id: transactionId,
      },
      refetchQueries: ["PaymentsGetAllScheduledPayments"],
    }).then((res) => {
      if (res.data?.admin?.addPaymentTransactionMatch?.payment) {
        refetch();
      }
    });
  };

  const onSetPage = (pageNumber: number) => setPage(pageNumber);

  return (
    <>
      <h3 className={css.subTitle}>
        Reconciliation&nbsp;&nbsp;
        {reconciliationResultStatus === RECONCILIATION_RESULT.FULL ? (
          <ReconciliationStatusFull>Reconciled</ReconciliationStatusFull>
        ) : reconciliationResultStatus === RECONCILIATION_RESULT.PARTIAL ? (
          <ReconciliationStatusPartial>
            Partial matches found
          </ReconciliationStatusPartial>
        ) : (
          <StatusBlock>No matches found</StatusBlock>
        )}
      </h3>
      <div className={css.blockContainer}>
        {reconciliationResultStatus === RECONCILIATION_RESULT.NOT_FOUND && (
          <div className={css.blockText}>
            No match found, click 'Find match manually' to look for the payment
            in the Account Statement
          </div>
        )}
        {reconciliationResultStatus === RECONCILIATION_RESULT.FULL && (
          <div className={css.blockText}>
            Full match found. If this is not the correct payment, you can click
            'Override match' to find the correct payment in the Account
            Statement
          </div>
        )}

        {showTransactions && (
          <>
            {!payment ? (
              <>No payment</>
            ) : !data && loading ? (
              <Spinner />
            ) : error ? (
              <SystemError error={error} />
            ) : (
              <TableWrapper className={css.tableWrapper}>
                {(addMatchLoading || (data && loading)) && <TableOverlay />}

                <Table>
                  <TransactionHead />
                  <Tbody>
                    {transactions.map((t) => (
                      <TransactionItem
                        key={t.id}
                        onMatch={() => handleMatchPayment(payment.id, t.id)}
                        paymentId={payment.id}
                        showMatchButton={!showMatchedTransaction}
                        transaction={t}
                      />
                    ))}
                  </Tbody>
                </Table>
                <Pagebar
                  changePage={onSetPage}
                  curPage={page}
                  showNPages={3}
                  total={Math.ceil(
                    (data?.admin?.allTransactions?.total ?? 1) / perPage
                  )}
                />
              </TableWrapper>
            )}
          </>
        )}

        {addMatchError && <SystemError error={addMatchError} />}

        {(reconciliationResultStatus === RECONCILIATION_RESULT.NOT_FOUND ||
          reconciliationResultStatus === RECONCILIATION_RESULT.PARTIAL) && (
          <div className={css.blockButtons}>
            <SecondaryButton
              onClick={() => setShowTransactions(true)}
              type="button"
            >
              Find match manually
            </SecondaryButton>
          </div>
        )}
        {isFULL && (
          <div className={css.blockButtons}>
            <SecondaryButton
              onClick={() => setShowMatchedTransaction(false)}
              type="button"
            >
              Override match
            </SecondaryButton>
          </div>
        )}
      </div>
    </>
  );
};

export default Reconciliation;

const TransactionHead: FC = () => (
  <Thead>
    <Tr>
      <Th>Received at</Th>
      <Th>Amount</Th>
      <Th>Payer</Th>
      <Th>Reference</Th>
      <Th>Matched</Th>
      <Th></Th>
    </Tr>
  </Thead>
);

const TransactionItem: FC<{
  transaction: PaymentsGetAllTransactions_admin_allTransactions_transactions;
  paymentId: string;
  onMatch: () => void;
  showMatchButton: boolean;
}> = ({ transaction, paymentId, onMatch, showMatchButton }) => {
  return (
    <Tr>
      <Td>{formatDateTime(transaction.created)}</Td>
      <Td>{formatPrice(transaction.amount)}</Td>
      <Td>{transaction.description}</Td>
      <Td>{transaction.ivr}</Td>
      <Td>
        {(transaction.matches ?? [])
          .filter((m) => m.payment?.id === paymentId)[0]
          ?.matched?.map((m) =>
            sentenceCase(m.replace("TRANSACTION_MATCH_", ""))
          )
          .join(", ")}
      </Td>
      <Td>
        {showMatchButton && (
          <PrimaryButton
            className={css.tableButton}
            onClick={onMatch}
            type="button"
          >
            Match
          </PrimaryButton>
        )}
      </Td>
    </Tr>
  );
};
