import { Field, Form } from "react-final-form";
import { OnChange } from "react-final-form-listeners";
import {
  PaymentsGetAllScheduledPayments,
  PaymentsGetAllScheduledPaymentsVariables,
} from "../graphql/queries/__generated__/PaymentsGetAllScheduledPayments";
import {
  PaymentsGetOfferForTenancy,
  PaymentsGetOfferForTenancyVariables,
} from "../graphql/queries/__generated__/PaymentsGetOfferForTenancy";
import {
  PaymentsGetProperty,
  PaymentsGetPropertyVariables,
} from "../graphql/queries/__generated__/PaymentsGetProperty";
import {
  PaymentsGetTenancies,
  PaymentsGetTenanciesVariables,
} from "../graphql/queries/__generated__/PaymentsGetTenancies";
import {
  PropertyCardItemTitle,
  PropertyHeader,
} from "../../../ui/atoms/Property";
import { common_api_generic_query_QueryFilterOperatorEnum } from "../../../globalTypes";
import { formatShortDate } from "../../../util/date";
import { useParams } from "react-router-dom";
import { useQuery } from "react-apollo";
import AdvancedRentModal from "./AdvancedRentModal";
import Dropdown from "../../../ui/molecules/Dropdown";
import GET_OFFER from "../graphql/queries/PaymentsGetOfferForTenancy.graphql";
import GET_PROPERTY from "../graphql/queries/PaymentsGetProperty.graphql";
import GET_SCHEDULE from "../graphql/queries/PaymentsGetAllScheduledPayments.graphql";
import GET_TEANANCIES from "../graphql/queries/PaymentsGetTenancies.graphql";
import LandlordInfo from "./LandlordInfo";
import Notes from "../../../components/Notes";
import Payments from "./Payments";
import PropertyHead from "./PropertyHead";
import React, { FC, useEffect, useMemo, useState } from "react";
import Spinner from "../../../ui/atoms/Spinner";
import SystemError from "../../../ui/atoms/SystemError";
import css from "./index.module.css";

type Option = { label: string; value: string };

interface FormValues {
  tenancies?: Option;
}

const PropertyPayments: FC = () => {
  const { propertyId } = useParams();
  const [currentOfferId, setCurrentOfferId] = useState<string | null>(null);

  const {
    data: propertyData,
    loading: propertyLoading,
    error: propertyError,
  } = useQuery<PaymentsGetProperty, PaymentsGetPropertyVariables>(
    GET_PROPERTY,
    {
      variables: {
        id: propertyId,
      },
      skip: propertyId == null,
    }
  );

  const property = propertyData?.admin?.Property;

  const {
    data: tenanciesData,
    loading: tenanciesLoading,
    error: tenanciesError,
  } = useQuery<PaymentsGetTenancies, PaymentsGetTenanciesVariables>(
    GET_TEANANCIES,
    {
      variables: {
        filter: [
          {
            fields: ["property_id"],
            value: propertyId,
            operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
          },
        ],
        sort: [
          {
            field: "start_date",
            order: -1,
          },
        ],
      },
      skip: propertyId == null,
    }
  );

  const tenancies = useMemo(
    () =>
      (tenanciesData?.admin?.allTenancies?.data ?? []).filter(
        (t) => t.offer != null && t.offer !== ""
      ),
    [tenanciesData]
  );

  const tenanciesOptions: Option[] = useMemo(
    () =>
      tenancies.map((t) => ({
        label: `${formatShortDate(t.movedin, "/", "N/A")} – ${formatShortDate(
          t.movedout,
          "/",
          "N/A"
        )}`,
        value: t.offer!,
      })),
    [tenancies]
  );

  const initialValues: FormValues = useMemo(
    () => ({
      tenancies: tenanciesOptions[0],
    }),
    [tenanciesOptions]
  );

  useEffect(() => {
    if (tenancies.length > 0 && currentOfferId == null) {
      setCurrentOfferId(tenancies[0].offer);
    }
  }, [tenancies, currentOfferId]);

  const {
    data: offerData,
    loading: offerLoading,
    error: offerError,
  } = useQuery<PaymentsGetOfferForTenancy, PaymentsGetOfferForTenancyVariables>(
    GET_OFFER,
    {
      variables: {
        id: currentOfferId,
      },
      skip: currentOfferId == null,
    }
  );

  const offer = offerData?.admin?.Offer;

  const {
    data: scheduledData,
    loading: scheduledLoading,
    error: scheduledError,
  } = useQuery<
    PaymentsGetAllScheduledPayments,
    PaymentsGetAllScheduledPaymentsVariables
  >(GET_SCHEDULE, {
    variables: {
      filter: [
        {
          fields: ["offer"],
          operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
          value: [offer?.id ?? ""],
        },
      ],
      pagination: {
        page: 0,
        perPage: 1000,
      },
    },
    skip: offer == null,
  });

  const scheduledPayments =
    scheduledData?.admin?.allScheduledPayments?.data ?? [];

  const hasError = tenanciesError || offerError || scheduledError;

  if (propertyId == null) {
    return null;
  }

  return (
    <>
      <PropertyHeader>
        {propertyLoading ? (
          <Spinner />
        ) : propertyError ? (
          <SystemError error={propertyError} />
        ) : property ? (
          <>
            <PropertyHead offer={offer} property={property} />
            <div className={css.headColumn}>
              <Form<FormValues>
                initialValues={initialValues}
                onSubmit={() => {}}
              >
                {({ handleSubmit }) => (
                  <form onSubmit={handleSubmit}>
                    <PropertyCardItemTitle>Tenancies</PropertyCardItemTitle>
                    <div className={css.dropDownContainer}>
                      <Field
                        name="tenancies"
                        render={({ input }) => (
                          <Dropdown
                            disabled={tenanciesLoading}
                            input={input}
                            options={tenanciesOptions}
                          />
                        )}
                      />
                    </div>

                    <OnChange name="tenancies">
                      {(e: Option | undefined) => {
                        if (e) {
                          setCurrentOfferId(e.value);
                        }
                      }}
                    </OnChange>
                  </form>
                )}
              </Form>

              <Notes
                buttonClassName={css.notesButton}
                propertyId={property.id}
                title="Property Notes"
              />
              {offer?.id && <AdvancedRentModal offerId={offer.id} />}
            </div>
            <LandlordInfo landlord={property.people.landlord} />
          </>
        ) : null}
      </PropertyHeader>

      {hasError && <SystemError error={hasError} />}

      {offerLoading || (!scheduledData && scheduledLoading) ? (
        <Spinner />
      ) : offer && property ? (
        <Payments
          offer={offer}
          property={property}
          scheduledPayments={scheduledPayments}
        />
      ) : null}
    </>
  );
};

export default PropertyPayments;
