import {
  Fetch_User_Invoices,
  Fetch_User_InvoicesVariables,
} from "./__generated__/Fetch_User_Invoices";
import {
  Fetch_User_Offers,
  Fetch_User_OffersVariables,
} from "./__generated__/Fetch_User_Offers";
import {
  Fetch_User_Properties,
  Fetch_User_PropertiesVariables,
} from "./__generated__/Fetch_User_Properties";
import { Link } from "react-router-dom";
import { Table, TdInner, ThInner } from "../../../ui/atoms/Table";
import { common_api_generic_query_QueryFilterOperatorEnum } from "../../../globalTypes";
import {
  fetch_user_Maintenance,
  fetch_user_MaintenanceVariables,
} from "./__generated__/fetch_user_Maintenance";
import { formatDate } from "../../../util/date";
import { formatPrice } from "../../../util/price";
import { sentenceCase } from "change-case";
import { useQuery } from "react-apollo";
import CopyText from "../../../ui/molecules/CopyToClipboard";
import React, { FC } from "react";
import Spinner from "../../../ui/atoms/Spinner";
import gql from "graphql-tag";

type Props = {
  userId: string;
};

const Associated: FC<Props> = ({ userId }) => {
  const maintenanceRest = useQuery<
    fetch_user_Maintenance,
    fetch_user_MaintenanceVariables
  >(FETCH_USER_MAINTENANCE, {
    variables: {
      pagination: {
        page: 1,
        perPage: 100,
      },
      filter: [
        {
          operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
          fields: ["user", "landlord", "tenants"],
          value: [userId as string],
        },
      ],
    },
  });

  const invoicesRest = useQuery<
    Fetch_User_Invoices,
    Fetch_User_InvoicesVariables
  >(FETCH_USER_INVOICES, {
    variables: {
      pagination: {
        page: 1,
        perPage: 100,
      },
      filter: [
        {
          operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
          fields: ["user"],
          value: [userId as string],
        },
      ],
    },
  });

  const offersRest = useQuery<Fetch_User_Offers, Fetch_User_OffersVariables>(
    FETCH_USER_OFFERS,
    {
      variables: {
        pagination: {
          page: 1,
          perPage: 100,
        },
        filter: [
          {
            operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
            fields: ["landlord.user", "tenants.user"],
            value: [userId as string],
          },
        ],
      },
    }
  );

  const propertyRest = useQuery<
    Fetch_User_Properties,
    Fetch_User_PropertiesVariables
  >(FETCH_USER_PROPERTIES, {
    variables: {
      pagination: {
        page: 1,
        perPage: 100,
      },
      filter: [
        {
          operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
          fields: ["people.landlord", "people.manager", "people.submitter"],
          value: [userId as string],
        },
      ],
    },
  });

  return (
    <>
      <h3>Maintenance</h3>
      <Table>
        <thead>
          <tr>
            <ThInner>Title</ThInner>
            <ThInner>Place</ThInner>
            <ThInner>Created At</ThInner>
            <ThInner>Repair Date</ThInner>
            <ThInner>Status</ThInner>
            <ThInner>ID</ThInner>
          </tr>
        </thead>
        <tbody>
          {maintenanceRest?.loading ? (
            <tr>
              <TdInner colSpan={5}>
                <Spinner size="md" />
              </TdInner>
            </tr>
          ) : (
            <>
              {maintenanceRest?.data?.admin?.allMaintenances?.data?.map(
                ({ id, created_at, place, status, title, repair_date }) => (
                  <tr key={id}>
                    <TdInner>{title}</TdInner>
                    <TdInner>
                      <b>{sentenceCase(place)}</b>
                    </TdInner>
                    <TdInner>{formatDate(created_at)}</TdInner>
                    <TdInner>{formatDate(repair_date)}</TdInner>
                    <TdInner>
                      <b>{sentenceCase(status)}</b>
                    </TdInner>
                    <TdInner>
                      <CopyText text="Copy ID" textForCopy={id} />
                    </TdInner>
                  </tr>
                )
              )}
            </>
          )}
        </tbody>
      </Table>

      <h3>Invoices</h3>
      <Table>
        <thead>
          <tr>
            <ThInner>Amount</ThInner>
            <ThInner>Created At</ThInner>
            <ThInner>Invoice ID</ThInner>
            <ThInner>Type</ThInner>
            <ThInner>Status</ThInner>
          </tr>
        </thead>
        <tbody>
          {invoicesRest?.loading ? (
            <tr>
              <TdInner colSpan={5}>
                <Spinner size="md" />
              </TdInner>
            </tr>
          ) : (
            <>
              {invoicesRest?.data?.admin?.allInvoices?.invoices?.map(
                ({ amount, created_at, invoice_id, status, type }) => (
                  <tr key={invoice_id}>
                    <TdInner>{formatPrice(amount)}</TdInner>
                    <TdInner>{formatDate(created_at)}</TdInner>
                    <TdInner>
                      See&nbsp;invoice&nbsp;&mdash;&nbsp;
                      <Link
                        key={invoice_id}
                        className="link"
                        to={`/invoices/details?invoiceId=${invoice_id}`}
                      >
                        {invoice_id}
                      </Link>
                    </TdInner>
                    <TdInner>{sentenceCase(type)}</TdInner>
                    <TdInner>
                      <b>{sentenceCase(status)}</b>
                    </TdInner>
                  </tr>
                )
              )}
            </>
          )}
        </tbody>
      </Table>

      <h3>Offers</h3>
      <Table>
        <thead>
          <tr>
            <ThInner>Address</ThInner>
            <ThInner>Offers</ThInner>
            <ThInner>Price</ThInner>
            <ThInner>Move In</ThInner>
            <ThInner>Status</ThInner>
          </tr>
        </thead>
        <tbody>
          {offersRest?.loading ? (
            <tr>
              <TdInner colSpan={5}>
                <Spinner size="md" />
              </TdInner>
            </tr>
          ) : (
            <>
              {offersRest?.data?.admin?.allOffers?.data?.map(
                ({
                  id,
                  property,
                  state,
                  conditions: { price, movein_date },
                }) => (
                  <tr key={id}>
                    <TdInner>
                      See property&nbsp;&mdash;&nbsp;
                      <Link
                        className="link"
                        to={`/listing/${property?.id}/edit`}
                      >
                        &hellip;{property?.id.slice(-12)}
                      </Link>
                      <br />
                      {property?.location.address}
                      <br />
                      {property?.location.city && (
                        <b>{property?.location.city}</b>
                      )}
                      {property?.location.postcode && (
                        <>
                          &nbsp;&nbsp;<b>{property?.location.postcode}</b>
                        </>
                      )}
                    </TdInner>
                    <TdInner>
                      See&nbsp;offer&nbsp;&mdash;&nbsp;
                      <Link key={id} className="link" to={`/offers/${id}/edit`}>
                        &hellip;&nbsp;{id.slice(-12)}
                      </Link>
                    </TdInner>
                    <TdInner>{formatPrice(price)}</TdInner>
                    <TdInner>{formatDate(movein_date)}</TdInner>
                    <TdInner>
                      <b>{sentenceCase(state)}</b>
                    </TdInner>
                  </tr>
                )
              )}
            </>
          )}
        </tbody>
      </Table>

      <h3>Properties</h3>
      <Table>
        <thead>
          <tr>
            <ThInner>Address</ThInner>
            <ThInner>Status</ThInner>
            <ThInner>Landlord</ThInner>
            <ThInner>Manager</ThInner>
            <ThInner>Tenants</ThInner>
          </tr>
        </thead>
        <tbody>
          {propertyRest?.loading ? (
            <tr>
              <TdInner colSpan={5}>
                <Spinner size="md" />
              </TdInner>
            </tr>
          ) : (
            propertyRest?.data?.admin?.allProperties?.data?.map(
              ({
                id,
                location,
                status,
                people: { landlord, manager, tenants },
              }) => {
                return (
                  <tr key={id}>
                    <TdInner>
                      See property&nbsp;&mdash;&nbsp;
                      <Link className="link" to={`/listing/${id}/edit`}>
                        &hellip;{id.slice(-12)}
                      </Link>
                      <br />
                      {location.address}
                      <br />
                      {location.city && <b>{location.city}</b>}
                      {location.postcode && (
                        <>
                          &nbsp;&nbsp;<b>{location.postcode}</b>
                        </>
                      )}
                    </TdInner>
                    <TdInner>
                      <b>{sentenceCase(status)}</b>
                    </TdInner>
                    <TdInner>
                      <Link className="link" to={`/users/${landlord?.id}/view`}>
                        {landlord?.details?.first_name}
                        &nbsp;
                        {landlord?.details?.last_name}
                      </Link>
                    </TdInner>
                    <TdInner>
                      {manager?.id ? (
                        <Link
                          className="link"
                          to={`/users/${manager?.id}/view`}
                        >
                          {manager?.details?.first_name}
                          &nbsp;
                          {manager?.details?.last_name}
                        </Link>
                      ) : (
                        "- - -"
                      )}
                    </TdInner>
                    <TdInner>
                      {tenants.length > 0
                        ? tenants.map(({ user }) => (
                            <Link
                              key={user?.id}
                              className="link"
                              to={`/users/${user?.id}/view`}
                            >
                              {user?.details?.first_name}
                              &nbsp;
                              {user?.details?.last_name}
                            </Link>
                          ))
                        : "- - -"}
                    </TdInner>
                  </tr>
                );
              }
            )
          )}
        </tbody>
      </Table>
    </>
  );
};

const FETCH_USER_MAINTENANCE = gql`
  query fetch_user_Maintenance(
    $pagination: gateway_impl_resolvers_admin_allParamsPaginationInput
    $sort: [common_api_generic_query_QueryParamsSortInput!]
    $filter: [common_api_generic_query_QueryParamsFilterInput!]
  ) {
    admin {
      allMaintenances(pagination: $pagination, sort: $sort, filter: $filter) {
        data {
          id
          created_at
          place
          status
          title
          repair_date
        }
      }
    }
  }
`;

const FETCH_USER_INVOICES = gql`
  query Fetch_User_Invoices(
    $pagination: gateway_impl_resolvers_admin_allParamsPaginationInput
    $sort: [common_api_generic_query_QueryParamsSortInput!]
    $filter: [common_api_generic_query_QueryParamsFilterInput!]
  ) {
    admin {
      allInvoices(pagination: $pagination, sort: $sort, filter: $filter) {
        invoices {
          amount
          created_at
          invoice_id
          status
          type
        }
      }
    }
  }
`;

const FETCH_USER_OFFERS = gql`
  query Fetch_User_Offers(
    $pagination: gateway_impl_resolvers_admin_allParamsPaginationInput
    $sort: [common_api_generic_query_QueryParamsSortInput!]
    $filter: [common_api_generic_query_QueryParamsFilterInput!]
  ) {
    admin {
      allOffers(pagination: $pagination, sort: $sort, filter: $filter) {
        data {
          id
          state
          conditions {
            price
            movein_date
          }
          property {
            id
            location {
              address
              city
              postcode
            }
          }
        }
      }
    }
  }
`;

const FETCH_USER_PROPERTIES = gql`
  query Fetch_User_Properties(
    $pagination: gateway_impl_resolvers_admin_allParamsPaginationInput
    $sort: [common_api_generic_query_QueryParamsSortInput!]
    $filter: [common_api_generic_query_QueryParamsFilterInput!]
  ) {
    admin {
      allProperties(pagination: $pagination, sort: $sort, filter: $filter) {
        data {
          id
          location {
            address
            city
            postcode
          }
          status
          people {
            landlord {
              id
              details {
                first_name
                last_name
              }
            }
            manager {
              id
              details {
                first_name
                last_name
              }
            }
            submitter {
              id
              details {
                first_name
                last_name
              }
            }
            tenants {
              user {
                id
                details {
                  first_name
                  last_name
                }
              }
            }
          }
        }
      }
    }
  }
`;

export default Associated;
