import {
  ColumnNarrow,
  ColumnTitle,
  ColumnValue,
  ColumnWide,
  PaymentAmount,
  PaymentBody,
  PaymentClose,
  PaymentDescription,
  PaymentHead,
  PaymentOpen,
  Status,
} from "../../../ui/atoms/Payment";
import {
  FetchUserPayments,
  FetchUserPaymentsVariables,
} from "./__generated__/FetchUserPayments";
import { UserFormGroup } from "../../../ui/atoms/UserForm";
import {
  cardpayment_api_PaymentStatusEnum,
  common_api_generic_query_QueryFilterOperatorEnum,
} from "../../../globalTypes";
import { humanizeEnum } from "../../../util/enums_converter";
import { useQuery } from "react-apollo";
import AdminInfo from "../../payments/AdminInfo";
import Collapse from "../../../ui/molecules/Collapse";
import PropertyInfo from "../../payments/PropertyInfo";
import React, { FC } from "react";
import Spinner from "../../../ui/atoms/Spinner";
import css from "./index.module.css";
import format from "date-fns/format";
import gql from "graphql-tag";

type Props = {
  userId: string;
};

const UserPayments: FC<Props> = ({ userId }) => {
  const paymentsData = useQuery<FetchUserPayments, FetchUserPaymentsVariables>(
    FETCH_USER_PAYMENTS,
    {
      variables: {
        pagination: {
          page: 1,
          perPage: 100,
        },
        filter: [
          {
            operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
            fields: ["user"],
            value: [userId as string],
          },
        ],
      },
    }
  );

  const payments = paymentsData.data?.admin?.allCardpayments;

  return (
    <UserFormGroup>
      <h3>Payments</h3>
      {paymentsData.loading ? (
        <div className={css.loader}>
          <Spinner />
        </div>
      ) : paymentsData && paymentsData.data ? (
        <>
          {(payments?.total as number) > 0 ? (
            <div style={{ margin: "0 -20px" }}>
              {(payments?.data ?? []).map((payment) => {
                if (!payment.id) {
                  return null;
                }

                return (
                  <Collapse key={payment.id} opened={false}>
                    {({ toggle, isOpened }) => (
                      <>
                        <PaymentHead>
                          {isOpened ? (
                            <PaymentClose onClick={toggle} />
                          ) : (
                            <PaymentOpen onClick={toggle} />
                          )}
                          <PaymentDescription>
                            <b>{humanizeEnum(payment.type || "").label}</b>
                          </PaymentDescription>
                          <PaymentAmount>&pound;{payment.price}</PaymentAmount>
                          <Status
                            status={
                              payment.charge_status ===
                                cardpayment_api_PaymentStatusEnum.CANCELLED ||
                              payment.charge_status ===
                                cardpayment_api_PaymentStatusEnum.FAILED
                                ? "danger"
                                : payment.charge_status ===
                                  cardpayment_api_PaymentStatusEnum.PROCESSING
                                ? "warning"
                                : payment.charge_status ===
                                  cardpayment_api_PaymentStatusEnum.SUCCEEDED
                                ? "success"
                                : payment.charge_status ===
                                  cardpayment_api_PaymentStatusEnum.UNKNOWN_PAYMENT_STATUS
                                ? "default"
                                : payment.charge_status ===
                                  cardpayment_api_PaymentStatusEnum.UNPAID
                                ? "info"
                                : "default"
                            }
                          />
                        </PaymentHead>
                        {isOpened && (
                          <PaymentBody>
                            <ColumnWide>
                              <ColumnTitle>Payment description</ColumnTitle>
                              <ColumnValue>
                                {payment.description || "- - -"}
                              </ColumnValue>
                            </ColumnWide>
                            <ColumnNarrow>
                              <ColumnTitle>Status</ColumnTitle>
                              <ColumnValue>
                                <b>
                                  {
                                    humanizeEnum(payment.charge_status || "")
                                      .label
                                  }
                                </b>
                              </ColumnValue>
                            </ColumnNarrow>
                            <ColumnWide>
                              <ColumnTitle>Property id</ColumnTitle>
                              <ColumnValue>
                                <PropertyInfo propertyId={payment.property} />
                              </ColumnValue>
                            </ColumnWide>
                            <ColumnNarrow>
                              <ColumnTitle>Created at</ColumnTitle>
                              <ColumnValue>
                                {payment.created_at
                                  ? format(
                                      new Date(payment.created_at),
                                      "dd MMM yyyy"
                                    )
                                  : "- - -"}
                                <br />
                                {payment.created_at
                                  ? format(
                                      new Date(payment.created_at),
                                      "HH:mm:ss"
                                    )
                                  : "- - -"}
                              </ColumnValue>
                            </ColumnNarrow>
                            <ColumnWide>
                              <ColumnTitle>ID</ColumnTitle>
                              <ColumnValue>
                                <b>{payment.id}</b>
                              </ColumnValue>
                            </ColumnWide>
                            <ColumnNarrow>
                              <ColumnTitle>Invoice number</ColumnTitle>
                              <ColumnValue>
                                {payment.invoice_number}
                              </ColumnValue>
                            </ColumnNarrow>
                            {payment?.confirmed ? (
                              <>
                                <ColumnWide>
                                  <ColumnTitle>Confirmed by</ColumnTitle>
                                  <ColumnValue>
                                    <AdminInfo adminId={payment?.admin} />
                                  </ColumnValue>
                                </ColumnWide>
                                <ColumnNarrow>
                                  <ColumnTitle>Confirmed At</ColumnTitle>
                                  <ColumnValue>
                                    {format(
                                      new Date(payment?.confirmed_at),
                                      "dd MMM yyyy HH:mm:ss"
                                    )}
                                  </ColumnValue>
                                </ColumnNarrow>
                              </>
                            ) : (
                              <b>Paymeny wasn't confirmed</b>
                            )}
                          </PaymentBody>
                        )}
                      </>
                    )}
                  </Collapse>
                );
              })}
            </div>
          ) : (
            <>User haven't made payments yet.</>
          )}
        </>
      ) : (
        "N/A"
      )}
    </UserFormGroup>
  );
};

const FETCH_USER_PAYMENTS = gql`
  query FetchUserPayments(
    $pagination: gateway_impl_resolvers_admin_allParamsPaginationInput
    $sort: [common_api_generic_query_QueryParamsSortInput!]
    $filter: [common_api_generic_query_QueryParamsFilterInput!]
  ) {
    admin {
      allCardpayments(pagination: $pagination, sort: $sort, filter: $filter) {
        total
        data {
          charge_id
          charge_status
          charge_type
          admin
          confirmed
          confirmed_at
          created_at
          description
          id
          invoice_number
          price
          property
          role
          type
          updated_at
        }
      }
    }
  }
`;

export default UserPayments;
