import {
  GetTenancyOffer,
  GetTenancyOfferVariables,
} from "./graphql/__generated__/GetTenancyOffer";
import {
  GetTenancyProperty,
  GetTenancyPropertyVariables,
} from "./graphql/__generated__/GetTenancyProperty";
import { Link } from "react-router-dom";
import {
  StatusActive,
  StatusListed,
  StatusUnknown,
  StatusVerifying,
} from "../../ui/atoms/Status";
import { Td, Tr } from "../../ui/atoms/Table";
import { addDays } from "date-fns";
import { formatDate } from "../../util/date";
import {
  invoice_api_TransactionReconciliationEnum,
  property_api_TenancyStateEnum,
} from "../../globalTypes";
import { useQuery } from "react-apollo";
import FETCH_OFFER from "./graphql/GetTenancyOffer.graphql";
import FETCH_PROPERTY from "./graphql/GetTenancyProperty.graphql";
import React, { FC, useMemo } from "react";
import Spinner from "../../ui/atoms/Spinner";
import SystemError from "../../ui/atoms/SystemError";

type Props = {
  propertyId: string | null;
  movedIn: string;
  movedOut: string;
  status: property_api_TenancyStateEnum;
  tenantName: string[];
  tenantId: string | undefined;
  offerId: string | null;
};

const ListItem: FC<Props> = ({
  propertyId,
  movedIn,
  movedOut,
  tenantName,
  tenantId,
  status,
  offerId,
}) => {
  const { data, loading, error } = useQuery<
    GetTenancyProperty,
    GetTenancyPropertyVariables
  >(FETCH_PROPERTY, {
    variables: {
      id: propertyId,
    },
    skip: propertyId == null,
  });

  const {
    data: offerData,
    loading: offerLoading,
    error: offerError,
  } = useQuery<GetTenancyOffer, GetTenancyOfferVariables>(FETCH_OFFER, {
    variables: {
      id: offerId,
    },
    skip: offerId == null,
  });
  const location = data?.admin?.Property?.location;

  const offer = offerData?.admin?.Offer;
  const landlord = offer?.landlord;
  const invoices = offer?.invoices;
  const rentPaymentDate = offer?.conditions?.first_rent_payment_date
    ? new Date(offer?.conditions?.first_rent_payment_date)
    : movedIn
    ? addDays(new Date(movedIn), 3)
    : null;

  const filteredTransactionsIds = useMemo(() => {
    const result: string[] = [];

    // eslint-disable-next-line no-unused-expressions
    invoices?.forEach(({ payments }) => {
      // eslint-disable-next-line no-unused-expressions
      payments?.forEach(({ matches }) => {
        // eslint-disable-next-line no-unused-expressions
        matches?.forEach(({ transaction }) => {
          if (
            transaction?.id &&
            transaction?.reconciliation !==
              invoice_api_TransactionReconciliationEnum.UNKNOWN_TRANSACTION_RECONCILIATION
          ) {
            result.push(transaction?.id);
          }
        });
      });
    });

    return result;
  }, [invoices]);

  return (
    <Tr>
      <Td colSpan={3}>
        {loading ? (
          <Spinner size="sm" />
        ) : error ? (
          <SystemError error={error} />
        ) : (
          <>
            {location?.address1}
            <br />
            <b>{location?.city}</b>
            <br />
            <b>{location?.postcode}</b>
          </>
        )}
      </Td>
      <Td colSpan={2}>
        {offerLoading ? (
          <Spinner size="sm" />
        ) : offerError ? (
          <SystemError error={offerError} />
        ) : (
          <Link
            className="link"
            target="_blank"
            to={`/users/${landlord?.user}/view`}
          >
            {landlord?.first_name} {landlord?.last_name}
          </Link>
        )}
      </Td>
      <Td colSpan={2}>
        {tenantId ? (
          <Link className="link" target="_blank" to={`/users/${tenantId}/view`}>
            {tenantName.join(" ")}
          </Link>
        ) : (
          <span>{tenantName.join(" ")}</span>
        )}
      </Td>
      <Td colSpan={2}>
        In&nbsp;&ndash;&nbsp;
        <b>{formatDate(movedIn)}</b>
        <br />
        Out&nbsp;&ndash;&nbsp;
        <b>{formatDate(movedOut)}</b>
      </Td>
      <Td colSpan={2}>
        {offerLoading ? (
          <Spinner size="sm" />
        ) : offerError ? (
          <SystemError error={offerError} />
        ) : (
          <>{formatDate(rentPaymentDate)}</>
        )}
      </Td>
      <Td colSpan={2}>
        {offerLoading ? (
          <Spinner size="sm" />
        ) : offerError ? (
          <SystemError error={offerError} />
        ) : offer?.contract ? (
          <a
            className="link"
            download
            href={`/media/${offer.contract}`}
            rel="noopener noreferrer"
            target="_blank"
          >
            Download AST document
          </a>
        ) : null}
      </Td>
      <Td colSpan={2}>
        {propertyId && (
          <Link className="link" to={`/property-payments/${propertyId}`}>
            View property payments
          </Link>
        )}
        {offerLoading ? (
          <Spinner size="sm" />
        ) : offerError ? (
          <SystemError error={offerError} />
        ) : (
          <>
            {filteredTransactionsIds.length > 0 &&
              filteredTransactionsIds.map((id) => (
                <React.Fragment key={id}>
                  <br />
                  <Link
                    className="link"
                    target="_blank"
                    to={`/statement/${id}`}
                  >
                    See transaction
                  </Link>
                </React.Fragment>
              ))}
          </>
        )}
      </Td>
      <Td colSpan={2}>
        {status === property_api_TenancyStateEnum.TENANCY_ACTIVE ? (
          <StatusActive>Active</StatusActive>
        ) : status === property_api_TenancyStateEnum.TENANCY_CLOSED ? (
          <StatusListed>Closed</StatusListed>
        ) : status === property_api_TenancyStateEnum.TENANCY_TERMINATING ? (
          <StatusVerifying>Terminating</StatusVerifying>
        ) : (
          <StatusUnknown>Unknown</StatusUnknown>
        )}
      </Td>
    </Tr>
  );
};

export default ListItem;
