import { Field, Form } from "react-final-form";
import {
  FilterSort,
  FilterSortItem,
  FilterSortItemInner,
} from "../../ui/atoms/FilterSort";
import {
  GetTransactions,
  GetTransactionsVariables,
} from "./graphql/__generated__/GetTransactions";
import {
  common_api_generic_query_QueryFilterOperatorEnum as OperatorEnum,
  common_api_generic_query_QueryParamsFilterInput,
  invoice_api_TransactionReconciliationEnum,
  invoice_api_TransactionTypeEnum,
} from "../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import { Query } from "react-apollo";
import { SearchFieldWide } from "../../ui/atoms/SearchField";
import { Table, Tbody, Thead } from "../../ui/atoms/Table";
import { TextInput } from "../../ui/atoms/Input";
import { makeFilters } from "../../util/search_filters";
import { usePagination } from "../../ui/molecules/Pagination";
import Dropdown from "../../ui/molecules/Dropdown";
import FETCH_TRANSACTIONS from "./graphql/GetTransactions.graphql";
import ListHead from "./ListHead";
import ListItem from "./ListItem";
import Pagebar from "../../ui/molecules/Pagebar";
import React, { FC, useState } from "react";
import Sortable from "../../ui/molecules/Sortable";
import Spinner from "../../ui/atoms/Spinner";
import SystemError from "../../ui/atoms/SystemError";
import TableWrapper from "../../ui/atoms/TableWrapper";

const PER_PAGE = 30;

type FormValues = {
  postcode: string | undefined;
  type: {
    label: string;
    value: string;
  };
  search: string | undefined;
};

const TYPE_OPTIONS = [
  {
    label: "Credit",
    value: invoice_api_TransactionTypeEnum.TRANSACTION_TYPE_CREDIT,
  },
  {
    label: "Debit",
    value: invoice_api_TransactionTypeEnum.TRANSACTION_TYPE_DEBIT,
  },
  {
    label: "All",
    value: "All",
  },
];

const getFilters = (values: FormValues) => {
  return makeFilters({
    values,
    filters: [
      {
        fields: ["ivr", "description", "amount"],
        key: "search",
        operator: OperatorEnum.CONTAINS,
      },
      {
        fields: ["type"],
        key: "type.value",
        operator: OperatorEnum.EQUALS,
      },
    ],
  });
};

const List: FC = () => {
  const basicFilter = [
    {
      fields: ["reconciliation"],
      operator: OperatorEnum.EQUALS,
      not: true,
      value: [
        invoice_api_TransactionReconciliationEnum.TRANSACTION_RECONCILIATION_FULL,
      ],
    },
    {
      fields: ["type"],
      operator: OperatorEnum.EQUALS,
      value: [invoice_api_TransactionTypeEnum.TRANSACTION_TYPE_CREDIT],
    },
  ];
  const [filter, setFilter] = useState<
    common_api_generic_query_QueryParamsFilterInput[]
  >(basicFilter);
  const { page, setPage, perPage } = usePagination({
    perPage: PER_PAGE,
    filter,
  });
  const handleFormSubmit = (values: FormValues) => {
    const filters = getFilters(values);
    if (values.type.value === "All") {
      setFilter((state) => [
        ...state.filter(({ fields }) => !fields?.includes("type")),
        ...(filters.filter(({ fields }) => !fields?.includes("type")) ?? []),
      ]);
    } else {
      setFilter((state) => {
        return filters
          .map(({ fields }) => fields)
          .flat()
          .includes("type")
          ? [
              ...state.filter(({ fields }) => !fields?.includes("type")),
              ...filters,
            ]
          : [...state, ...filters];
      });
    }
  };

  const handleFilterReset = (resetForm: () => void) => {
    resetForm();
    setFilter(basicFilter);
  };
  const filterNotCredit = !filter
    .find(({ fields }) => fields?.includes("type"))
    ?.value?.includes(invoice_api_TransactionTypeEnum.TRANSACTION_TYPE_CREDIT);
  const isFiltersExist = Object.keys(filter).length > 2 || filterNotCredit;

  return (
    <>
      <h2>Unmatched Transactions</h2>

      <Form<FormValues>
        onSubmit={handleFormSubmit}
        render={({ form, values, submitting }) => {
          return (
            <>
              <SearchFieldWide>
                <Field
                  component={TextInput}
                  name="search"
                  placeholder="Enter ivr, description or amount"
                />
                <PrimaryButton
                  disabled={!values.search || submitting}
                  onClick={() => handleFormSubmit(values)}
                  type="button"
                >
                  Search
                </PrimaryButton>
                &nbsp;&nbsp;
                {values.search ? (
                  <SecondaryButton
                    onClick={() => handleFilterReset(form.reset)}
                    type="reset"
                  >
                    Clear
                  </SecondaryButton>
                ) : null}
              </SearchFieldWide>
              <FilterSort>
                <FilterSortItem>
                  <label htmlFor="status">Transaction type:</label>
                  <div>
                    <Field
                      name="type"
                      render={({ input }) => (
                        <Dropdown
                          input={input}
                          options={TYPE_OPTIONS}
                          placeholder="Choose transaction type"
                        />
                      )}
                    />
                  </div>
                </FilterSortItem>
                <FilterSortItem>
                  <PrimaryButton
                    disabled={!values.type || submitting}
                    onClick={() => handleFormSubmit(values)}
                  >
                    Filter
                  </PrimaryButton>
                  {isFiltersExist && (
                    <FilterSortItemInner>
                      <SecondaryButton
                        onClick={() => handleFilterReset(form.reset)}
                        type="reset"
                      >
                        Clear
                      </SecondaryButton>
                    </FilterSortItemInner>
                  )}
                </FilterSortItem>
              </FilterSort>
            </>
          );
        }}
      />

      <Sortable field={[]}>
        {() => (
          <Query<GetTransactions, GetTransactionsVariables>
            query={FETCH_TRANSACTIONS}
            variables={{
              pagination: { page, perPage },
              filter,
              sort: [{ field: "created", order: -1 }],
            }}
          >
            {({ data, loading, error }) => {
              if (error) {
                return <SystemError error={error} />;
              } else if (loading) {
                return <Spinner />;
              }
              return (
                <TableWrapper>
                  {/* {loading && <TableOverlay />} */}

                  <Pagebar
                    changePage={setPage}
                    curPage={page}
                    showNPages={3}
                    total={Math.ceil(
                      (data?.admin?.allTransactions?.total || 1) / perPage
                    )}
                  />

                  <Table style={{ tableLayout: "fixed" }}>
                    <Thead>
                      <ListHead />
                    </Thead>
                    <Tbody>
                      {(data?.admin?.allTransactions?.transactions ?? []).map(
                        (item) => {
                          return (
                            <ListItem
                              key={item.id}
                              amount={item.amount}
                              description={item.description}
                              ivr={item.ivr}
                              transaction_date={item.created}
                              transaction_id={item.id}
                              type={item.type}
                            />
                          );
                        }
                      )}
                    </Tbody>
                  </Table>

                  <Pagebar
                    changePage={setPage}
                    curPage={page}
                    showNPages={3}
                    total={Math.ceil(
                      (data?.admin?.allTransactions?.total || 1) / perPage
                    )}
                  />
                </TableWrapper>
              );
            }}
          </Query>
        )}
      </Sortable>
    </>
  );
};

export default List;
