import {
  AddPaymentTransactionMatch,
  AddPaymentTransactionMatchVariables,
} from "../payments/graphql/mutations/__generated__/AddPaymentTransactionMatch";
import { Field, Form } from "react-final-form";
import { Invoice_Api_PaymentStatusEnum } from "../../graphql-schema-types.generated";
import { OnChange } from "react-final-form-listeners";
import {
  common_api_generic_query_QueryFilterOperatorEnum as OperatorEnum,
  invoice_api_InvoiceStatusEnum,
  invoice_api_TransactionTypeEnum,
} from "../../globalTypes";
import {
  PaymentsGetAllInvoices,
  PaymentsGetAllInvoicesVariables,
} from "../payments/graphql/queries/__generated__/PaymentsGetAllInvoices";
import {
  PaymentsGetInvoiceQuery,
  PaymentsGetInvoiceQueryVariables,
} from "../payments/graphql/queries/PaymentsGetInvoice.generated";
import { PrimaryButton } from "../../ui/atoms/Button";
import { Td, Tr } from "../../ui/atoms/Table";
import { formatDate } from "../../util/date";
import { useMutation, useQuery } from "react-apollo";
import ADD_TRANSACTION_MATCH from "../payments/graphql/mutations/AddPaymentTransactionMatch.graphql";
import Dropdown from "../../ui/molecules/Dropdown";
import FETCH_INVOICES from "../payments/graphql/queries/PaymentsGetAllInvoices.graphql";
import FindAddress from "./FindAddress";
import GET_INVOICE from "../payments/graphql/queries/PaymentsGetInvoice.graphql";
import React, { FC, useEffect, useState } from "react";
import Spinner from "../../ui/atoms/Spinner";
import styled from "@emotion/styled";

type Props = {
  amount: number;
  description: string;
  transaction_id: string;
  ivr: string;
  transaction_date: string;
  type: string;
};

type FormValues = {
  location: string;
  invoice_id: string | undefined;
  property_id: string;
  payment_id: string;
};

type oneOption = {
  label: string;
  value: string;
};

const ListItem: FC<Props> = ({
  amount,
  description,
  transaction_id,
  ivr,
  transaction_date,
  type,
}) => {
  const [paymentOptions, setPaymentOptions] = useState<oneOption[]>([]);
  const [buttonMerge, setButtonMerge] = useState<boolean>(false);
  const [createPayment, { loading: createPaymentLoading }] = useMutation<
    AddPaymentTransactionMatch,
    AddPaymentTransactionMatchVariables
  >(ADD_TRANSACTION_MATCH);

  const [propertyInput, setPropertyInput] = useState<FormValues>({
    location: "",
    property_id: "",
    invoice_id: "",
    payment_id: "",
  });
  const admin_id = window.localStorage.getItem("adminId") ?? "";

  const {
    data: dataInvoices,
    loading: invoicesLoading,
    networkStatus: invoicesStatus,
  } = useQuery<PaymentsGetAllInvoices, PaymentsGetAllInvoicesVariables>(
    FETCH_INVOICES,
    {
      variables: {
        pagination: { page: 1, perPage: 400 },
        filter: [
          {
            fields: ["status"],
            not: true,
            operator: OperatorEnum.EQUALS,
            value: [invoice_api_InvoiceStatusEnum.INVOICE_FULFILLED],
          },
          {
            fields: ["correlation_target"],
            operator: OperatorEnum.CONTAINS_ANY,
            value: ["tenant-services", "standing-order", "deposit-replacement"],
          },
          {
            fields: ["property"],
            operator: OperatorEnum.EQUALS,
            value: [propertyInput.property_id],
          },
        ],
      },
      skip: propertyInput.location === "" || propertyInput.property_id === "",
    }
  );

  useEffect(() => {
    setPaymentOptions([]);
    const standingOrderInvoices =
      dataInvoices?.admin?.allInvoices?.invoices?.filter(
        (oneInvoice) => oneInvoice.correlation_target === "standing-order"
      ) ?? [];
    const firstMonthInvoices =
      dataInvoices?.admin?.allInvoices?.invoices?.filter(
        (oneInvoice) => oneInvoice.correlation_target === "tenant-services"
      ) ?? [];
    const depositReplacementInvoices =
      dataInvoices?.admin?.allInvoices?.invoices?.filter(
        (oneInvoice) => oneInvoice.correlation_target === "deposit-replacement"
      ) ?? [];

    const selectedStandingOrder =
      standingOrderInvoices.length > 0
        ? standingOrderInvoices.reduce((acc, invoice) =>
            Date.parse(invoice.updated_at) < Date.parse(acc.updated_at)
              ? invoice
              : acc
          )
        : { invoice_id: "", correlation_target: "", amount: 0 };
    const selectedFirstMonth =
      firstMonthInvoices.length > 0
        ? firstMonthInvoices.reduce((acc, invoice) =>
            Date.parse(invoice.updated_at) < Date.parse(acc.updated_at)
              ? invoice
              : acc
          )
        : { invoice_id: "", correlation_target: "", amount: 0 };

    const selectedDepositReplacement =
      depositReplacementInvoices.length > 0
        ? depositReplacementInvoices.reduce((acc, invoice) =>
            Date.parse(invoice.updated_at) < Date.parse(acc.updated_at)
              ? invoice
              : acc
          )
        : { invoice_id: "", correlation_target: "", amount: 0 };

    if (standingOrderInvoices.length > 0) {
      setPaymentOptions((state) =>
        !state
          .map(({ value }) => value)
          .includes(selectedStandingOrder.invoice_id)
          ? [
              ...state,
              {
                label: `${selectedStandingOrder.correlation_target} ${selectedStandingOrder.amount}`,
                value: selectedStandingOrder.invoice_id,
              },
            ]
          : state
      );
    }
    if (firstMonthInvoices.length > 0) {
      setPaymentOptions((state) =>
        !state.map(({ value }) => value).includes(selectedFirstMonth.invoice_id)
          ? [
              ...state,
              {
                label: `${selectedFirstMonth.correlation_target} ${selectedFirstMonth.amount}`,
                value: selectedFirstMonth.invoice_id,
              },
            ]
          : state
      );
    }
    if (depositReplacementInvoices.length > 0) {
      setPaymentOptions((state) =>
        !state
          .map(({ value }) => value)
          .includes(selectedDepositReplacement.invoice_id)
          ? [
              ...state,
              {
                label: `${selectedDepositReplacement.correlation_target} ${selectedDepositReplacement.amount}`,
                value: selectedDepositReplacement.invoice_id,
              },
            ]
          : state
      );
    }
  }, [dataInvoices]);

  const {
    data: dataInvoicePayments,
  } = useQuery<PaymentsGetInvoiceQuery, PaymentsGetInvoiceQueryVariables>(
    GET_INVOICE,
    {
      variables: {
        invoice_id: propertyInput.invoice_id,
      },
      skip: propertyInput.invoice_id === "",
    }
  );

  useEffect(() => {
    if (dataInvoicePayments !== undefined) {
      const filteredPayments = dataInvoicePayments.admin?.Invoice?.payments?.filter(
        ({ status }) => status === Invoice_Api_PaymentStatusEnum.PaymentPending
      );
      const selectedPayment = filteredPayments?.reduce((acc, payment) =>
        Date.parse(payment.scheduled_at) < Date.parse(acc.scheduled_at)
          ? payment
          : acc
      );
      setPropertyInput((state) => ({
        ...state,
        payment_id: selectedPayment?.id ?? "",
      }));
    }
  }, [dataInvoicePayments]);

  const handleUpdateStatus = async (transaction_id: string) => {
    await createPayment({
      variables: {
        payment_id: propertyInput.payment_id,
        transaction_id,
        comment: "custom matching",
        admin_id,
      },
      refetchQueries: ["GetTransactions"],
    });
  };

  return (
    <>
      <Tr>
        <Td colSpan={3}>
          <span>{description}</span>
        </Td>
        <Td colSpan={2}>
          <span>£{amount}</span>
        </Td>
        <Td colSpan={2}>
          <span>{ivr}</span>
        </Td>
        <Td colSpan={2}>
          <span>{formatDate(transaction_date)}</span>
        </Td>
        <Td colSpan={1}>
          <span>
            {type === invoice_api_TransactionTypeEnum.TRANSACTION_TYPE_DEBIT
              ? "Debit"
              : "Credit"}
          </span>
        </Td>
        <Td colSpan={2}>
          <ButtonMerge
            onClick={() => setButtonMerge((state) => !state)}
            type="button"
          >
            Merge
          </ButtonMerge>
        </Td>
      </Tr>
      {buttonMerge && (
        <Form<FormValues>
          onSubmit={() => {
            if (propertyInput.invoice_id && transaction_id) {
              handleUpdateStatus(transaction_id);
            }
          }}
        >
          {() => {
            return (
              <Tr>
                <Td colSpan={5}>
                  <FindAddress setPropertyInput={setPropertyInput} />
                </Td>
                <Td colSpan={2}></Td>
                <Td colSpan={3}>
                  <div>
                    {invoicesLoading ? (
                      <Loader>
                        <Spinner />
                      </Loader>
                    ) : paymentOptions.length > 0 ? (
                      <Field
                        name="invoice_id"
                        render={({ input }) => (
                          <Dropdown
                            input={input}
                            options={paymentOptions}
                            placeholder="Payment date"
                          />
                        )}
                      />
                    ) : Boolean(invoicesStatus) ? (
                      <span>no appropriate invoices</span>
                    ) : null}

                    <OnChange name="invoice_id">
                      {(value: any) => {
                        if (value) {
                          setPropertyInput((state) => ({
                            ...state,
                            invoice_id: value.value,
                          }));
                        }
                      }}
                    </OnChange>
                  </div>
                </Td>

                <Td colSpan={2}>
                  <PrimaryButton
                    disabled={!(propertyInput.invoice_id && transaction_id)}
                    onClick={() => {
                      handleUpdateStatus(transaction_id);
                      setButtonMerge(false);
                    }}
                    type="button"
                  >
                    {createPaymentLoading ? "loading" : "Complete merge"}
                  </PrimaryButton>
                </Td>
              </Tr>
            );
          }}
        </Form>
      )}
    </>
  );
};

const ButtonMerge = styled(PrimaryButton)`
  width: 100%;
`;

const Loader = styled.div`
  width: 16px;
  height: 16px;
  display: flex;
  align-items: center;
  justify-content: center;
`;

export default ListItem;
