import { FieldMetaState, FieldRenderProps } from "react-final-form";
import { isEmpty } from "../../../util/validations";
import Downshift from "downshift";
import React, {
  ReactNode,
  useCallback,
  useMemo,
  useRef,
  useState,
} from "react";
import cn from "classnames";
import css from "./index.module.css";

type Props<T> = Pick<FieldRenderProps<T, HTMLElement>, "input"> & {
  allowEmpty?: boolean;
  className?: string;
  disabled?: boolean;
  "data-testid"?: string;
  emptyLabel?: string;
  initialInputValue?: string;
  itemToString?: (item: T) => string;
  onInputValueChange?: (inputValue: string | null) => void;
  options: readonly T[];
  label?: string;
  dropdownBottom?: boolean;
  readonly?: boolean;
  withArrow?: boolean;
  getKey?: (item: T) => string | null | undefined;
  meta?: FieldMetaState<any>;
  hideError?: boolean;
  icon?: ReactNode;
  withFilterByValue?: boolean;
};

export const Select2 = <T extends { toString(): string } | null>({
  allowEmpty,
  emptyLabel = "",
  options,
  input,
  "data-testid": dataTestid,
  initialInputValue,
  itemToString = (a) => a?.toString() ?? emptyLabel,
  onInputValueChange,
  label,
  disabled = false,
  dropdownBottom = false,
  className,
  readonly = true,
  withArrow = true,
  meta,
  hideError = false,
  icon,
  getKey: rawGetKey,
  withFilterByValue,
}: Props<T>) => {
  const inputElement = useRef<HTMLInputElement>(null);
  const [dropdownPosition, setDropdownPosition] = useState<"top" | "bottom">(
    "bottom"
  );
  const getLabel = useCallback(
    (item: T | string | null) =>
      item != null && item !== "" ? itemToString(item as T) : emptyLabel,
    [itemToString, emptyLabel]
  );

  const keyFn = useMemo(() => rawGetKey ?? getLabel, [rawGetKey, getLabel]);

  const getKey = useCallback(
    (el: T | null) => (el ? keyFn(el) : getLabel(el)),
    [keyFn, getLabel]
  );

  return (
    <>
      <Downshift<T>
        initialInputValue={initialInputValue ?? getLabel(input.value)}
        itemToString={getLabel}
        onChange={(selection) => {
          input.onChange(
            (selection as any) === emptyLabel ||
              getLabel(selection) === emptyLabel
              ? null
              : selection
          );
        }}
        onInputValueChange={(v, s) => {
          if (onInputValueChange && v) {
            if (v.length === 0) {
              onInputValueChange(null);
            } else if (v.length > 1) {
              onInputValueChange(v);
            }
          }
        }}
        selectedItem={input.value}
      >
        {({
          getInputProps,
          getItemProps,
          getMenuProps,
          isOpen,
          inputValue,
          getRootProps,
          selectedItem,
          openMenu,
          getToggleButtonProps,
        }) => {
          const items: ReadonlyArray<T | null> = allowEmpty
            ? ([null] as Array<T | null>).concat(options)
            : options;

          const handleOpen = () => {
            if (
              dropdownBottom ||
              typeof window === "undefined" ||
              !inputElement.current
            ) {
              openMenu();
              return;
            }

            const clientHeight = document.documentElement.clientHeight;
            const offsetTop = inputElement.current.getBoundingClientRect().top;

            setDropdownPosition(
              Math.min(items.length * 40 + 20, 280) > clientHeight - offsetTop
                ? "top"
                : "bottom"
            );

            openMenu();
          };

          return (
            <div
              {...getRootProps(undefined, { suppressRefError: true })}
              className={cn(css.dropdownContainer, className, {
                [css.disabled]: disabled,
              })}
              data-testid={dataTestid}
            >
              <input
                {...getInputProps({ defaultValue: "" })}
                ref={inputElement}
                className={cn(css.input, {
                  [css.error]: meta?.touched && !meta.valid,
                  [css.iconBefore]: Boolean(icon),
                })}
                onBlur={(e) => {
                  input.onBlur(e);
                }}
                onClick={() => !disabled && handleOpen()}
                readOnly={readonly}
              />

              {icon && <div className={css.icon}>{icon}</div>}

              {label && (
                <label
                  className={cn(css.label, {
                    [css.shrinked]:
                      !isEmpty(input.value) || !isEmpty(inputValue),
                    [css.iconBeforeLabel]: Boolean(icon),
                  })}
                >
                  {label}
                </label>
              )}

              {isOpen && items.length > 0 && (
                <ul
                  {...getMenuProps()}
                  className={cn(css.dropdownList, {
                    [css.isOnTop]: dropdownPosition === "top",
                  })}
                >
                  {items
                    .filter(
                      (item) =>
                        !withFilterByValue ||
                        !inputValue ||
                        getLabel(item)
                          .toLocaleLowerCase()
                          .includes(inputValue.toLocaleLowerCase())
                    )
                    .map((item, index) => (
                      <li
                        key={getLabel(item) + index}
                        className={cn(css.dropdownItem, {
                          [css.selected]:
                            inputValue === getLabel(item) ||
                            selectedItem === item,
                        })}
                        {...getItemProps({
                          key: `${getKey(item)}${index}`,
                          index,
                          item: item ? item : ((emptyLabel as any) as T),
                        })}
                      >
                        {getLabel(item)}
                      </li>
                    ))}
                </ul>
              )}

              {withArrow && (
                <button
                  type="button"
                  {...getToggleButtonProps()}
                  className={cn(css.dropdownButton, {
                    [css.opened]: isOpen,
                  })}
                  disabled={disabled}
                ></button>
              )}
            </div>
          );
        }}
      </Downshift>
      {!hideError && meta?.touched && meta.error != null && (
        <div className={css.errorText}>{meta.error}</div>
      )}
    </>
  );
};
